﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ADQInfo project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;

namespace ADQInfo.Utility
{
    public static class AdqCommon
    {
        private static readonly string _formatStr = "{0:0.######}";

        public static uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_ERROR | AdqErrors.ADQ_ERROR_NOT_OPEN;

        public static string GetModuleTypeName(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeNames, moduleType);
        }

        public static string GetPhysicalUnitName(uint physicalUnit)
        {
            return GetDictionaryValue(_physicalUnitNames, physicalUnit);
        }

        public static string GetPhysicalUnitSymbol(uint physicalUnit)
        {
            return GetDictionaryValue(_physicalUnitSymbols, physicalUnit);
        }

        public static string GetClockType(uint clockType)
        {
            return GetDictionaryValue(_clockTypeNames, clockType);
        }

        public static string GetCounterTypeName(uint counterType)
        {
            return GetDictionaryValue(_counterTypeNames, counterType);
        }

        public static string GetCounterClockSourceName(uint counterClockSource)
        {
            return GetDictionaryValue(_counterClockSourceNames, counterClockSource);
        }

        public static string GetBusTypeName(uint busType)
        {
            return GetDictionaryValue(_busTypeNames, busType);
        }

        public static string GetTTLVoltageLevelText(uint voltageLevel)
        {
            return GetDictionaryValue(_ttlVoltageLevels, voltageLevel);
        }

        public static string GetDioDirectionText(uint dioDirection)
        {
            return GetDictionaryValue(_dioDirections, dioDirection);
        }

        public static string GetCalibrationText(uint calibration)
        {
            return GetDictionaryValue(_calibrations, calibration);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            return found ? stringValue : string.Format("0x{0:X8}", key);
        }

        private static readonly Dictionary<uint, string> _moduleTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        };

        private static readonly Dictionary<uint, string> _physicalUnitNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "Volt"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "Ampere"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ohm"}
        };

        private static readonly Dictionary<uint, string> _physicalUnitSymbols = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
        };

        private static readonly Dictionary<uint, string> _clockTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CLOCK_TYPE_HARDWARE, "Hardware"},
            {AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE, "Software"},
            {AdqConstants.ADQ_CLOCK_TYPE_NONE, "None"}
        };

        private static readonly Dictionary<uint, string> _counterTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CNTR_TYPE_8254, "8254"},
            {AdqConstants.ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT, "Count down 32 Bit"}
        };

        private static readonly Dictionary<uint, string> _counterClockSourceNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_DEFAULT, "Default"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, "External"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ, "Internal 10 MHz"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ, "Internal 66 MHz"}
        };

        private static readonly Dictionary<uint, string> _busTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_BUS_TYPE_PCI, "PCI"},
            {AdqConstants.ADQ_BUS_TYPE_USB, "USB"}
        };

        private static readonly Dictionary<uint, string> _ttlVoltageLevels = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS, string.Format("{0:0.0} V", 5.0)},
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS, string.Format("{0:0.0} V", 3.3)}
        };

        private static readonly Dictionary<uint, string> _dioDirections = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_DIO_INPUT, "Input"},
            {AdqConstants.ADQ_DIO_OUTPUT, "Output"}
        };

        private static readonly Dictionary<uint, string> _calibrations = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CALIBRATION_FACTORY, "Factory"},
            {AdqConstants.ADQ_CALIBRATION_USER_1, "User"}
        };
       
        public static string FormatDoubleValue(double value, string unitText)
        {
            string valueStr;
            string unitStr = string.Empty;
            double tmpVal = Math.Abs(value);
            if (tmpVal == 0)
            {
                valueStr = "0";
            }
            else if (tmpVal < 0.000000001)
            {
                valueStr = string.Format(_formatStr, value * 1000000000000);
                unitStr = "p";
            }
            else if (tmpVal < 0.000001)
            {
                valueStr = string.Format(_formatStr, value * 1000000000);
                unitStr = "n";
            }
            else if (tmpVal < 0.001)
            {
                valueStr = string.Format(_formatStr, value * 1000000);
                unitStr = "µ";
            }
            else if (tmpVal < 1)
            {
                valueStr = string.Format(_formatStr, value * 1000);
                unitStr = "m";
            }
            else if (tmpVal >= 1000000000)
            {
                valueStr = string.Format(_formatStr, value / 1000000000);
                unitStr = "G";
            }
            else if (tmpVal >= 1000000)
            {
                valueStr = string.Format(_formatStr, value / 1000000);
                unitStr = "M";
            }
            else if (tmpVal >= 1000)
            {
                valueStr = string.Format(_formatStr, value / 1000);
                unitStr = "k";
            }
            else
            {
                valueStr = string.Format(_formatStr, value);
            }
            return valueStr + " " + unitStr + unitText;
        }

        public static string FormatBitsPerSecond(uint value)
        {
            string valueStr;
            string unitStr = string.Empty;
            if (value >= 1000000000)
            {
                valueStr = string.Format("{0}", value / 1000000000);
                unitStr = "G";
            }
            else if (value >= 1000000)
            {
                valueStr = string.Format("{0}", value / 1000000);
                unitStr = "M";
            }
            else if (value >= 1000)
            {
                valueStr = string.Format("{0}", value / 1000);
                unitStr = "k";
            }
            else
            {
                valueStr = value.ToString();
            }
            return valueStr + " " + unitStr + "bit/s";
        }

        public static string FormatVersion(uint version)
        {
            return string.Format("{0}.{1}.{2}.{3}",
                        (version >> 24) & 0xFF,
                        (version >> 16) & 0xFF,
                        (version >> 8) & 0xFF,
                        (version) & 0xFF);
        }

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }
            return boardName.Substring(posADQ, posEnd - posADQ);
        }
    }
}
