﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadContinuous project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadContinuous.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace AIReadContinuous.ALLDAQ
{
    public class ModuleLocator
    {
        private uint _adqOpenError = AdqCommon.NotOpen;
        private readonly ICollection<AnalogInputModule> _aiModules;

        public event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;

        public ModuleLocator(ICollection<AnalogInputModule> aiModules, 
            EventHandler<InfoTextChangedEventArgs> moduleInfoTextChanged)
        {
            _aiModules = aiModules;
            ModuleInfoTextChanged += moduleInfoTextChanged;
        }

        public void FindModules()
        {
            _adqOpenError = OpenAdqDriver();
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                uint numberOfBoards = GetNumberOfBoards();
                for (uint boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
                {
                    uint adqError = AdqDriver.adqGetNumberOfModules(boardIdx, out uint numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                        continue;
                    }

                    adqError = GetModuleTypes(boardIdx, numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
        }

        private uint GetModuleTypes(uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;

            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleType == AdqConstants.ADQ_MODULE_TYPE_AI)
                {
                    adqError = GetAiModuleInfo(boardIndex, moduleIdx);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
            return adqError;
        }

        private uint GetAiModuleInfo(uint boardIndex, uint moduleIndex)
        {
            uint adqError;
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqAiModuleInfo = new AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqAiModuleInfo.Size);
            Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, false);

            adqError = AdqDriver.adqGetModuleInfo(boardIndex,
                                                  moduleIndex,
                                                  ref adqGeneralModuleInfo,
                                                  adqAiModuleInfo.Size,
                                                  moduleInfo,
                                                  out uint infoSize);
            adqAiModuleInfo = (AdqDriver.ADQAIModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQAIModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqAiModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize));

            string boardName = GetBoardName(boardIndex);
            if (adqAiModuleInfo.RangeInfoPerChannel)
            {
                UpdateInfoText(string.Format("{0}, Board {1}, Module {2} has individual ranges."
                    + Environment.NewLine
                    + "Please update to a newer version of this program.",
                    boardName,
                    boardIndex,
                    moduleIndex));
                return adqError;
            }

            if (IsClockControlledModule(adqGeneralModuleInfo.ClockType))
            {
                Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, string.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex));
                var aiModule = new AnalogInputModule(boardName, boardIndex, moduleIndex, ref adqGeneralModuleInfo, ref adqAiModuleInfo);
                    _aiModules.Add(aiModule);

            }
            return adqError;
        }

        private bool IsClockControlledModule(uint clockType)
        {
            return clockType == AdqConstants.ADQ_CLOCK_TYPE_HARDWARE || clockType == AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE;
        }

        private string GetBoardName(uint boardIndex)
        {
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetBoardInfo", adqError);
                return "";
            }
            return boardInfo.BoardName;
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        public void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            UpdateInfoText(string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError));
        }

        private void UpdateInfoText(string text)
        {
            InfoTextChangedEventArgs args = new InfoTextChangedEventArgs
            {
                InfoText = text
            };
            RaiseOnModuleInfoTextChanged(args);
        }

        protected virtual void RaiseOnModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            ModuleInfoTextChanged?.Invoke(this, e);
        }
    }
}
