﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadInterval project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadInterval.Utility;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;

namespace AIReadInterval.ALLDAQ
{
    public class CsvFileWriter
    {
        private const string _fmtTime = "HH:mm:ss.fffffff";

        private bool _saveAllValues;
        private bool _saveCalculatedValues;
        private bool _saveFftValues;
        private bool _saveFftCoherentValues;
        private string _fmtStringDigitalAndPhysical;
        private string _fmtStringPhysValues;
        private string _fmtStringFftValues;
        private AnalogInputModule _aiModule;
        private SignalUtility _signalUtility;
        private string _folderName;

        public event EventHandler<InfoTextChangedEventArgs> PropertyChanged;

        public void UpdateCsvFileWriter(CsvProperties csvProperties)
        {
            _saveAllValues = csvProperties.SaveAllValues;
            _saveCalculatedValues = csvProperties.SaveCalculatedValues;
            _saveFftValues = csvProperties.SaveFftValues;
            _saveFftCoherentValues = csvProperties.SaveFftCoherentValues;
            _fmtStringDigitalAndPhysical = csvProperties.FormatStringDigitalAndPhysical;
            _fmtStringPhysValues = csvProperties.FormatStringPhysValues;
            _fmtStringFftValues = csvProperties.FormatStringFftValues;
            _signalUtility = csvProperties.SignalUtility;

            if (_aiModule != csvProperties.AiModule || _folderName != csvProperties.FolderName)
            {
                _aiModule = csvProperties.AiModule;
                _folderName = csvProperties.FolderName;

                CreateCsvFolder();
            }

            UpdateFileNames();
        }

        private string _csvFileNameAllValues;
        public string CsvFileNameAllValues
        {
            get
            {
                return _csvFileNameAllValues;
            }
            private set
            {
                if (_csvFileNameAllValues != value)
                {
                    _csvFileNameAllValues = value;
                    RaisePropertyChanged(nameof(CsvFileNameAllValues));
                }
            }
        }

        private string _csvFileNameCalculatedValues;
        public string CsvFileNameCalculatedValues
        {
            get
            {
                return _csvFileNameCalculatedValues;
            }
            private set
            {
                if (_csvFileNameCalculatedValues != value)
                {
                    _csvFileNameCalculatedValues = value;
                    RaisePropertyChanged(nameof(CsvFileNameCalculatedValues));
                }
            }
        }

        private string _csvFileNameFftValues;
        public string CsvFileNameFftValues
        {
            get
            {
                return _csvFileNameFftValues;
            }
            private set
            {
                if (_csvFileNameFftValues != value)
                {
                    _csvFileNameFftValues = value;
                    RaisePropertyChanged(nameof(CsvFileNameFftValues));
                }
            }
        }

        private string _csvFileNameFftCoherentValues;
        public string CsvFileNameFftCoherentValues
        {
            get
            {
                return _csvFileNameFftCoherentValues;
            }
            private set
            {
                if (_csvFileNameFftCoherentValues != value)
                {
                    _csvFileNameFftCoherentValues = value;
                    RaisePropertyChanged(nameof(CsvFileNameFftCoherentValues));
                }
            }
        }

        public string NumberDecimalSeparator
        {
            get
            {
                return Thread.CurrentThread.CurrentCulture.NumberFormat.NumberDecimalSeparator;
            }
        }


        private void CreateCsvFolder()
        {
            if (Directory.Exists(_folderName))
            {
                return;
            }

            Directory.CreateDirectory(_folderName);
        }

        private void UpdateFileNames()
        {
            string commonFileName = string.Empty;
            if (_saveAllValues || _saveCalculatedValues || _saveFftValues || _saveFftCoherentValues)
            {
                commonFileName = GetBaseFileName();
            }

            CsvFileNameAllValues = _saveAllValues ? commonFileName + " all values.csv" : string.Empty;
            CsvFileNameCalculatedValues = _saveCalculatedValues ? commonFileName + " calculated.csv" : string.Empty;
            CsvFileNameFftValues = _saveFftValues ? commonFileName + " FFT values.csv" : string.Empty;
            CsvFileNameFftCoherentValues = _saveFftCoherentValues ? commonFileName + " FFT coherent values.csv" : string.Empty;
        }

        private string GetBaseFileName()
        {
            return string.Format("{0}{1} Board {2} {3}",
                EnsureTrailingPathSeperator(_folderName),
                _aiModule.ModelName,
                _aiModule.BoardIndex,
                DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss"));
        }

        private string EnsureTrailingPathSeperator(string folderName)
        {
            if (string.IsNullOrEmpty(folderName))
            {
                throw new ApplicationException("Please enter a folder to save the CSV files.");
            }

            char lastChar = folderName[folderName.Length - 1];
            if (lastChar == Path.DirectorySeparatorChar)
            {
                return folderName;
            }
            return folderName + Path.DirectorySeparatorChar;
        }

        public void WriteCsvFileHeader()
        {
            if (_saveAllValues)
            {
                WriteCsvHeaderAllValues();
            }

            if (_saveCalculatedValues)
            {
                WriteCsvHeaderCalculatedValues();
            }

            if (_saveFftValues)
            {
                WriteCsvHeaderFftValues(CsvFileNameFftValues);
            }

            if (_saveFftCoherentValues)
            {
                WriteCsvHeaderFftValues(CsvFileNameFftCoherentValues);
            }
        }

        private void WriteCsvHeaderAllValues()
        {
            var sb = new StringBuilder();
            sb.Append("Frame;Time");
            foreach (ChannelSetting chanSetting in _aiModule.ChannelSettings)
            {
                sb.AppendFormat(";Ch{0} digital;Ch{0} [{1}]", chanSetting.ChannelIndex, chanSetting.Range.UnitSymbol);
            }

            using (StreamWriter sw = File.AppendText(CsvFileNameAllValues))
            {
                sw.WriteLine(sb.ToString());
            }
        }

        private void WriteCsvHeaderCalculatedValues()
        {
            var sb = new StringBuilder();
            sb.Append("Time");
            foreach (ChannelSetting chanSetting in _aiModule.ChannelSettings)
            {
                sb.AppendFormat(";Ch{0} Avg [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
                sb.AppendFormat(";Ch{0} RMS [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
                sb.AppendFormat(";Ch{0} Min [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
                sb.AppendFormat(";Ch{0} Max [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
                sb.AppendFormat(";Ch{0} P2P [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
                sb.AppendFormat(";Ch{0} Freq [Hz]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} FreqProp [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THD [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THDr+N [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THDf+N [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} RMS spect [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);

                sb.AppendFormat(";Ch{0} Freq coh [Hz]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} FreqProp coh [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THD coh [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THDr+N coh [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} THDf+N coh [%]", chanSetting.ChannelIndex);
                sb.AppendFormat(";Ch{0} RMS spect coh [{1}]", chanSetting.ChannelIndex, _aiModule.DefaultRange.UnitSymbol);
            }

            using (StreamWriter sw = File.AppendText(CsvFileNameCalculatedValues))
            {
                sw.WriteLine(sb.ToString());
            }
        }

        private void WriteCsvHeaderFftValues(string csvFileName)
        {
            if (_signalUtility == null)
            {
                return;
            }

            var sb = new StringBuilder();
            sb.Append("Frequency step");

            for (int idx = 0; idx < _signalUtility.FrameLength; idx++)
            {
                sb.AppendFormat(";Ch{0} Frequency", _signalUtility.GetChannelIndex(idx));
            }

            using (StreamWriter sw = File.AppendText(csvFileName))
            {
                sw.WriteLine(sb.ToString());
            }
        }

        public int WriteCsvFileValues()
        {
            if (_saveAllValues)
            {
                WriteCsvFileAllValues();
            }

            if (_saveCalculatedValues)
            {
                WriteCsvFileCalculatedValues();
            }

            if (_saveFftValues)
            {
                WriteCsvFileFftValues(CsvFileNameFftValues, _signalUtility.FrequencySpectrums, _signalUtility.SpectrumSize, _signalUtility.FrequencyStep);
            }

            if (_saveFftCoherentValues)
            {
                WriteCsvFileFftValues(_csvFileNameFftCoherentValues, _signalUtility.FrequencySpectrumsCoherent, _signalUtility.SpectrumSize, _signalUtility.FrequencyStepCoherent);
            }
            return 0;
        }

        private void WriteCsvFileAllValues()
        {
            if (_aiModule.NumberOfFramesRead == 0)
            {
                return;
            }

            uint currentFrameOffset;
            DateTime dt = _aiModule.DateTimeFirstFrame;
            uint frameLength = (uint)_aiModule.ChannelSettings.Count;
            var sbFrame = new StringBuilder();
            long ticks = (long)Math.Truncate(_aiModule.SampleTime * 10000000.0);
            currentFrameOffset = 0;
            for (uint frameIdx = 0; frameIdx < _aiModule.NumberOfFramesRead; frameIdx++)
            {
                uint digitalValue;
                sbFrame.Append(string.Format("{0};", frameIdx));
                sbFrame.Append(dt.ToString(_fmtTime));

                for (int idx = 0; idx < frameLength; idx++)
                {
                    digitalValue = _aiModule.Frames[currentFrameOffset + idx];
                    double physical = _aiModule.PhysicalFrames[currentFrameOffset + idx];
                    sbFrame.Append(string.Format(_fmtStringDigitalAndPhysical,
                        digitalValue,
                        physical));
                }
                sbFrame.AppendLine();
                dt = dt.AddTicks(ticks);
                currentFrameOffset += frameLength;
            }

            using (StreamWriter sw = File.AppendText(CsvFileNameAllValues))
            {
                sw.WriteLine(sbFrame.ToString());
            }
        }

        private void WriteCsvFileCalculatedValues()
        {
            if (_aiModule.NumberOfFramesRead == 0)
            {
                return;
            }

            uint frameLength = (uint)_aiModule.ChannelSettings.Count;
            var sb = new StringBuilder();
            sb.Append(_aiModule.DateTimeFirstFrame.ToString(_fmtTime));
            for (int idx = 0; idx < frameLength; idx++)
            {
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.AverageValues[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.RmsValues[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.MinValues[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.MaxValues[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.P2PValues[idx]));
                sb.Append(";");

                sb.Append(FormatPhysicalValue(_signalUtility.MainFrequencies[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.FrequProportions[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdValues[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdrPlusNValues[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdfPlusNValues[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.RmsValuesSpectrum[idx]));
                sb.Append(";");

                sb.Append(FormatPhysicalValue(_signalUtility.MainFrequenciesCoherent[idx]));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.FrequProportionsCoherent[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdValuesCoherent[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdrPlusNValuesCoherent[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.ThdfPlusNValuesCoherent[idx] * 100));
                sb.Append(";");
                sb.Append(FormatPhysicalValue(_signalUtility.RmsValuesSpectrumCoherent[idx]));
            }

            using (StreamWriter sw = File.AppendText(CsvFileNameCalculatedValues))
            {
                sw.WriteLine(sb.ToString());
            }
        }

        private void WriteCsvFileFftValues(string csvFileName, IList<double[]> frequencySpectrums, uint spectrumSize, double frequencyStep)
        {
            if (_signalUtility == null)
            {
                return;
            }

            var sb = new StringBuilder();

            double fStep = 0.0;
            for (int idx = 0; idx < spectrumSize; idx++)
            {
                sb.Append(fStep);
                foreach (double[] spectrum in frequencySpectrums)
                {
                    sb.Append(";");
                    sb.Append(string.Format(_fmtStringFftValues, spectrum[idx]));
                }
                sb.AppendLine();
                fStep += frequencyStep;
            }

            using (StreamWriter sw = File.AppendText(csvFileName))
            {
                sw.WriteLine(sb.ToString());
            }
        }

        private string FormatPhysicalValue(double value)
        {
            return string.Format(_fmtStringPhysValues, value);
        }

        protected virtual void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                var args = new InfoTextChangedEventArgs
                {
                    InfoText = propertyName
                };

                PropertyChanged(this, args);
            }
        }
    }
}
