﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AISingle.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace AISingle.ALLDAQ
{
    public class ModuleLocator
    {
        private uint _adqOpenError = AdqCommon.NotOpen;
        private readonly ICollection<AnalogInputModule> _aiModules;
        private readonly uint _moduleType;
        public event EventHandler<InfoTextChangedEventArgs> OnInfoTextChanged;

        public ModuleLocator(ICollection<AnalogInputModule> aiModules, uint moduleType, 
            EventHandler<InfoTextChangedEventArgs> infoTextChangedHandler)
        {
            _aiModules = aiModules;
            _moduleType = moduleType;
            OnInfoTextChanged += infoTextChangedHandler;
        }

        private string _infoText = string.Empty;
        public string InfoText
        {
            get { return _infoText; }
            set
            {
                if (_infoText != value)
                {
                    _infoText = value;
                    InfoTextChangedEventArgs args = new InfoTextChangedEventArgs
                    {
                        InfoText = _infoText
                    };

                    RaiseModuleInfoTextChanged(args);
                }
            }
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        public void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        public void FindModules()
        {
            _adqOpenError = OpenAdqDriver();
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                uint numberOfBoards = GetNumberOfBoards();
                for (uint boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
                {
                    uint adqError = AdqDriver.adqGetNumberOfModules(boardIdx, out uint numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                        continue;
                    }

                    adqError = GetModuleTypes(boardIdx, numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
        }

        private uint GetModuleTypes(uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;

            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleType == _moduleType && moduleType == AdqConstants.ADQ_MODULE_TYPE_AI)
                {
                    adqError = GetAiModuleInfo(boardIndex, moduleIdx);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
            return adqError;
        }

        private uint GetAiModuleInfo(uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqAiModuleInfo = new AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqAiModuleInfo.Size);
            Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(boardIndex,
                                                  moduleIndex,
                                                  ref adqGeneralModuleInfo,
                                                  adqAiModuleInfo.Size,
                                                  moduleInfo,
                                                  out uint infoSize);
            adqAiModuleInfo = (AdqDriver.ADQAIModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQAIModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqAiModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize));

            string boardName = GetBoardName(boardIndex);
            if (adqAiModuleInfo.RangeInfoPerChannel)
            {
                InfoText = string.Format("Board {0}, {1}, Module {2} has individual ranges."
                    + Environment.NewLine
                    + "Please update to a newer version of this program.",
                    boardIndex,
                    boardName,
                    moduleIndex);
                return adqError;
            }

            var aiModule = new AnalogInputModule(boardName, boardIndex, moduleIndex,
                ref adqGeneralModuleInfo, ref adqAiModuleInfo);
            _aiModules.Add(aiModule);
            return adqError;
        }

        private string GetBoardName(uint boardIndex)
        {
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetBoardInfo", adqError);
                return "";
            }
            return boardInfo.BoardName;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        protected virtual void RaiseModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            OnInfoTextChanged?.Invoke(this, e);
        }
    }
}
