﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AOSingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using AOSingle.Utility;
using System.Collections.Generic;
using System.ComponentModel;

namespace AOSingle.Models
{
    public class AoModule : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public AoModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQAOModuleInfo aoModuleInfo)
        {
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            SetModuleName(boardName);
            AddRanges(ref aoModuleInfo);
        }

        private void AddRanges(ref AdqDriver.ADQAOModuleInfo aoModuleInfo)
        {
            for (uint rangeIdx = 0; rangeIdx < aoModuleInfo.NumberOfRanges; rangeIdx++)
            {
                AdqDriver.ADQAORange adqRange = aoModuleInfo.Ranges[rangeIdx];
                var aoRange = new AoRange(rangeIdx,
                    adqRange.DigitalMin,
                    adqRange.DigitalMax,
                    adqRange.PhysicalUnit,
                    adqRange.PhysicalMin,
                    adqRange.PhysicalMax);
                AoRanges.Add(aoRange);
            }

            if (aoModuleInfo.NumberOfRanges >= 1)
            {
                SelectedAoRange = AoRanges[0];
            }
        }

        public AoRange SelectedAoRange { get; set; }
        public List<AoRange> AoRanges { get; } = new List<AoRange>();
        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public string ModuleName { get; private set; }

        public bool ResetModule()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqResetModule", adqError);
                return false;
            }
            return true;
        }

        public bool WriteSingleValue(uint digitalValue)
        {
            uint adqError = AdqDriver.adqWriteSingle(BoardIndex, ModuleIndex, digitalValue);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqWriteSingle", adqError);
                return false;
            }
            return true;
        }

        private void SetErrorText(string functionName, uint adqError)
        {
            ModuleErrorText = string.Format("Board {0}, Module {1}: API-function {2} returned error number: 0x{3:X}",
                BoardIndex,
                ModuleIndex,
                functionName,
                adqError);
        }

        string _errorText = string.Empty;
        public string ModuleErrorText
        {
            get { return _errorText; }
            set
            {
                _errorText = value;
                if (value != string.Empty)
                {
                    RaisePropertyChanged(nameof(ModuleErrorText));
                }
            }
        }

        void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SetModuleName(string boardName)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex);
        }
    }
}