﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using ALLDAQ.Boards;
using CommonSettings.Utility;

namespace CommonSettings.ALLDAQ
{
    public class Board
    {
        public event EventHandler<InfoTextChangedEventArgs> OnInfoTextChanged;

        public Board(string boardName, uint boardIndex,
            ref AdqDriver.ADQBoardInfo boardInfo)
        {
            BoardIndex = boardIndex;
            BuildBoardName(boardName);
            FillTTLVoltageLevels(boardInfo.TTLVoltageLevel);
        }

        public uint BoardIndex { get; }
        public string BoardName { get; private set; }

        public List<KeyValuePair<uint, string>> VoltageLevels { get; } = new List<KeyValuePair<uint, string>>();

        private void BuildBoardName(string boardName)
        {
            BoardName = string.Format("{0}, Board {1}",
                AdqCommon.GetModelName(boardName),
                BoardIndex);
        }

        private void FillTTLVoltageLevels(uint ttlVoltageLevel)
        {
            AddVoltageLevel(ttlVoltageLevel, AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS);
            AddVoltageLevel(ttlVoltageLevel, AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS);
        }

        private void AddVoltageLevel(uint ttlVoltageLevel, uint voltageConstant)
        {
            if((ttlVoltageLevel & voltageConstant) == voltageConstant)
            {
                VoltageLevels.Add(new KeyValuePair<uint, string>(voltageConstant, AdqCommon.GetVoltageText(voltageConstant)));
            }
        }

        public bool GetTTLVoltage(out uint voltage)
        {
            uint adqError = AdqDriver.adqGetBoardTTLVoltage(BoardIndex, out voltage);
            if(adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetBoardTTLVoltage", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        public bool SetTTLVoltage(uint voltage)
        {
            uint adqError = AdqDriver.adqSetBoardTTLVoltage(BoardIndex, voltage);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqSetBoardTTLVoltage", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        public bool Reset()
        {
            Console.WriteLine("Test");
            uint adqError = AdqDriver.adqResetBoard(BoardIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetBoard", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            RaiseInfoTextChanged(AdqCommon.FormatFunctionErrorNumber(functionName, adqError));
        }

        protected virtual void RaiseInfoTextChanged(string text)
        {
            OnInfoTextChanged.Invoke(this, new InfoTextChangedEventArgs(text));
        }
    }
}
