﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;

namespace CommonSettings.Utility
{
    public static class AdqCommon
    {
        public static uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_ERROR | AdqErrors.ADQ_ERROR_NOT_OPEN;
        private static readonly string _formatStr = "{0:0.######}";

        public static string GetCalibrationText(uint calibrationId)
        {
            return GetDictionaryValue(_calibrationTexts, calibrationId);
        }

        public static string GetVoltageText(uint voltage)
        {
            return GetDictionaryValue(_voltageTexts, voltage);
        }

        public static string GetModuleTypeText(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeTexts, moduleType);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _moduleTypeTexts = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"}
        };

        private static readonly Dictionary<uint, string> _calibrationTexts = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CALIBRATION_FACTORY, "Factory"},
            {AdqConstants.ADQ_CALIBRATION_USER_1, "User"}
        };

        private static readonly Dictionary<uint, string> _voltageTexts = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS, string.Format("{0}V", 3.3)},
            {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS, string.Format("{0}V", 5.0)}
        };
        
        public static string FormatFunctionErrorNumber(string functionName, uint adqError)
        {
            return string.Format("API-Function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;
            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }
            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatDoubleValue(double value, string unitText)
        {
            string valueStr;
            string unitStr = string.Empty;
            double tmpVal = Math.Abs(value);
            if (tmpVal == 0)
            {
                valueStr = "0";
            }
            else if (tmpVal < 0.000000001)
            {
                valueStr = string.Format(_formatStr, value * 1e+12);
                unitStr = "p";
            }
            else if (tmpVal < 0.000001)
            {
                valueStr = string.Format(_formatStr, value * 1e+9);
                unitStr = "n";
            }
            else if (tmpVal < 0.001)
            {
                valueStr = string.Format(_formatStr, value * 1e+6);
                unitStr = "µ";
            }
            else if (tmpVal < 1)
            {
                valueStr = string.Format(_formatStr, value * 1e+3);
                unitStr = "m";
            }
            else if (tmpVal >= 1e+9)
            {
                valueStr = string.Format(_formatStr, value / 1e+9);
                unitStr = "G";
            }
            else if (tmpVal >= 1e+6)
            {
                valueStr = string.Format(_formatStr, value / 1e+6);
                unitStr = "M";
            }
            else if (tmpVal >= 1e+3)
            {
                valueStr = string.Format(_formatStr, value / 1e+3);
                unitStr = "k";
            }
            else
            {
                valueStr = string.Format(_formatStr, value);
            }
            return valueStr + " " + unitStr + unitText;
        }
    }
}
