﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the DIReadContinuous project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;

namespace CommonSettings.Utility
{
    public static class AdqCommon
    {
        private const double _oneMicrosecond = 0.000001;
        private const double _oneMillisecond = 0.001;
        private const double _oneSecond = 1.0;
        private const double _oneMinute = 60;
        private const double _oneHour = 3600;

        public static uint DefaultDisplayNumberOfValues = 1000;
        public static double DefaultDesiredSampleFrequency = 1000;
        public static uint DefaultMinNotifyAfterValues = 1;

        public static uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_ERROR | AdqErrors.ADQ_ERROR_NOT_OPEN;
        public static Color[] BitColors = {
            Color.SkyBlue,
            Color.LimeGreen,
            Color.MediumOrchid,
            Color.LightCoral,
            Color.SteelBlue,
            Color.YellowGreen,
            Color.Turquoise,
            Color.HotPink,
            Color.Khaki,
            Color.Tan,
            Color.DarkSeaGreen,
            Color.CornflowerBlue,
            Color.Plum,
            Color.CadetBlue,
            Color.PeachPuff,
            Color.LightSalmon};
        public static Color DefaultBackgroundColor = Color.Black;
        public static Color DefaultAxisColor = Color.DarkSlateGray;
        public static Color DefaultLabelColor = Color.LightSkyBlue;
        public static Color DefaultScrollBarColor = Color.LightGray;

        public static string GetModuleTypeName(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeNames, moduleType);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _moduleTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        };

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;
            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }
            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatFrequency(double frequency)
        {
            string s;
            if (frequency >= 1000000)
            {
                s = string.Format("{0:0.######}MHz", frequency / 1000000);
            }
            else if (frequency >= 1000)
            {
                s = string.Format("{0:0.######}kHz", frequency / 1000);
            }
            else
            {
                s = string.Format("{0:0.########}Hz", frequency);
            }
            return s;
        }

        public static string FormatSampleTime(double seconds)
        {
            if (seconds == 0.0) return "0";

            string s = "";

            if (seconds < _oneMicrosecond)
            {
                s = string.Format("{0:0##}ns", seconds * 1000000000);
            }
            else if (seconds < _oneMillisecond)
            {
                s = string.Format(CultureInfo.InvariantCulture, "{0:0.#######}µs", seconds * 1000000);
            }
            else if (seconds < _oneMinute)
            {
                s = string.Format(CultureInfo.InvariantCulture, "{0:0.#######}s", seconds);
            }
            else if (seconds < _oneHour)
            {
                int minutes = (int)(seconds / _oneMinute);
                double sec = seconds % _oneMinute;
                s = string.Format(CultureInfo.InvariantCulture, "{0}:{1:0.#######}m", minutes, sec);
            }
            else
            {
                int hours = (int)(seconds / _oneHour);
                int minutes = (int)(seconds % _oneHour);
                double sec = seconds % _oneMinute;
                s = string.Format(CultureInfo.InvariantCulture, "{0}:{1}:{2:0.#######}h", hours, minutes, sec);
            }
            return s;
        }

    }
}
