﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the DISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using DISingle.Utility;
using System;

namespace DISingle.ALLDAQ
{
    public class DigitalModule
    {
        public event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;

        public DigitalModule(string boardName, uint boardIndex, uint moduleIndex, ref AdqDriver.ADQGeneralModuleInfo generalInfo, uint portWidthBits)
        {
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            ModuleType = generalInfo.ModuleType;
            Description = generalInfo.ModuleDescription;
            PortWidthBits = portWidthBits;
            BuildModuleName(boardName);
        }

        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public uint ModuleType { get; }
        public string Description { get; }
        public uint PortWidthBits { get; }
        public string ModuleName { get; private set; }

        private void BuildModuleName(string boardName)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeName(ModuleType),
                PortWidthBits);
        }

        public bool ReadSingle(out uint value)
        {
            value = 0;
            bool success = SetDioDirectionToInput();
            if (!success) return false;

            uint adqError = AdqDriver.adqReadSingle(BoardIndex, ModuleIndex, out value);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqReadSingle", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        private bool SetDioDirectionToInput()
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
            {
                adqError = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_INPUT);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqDIOSetDirection", adqError);
                }
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        public bool ResetModule()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetModule", adqError);
            }

            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            UpdateInfoText(string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError));
        }

        private void UpdateInfoText(string text)
        {
            InfoTextChangedEventArgs args = new InfoTextChangedEventArgs
            {
                InfoText = text
            };
            RaiseOnModuleInfoTextChanged(args);
        }

        protected virtual void RaiseOnModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            ModuleInfoTextChanged?.Invoke(this, e);
        }
    }
}
