﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the DISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using ALLDAQ.Boards;
using DISingle.Utility;
using DISingle.ALLDAQ;

namespace DISingle
{
    public partial class FormMain : Form
    {
        private ModuleLocator _moduleLocator;
        private readonly List<DigitalModule> _diModules = new List<DigitalModule>();
        private DigitalModule _selectedModule;

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Shown(object sender, EventArgs e)
        {
            EnableUI(false);
            try
            {
                List<uint> moduleTypes = InitModuleTypesToFind();
                _moduleLocator = new ModuleLocator(_diModules, moduleTypes, InfoTextChanged);
                _moduleLocator.FindModules();

                FillModuleCombobox();
                if (coBoxModules.Items.Count > 0)
                {
                    coBoxModules.SelectedIndex = 0;
                }
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The device driver is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private List<uint> InitModuleTypesToFind()
        {
            List<uint> moduleTypes = new List<uint>
            {
                AdqConstants.ADQ_MODULE_TYPE_DI,
                AdqConstants.ADQ_MODULE_TYPE_DIO
            };

            return moduleTypes;
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (DigitalModule diModule in _diModules)
            {
                coBoxModules.Items.Add(diModule.ModuleName);
            }
        }

        private void BtnReadValue_Click(object sender, EventArgs e)
        {
            bool success = _selectedModule.ReadSingle(out uint value);
            if (success)
            {
                DisplayValue(value);
            }
        }

        private void DisplayValue(uint value)
        {
            AddInfoTextLine(string.Format("Read from Board {0}, Module {1}", 
                            _selectedModule.BoardIndex, _selectedModule.ModuleIndex)
                + string.Format(", Value hex: 0x{0:X}", value)
                + ", bin: " + AdqCommon.IntToBin(value, _selectedModule.PortWidthBits)
                + ", dec: " + value.ToString());
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_selectedModule != null)
            {
                _selectedModule.ModuleInfoTextChanged -= InfoTextChanged;
            }

            EnableUI(coBoxModules.SelectedIndex >= 0);
            if (coBoxModules.SelectedIndex >= 0)
            {
                _selectedModule = _diModules[coBoxModules.SelectedIndex];
                _selectedModule.ModuleInfoTextChanged += InfoTextChanged;
            }
            else
            {
                _selectedModule = null;
            }
            ShowModuleDescription(_selectedModule);
        }

        private void EnableUI(bool enabled)
        {
            btnReadValue.Enabled = enabled;
            btnReset.Enabled = enabled;
        }

        private void ShowModuleDescription(DigitalModule diModule)
        {
            textBoxDescription.Text = (diModule == null) ? "" : diModule.Description;
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            bool success = _selectedModule.ResetModule();
            if (success)
            {
                AddInfoTextLine(string.Format("Reset board {0} module {1}",
                    _selectedModule.BoardIndex,
                    _selectedModule.ModuleIndex));
            }
        }

        private void InfoTextChanged(object sender,  InfoTextChangedEventArgs e)
        {
            AddInfoTextLine(e.InfoText);
        }

        private void AddInfoTextLine(string text)
        {
            textBoxInfo.AppendText(string.Format("[{0}] {1}" + Environment.NewLine,
                    DateTime.Now.ToString("HH:mm:ss.fff"), text));
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseAdqDriver();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            textBoxInfo.Clear();
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }
    }
}
