﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the I2CReadWrite project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ_DLL.Devices.Boards;
using I2CReadWrite.Utility;
using System;
using System.ComponentModel;

namespace I2CReadWrite.Models
{
    public class I2cModule : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public I2cModule(string boardName, uint boardIndex, uint moduleIndex,
            ref AdqDriver.ADQGeneralModuleInfo generalInfo, ref AdqDriver.ADQI2CModuleInfo i2cModuleInfo)
        {
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            SetModuleName(boardName);
            SetModuleDescription(generalInfo.ModuleDescription, ref i2cModuleInfo);
        }

        public uint BoardIndex
        {
            get;
        }
        public uint ModuleIndex
        {
            get;
        }
        public string ModuleName
        {
            get; private set;
        }
        public string Description
        {
            get; private set;
        }

        public bool SetSlaveAddress(uint address)
        {
            uint adqError = AdqDriver.adqI2CSetSlaveAddress(BoardIndex, ModuleIndex, address);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqI2CSetSlaveAddress", adqError);
                return false;
            }
            return true;
        }

        public bool Read(uint numberOfBytesToRead)
        {
            if (BytesRead == null || BytesRead.Length != numberOfBytesToRead)
            {
                BytesRead = new byte[numberOfBytesToRead];
            }

            uint adqError = AdqDriver.adqI2CRead(BoardIndex, ModuleIndex, numberOfBytesToRead, BytesRead, out uint numberOfBytesReturned);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ModuleInfoText = string.Format("API-function adqI2CRead returned Error number: 0x{0:X}, number of bytes returned: {1}",
                                               adqError, numberOfBytesReturned);
                return false;
            }
            return true;
        }

        public byte[] BytesRead
        {
            get; private set;
        }

        public bool Write(byte[] bytes)
        {
            uint adqError = AdqDriver.adqI2CWrite(BoardIndex, ModuleIndex, (uint)bytes.Length, bytes, out uint numberOfBytesWritten);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ModuleInfoText = string.Format("API-function adqI2CWrite returned Error number: 0x{0:X}, number of bytes written: {1}",
                                               adqError, numberOfBytesWritten);
                return false;
            }
            return true;
        }

        public bool WriteThenRead(byte[] bytes, uint numberOfBytesToRead)
        {
            if (BytesRead == null || BytesRead.Length != numberOfBytesToRead)
            {
                BytesRead = new byte[numberOfBytesToRead];
            }

            uint adqError = AdqDriver.adqI2CWriteThenRead(BoardIndex, ModuleIndex, (uint)bytes.Length, bytes,
                out uint numberOfBytesWritten, numberOfBytesToRead, BytesRead, out uint numberOfBytesReturned);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ModuleInfoText = string.Format(
                    "API-function adqI2CWriteThenRead returned Error number: 0x{0:X}, number of bytes written: {1}, number of bytes returned: {2}",
                    adqError, numberOfBytesWritten, numberOfBytesReturned);
                return false;
            }
            return true;
        }

        public bool ResetModule()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetModule", adqError);
                return false;
            }
            return true;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            ModuleInfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        private string _moduleInfoText;
        public string ModuleInfoText
        {
            get
            {
                return _moduleInfoText;
            }
            private set
            {
                _moduleInfoText = value;
                NotifyPropertyChanged(nameof(ModuleInfoText));
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SetModuleName(string boardName)
        {
            ModuleName = AdqCommon.GetModelName(boardName)
                + string.Format(", Board {0}, Module {1}", BoardIndex, ModuleIndex);
        }

        private void SetModuleDescription(string moduleDescription, ref AdqDriver.ADQI2CModuleInfo i2cModuleInfo)
        {
            Description = moduleDescription + Environment.NewLine;
            Description += "- Speed bits per second: " + i2cModuleInfo.SpeedBitsPerSecond.ToString() + Environment.NewLine;
            Description += "- Supports 7-Bit addresses: " + i2cModuleInfo.Supports7BitAddresses.ToString() + Environment.NewLine;
            Description += "- Supports 10-Bit addresses: " + i2cModuleInfo.Supports10BitAddresses.ToString();
        }
    }
}
