﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using Interrupt.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Interrupt.Models
{
    public class ModuleLocator : IDisposable, INotifyPropertyChanged
    {
        private bool _disposed;
        private uint _adqOpenError = AdqCommon.NotOpen;

        public event PropertyChangedEventHandler PropertyChanged;

        public ModuleLocator(PropertyChangedEventHandler handler)
        {
            PropertyChanged += handler;
        }

        public void InitModuleLocator()
        {
            try
            {
                _adqOpenError = OpenAdqDriver();
                if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE) return;

                FindAdqModules();
            }
            catch (DllNotFoundException ex)
            {
                InfoText = "The device driver is not properly installed." + Environment.NewLine
                           + ex.Message;
            }
        }

        private void FindAdqModules()
        {
            uint numberOfBoards = GetNumberOfBoards();
            for (uint boardIndex = 0; boardIndex < numberOfBoards; boardIndex++)
            {
                uint adqError = AdqDriver.adqGetNumberOfModules(boardIndex, out uint numberOfModules);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                    continue;
                }

                string boardName = GetBoardName(boardIndex);
                GetIrqModules(boardName, boardIndex, numberOfModules);
            }
        }

        private uint GetIrqModules(string boardName, uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;

            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleType == AdqConstants.ADQ_MODULE_TYPE_DI)
                {
                    adqError = GetDIModuleInfo(boardName, boardIndex, moduleIdx);
                }
                else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
                {
                    adqError = GetDIOModuleInfo(boardName, boardIndex, moduleIdx);
                }
                else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_DO)
                {
                    adqError = GetDOModuleInfo(boardName, boardIndex, moduleIdx);
                }
                else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_EXTINT)
                {
                    adqError = GetExtIntModuleInfo(boardName, boardIndex, moduleIdx);
                }
                else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_CNTR)
                {
                    adqError = GetCounterModuleInfo(boardName, boardIndex, moduleIdx);
                }
                else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_AI)
                {
                    adqError = GetAIModuleInfo(boardName, boardIndex, moduleIdx);
                }

                if (adqError != AdqErrors.ADQ_ERROR_NONE) break;
            }
            return adqError;
        }

        private uint GetDIModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqDiModuleInfo = new AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqDiModuleInfo.Size);
            Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                adqDiModuleInfo.Size,
                moduleInfo,
                out uint infoSize);
            adqDiModuleInfo = (AdqDriver.ADQDIModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQDIModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqDiModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    adqDiModuleInfo.PortWidthBits);
            }
            return adqError;
        }

        private uint GetDOModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqDoModuleInfo = new AdqDriver.ADQDOModuleInfo(AdqDriver.ADQDOModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqDoModuleInfo.Size);
            Marshal.StructureToPtr(adqDoModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                adqDoModuleInfo.Size,
                moduleInfo,
                out uint infoSize);
            adqDoModuleInfo = (AdqDriver.ADQDOModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQDOModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqDoModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqDoModuleInfo.Size, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    adqDoModuleInfo.PortWidthBits);
            }
            return adqError;
        }

        private uint GetDIOModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqDioModuleInfo = new AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqDioModuleInfo.Size);
            Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                adqDioModuleInfo.Size,
                moduleInfo,
                out uint infoSize);
            adqDioModuleInfo = (AdqDriver.ADQDIOModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQDIOModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqDioModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    adqDioModuleInfo.PortWidthBits);
            }
            return adqError;
        }

        private uint GetExtIntModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            const uint moduleInfoSize = 0;
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                moduleInfoSize,
                IntPtr.Zero,
                out uint infoSize);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(moduleInfoSize == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, moduleInfoSize, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    portWidthBits: 0);
            }

            return adqError;
        }

        private uint GetCounterModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            const uint moduleInfoSize = 0;
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                moduleInfoSize,
                IntPtr.Zero,
                out uint infoSize);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(moduleInfoSize == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, moduleInfoSize, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    portWidthBits: 0);
            }

            return adqError;
        }

        private uint GetAIModuleInfo(string boardName, uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqAiModuleInfo = new AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqAiModuleInfo.Size);
            Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                ref adqGeneralModuleInfo,
                adqAiModuleInfo.Size,
                moduleInfo,
                out uint infoSize);
            adqAiModuleInfo = (AdqDriver.ADQAIModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQAIModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqAiModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize));

            if (adqGeneralModuleInfo.IsIRQSource)
            {
                adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    ref adqGeneralModuleInfo,
                    portWidthBits: 0);
            }
            return adqError;
        }

        private uint GetIrqInfo(string boardName, uint boardIndex, uint moduleIndex,
            ref AdqDriver.ADQGeneralModuleInfo generalInfo, uint portWidthBits)
        {
            var adqIrqInfo = new AdqDriver.ADQIRQInfo(AdqDriver.ADQIRQInfo_ID);

            uint adqError = AdqDriver.adqGetIRQInfo(boardIndex, moduleIndex, ref adqIrqInfo);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetIRQInfo", adqError);
                return adqError;
            }

            var irqModule = new IrqModule(boardName, boardIndex, moduleIndex, ref generalInfo, ref adqIrqInfo, portWidthBits);
            IrqModules.Add(irqModule);

            return adqError;
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private string GetBoardName(uint boardIndex)
        {
            string s = "";
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                s = boardInfo.BoardName;
            }
            return s;
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        public List<IrqModule> IrqModules { get; } = new List<IrqModule>();

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        private string _infoText;
        public string InfoText
        {
            get { return _infoText; }
            private set
            {
                _infoText = value;
                NotifyPropertyChanged(nameof(InfoText));
            }
        }

        private void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        ~ModuleLocator()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                _disposed = true;
                CloseAdqDriver();
            }
        }
    }
}
