﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using ReadBuffer.Utility;
using System;
using System.Runtime.InteropServices;

namespace ReadBuffer.ALLDAQ
{
    public class InputModule : IInputModule
    {
        public uint AdqError { get; set; }
        public event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;

        public InputModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQGeneralModuleInfo moduleInfo)
        {
            _numberOfValues = AdqCommon.DefaultNumberOfValues;
            _modelName = AdqCommon.GetModelName(boardName);
            _boardIndex = boardIndex;
            _moduleIndex = moduleIndex;
            _moduleType = moduleInfo.ModuleType;
            _clockType = moduleInfo.ClockType;
            _maxFrequencyHz = moduleInfo.ClockBaseFrequencyHz / moduleInfo.ClockMinimumDivider;
            _desiredSampleFrequency = _maxFrequencyHz;
            ActualSampleFrequency = _maxFrequencyHz;
        }

        protected string _moduleName;
        public string ModuleName
        {
            get { return _moduleName; }
        }

        protected string _modelName;
        public string ModelName
        {
            get { return _modelName; }
        }

        protected uint _boardIndex;
        public uint BoardIndex
        {
            get { return _boardIndex; }
        }

        protected uint _moduleIndex;
        public uint ModuleIndex
        {
            get { return _moduleIndex; }
        }

        protected uint _moduleType;
        public uint ModuleType 
        {
            get { return _moduleType; }  
        }

        protected uint _clockType;
        public uint ClockType 
        {
            get { return _clockType; }
        }

        protected double _maxFrequencyHz;
        public double MaxfrequencyHz 
        {
            get { return _maxFrequencyHz; }
        }

        private double _desiredSampleFrequency;
        public double DesiredSampleFrequency
        {
            get { return _desiredSampleFrequency; }
            set
            {
                _desiredSampleFrequency = value;
                SetSampleFrequency();
            }
        }

        public double ActualSampleFrequency { get; private set; }
        public double SampleTime { get; private set; }

        public uint SetSampleFrequency()
        {
            uint adqError = AdqDriver.adqSetFrequency(_boardIndex, _moduleIndex, _desiredSampleFrequency, out double actualFrequency);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ErrorTextChanged("adqSetFrequency", adqError);
                actualFrequency = 0.0;
            }
            else
            {
                SampleTime = 1.0 / actualFrequency;
            }
            ActualSampleFrequency = actualFrequency;
            return adqError;
        }

        protected uint _numberOfValues;
        public uint NumberOfValues 
        {
            get { return _numberOfValues; }
            set { _numberOfValues = value; } 
        }

        protected uint[] _values;
        public uint[] Values
        {
            get { return _values; }
        }

        protected uint _numberOfValuesRead;
        public uint NumberOfValuesRead
        {
            get { return _numberOfValuesRead; }
        }

        public double GetSecondsToWait()
        {
            if (ActualSampleFrequency <= 0.0)
            {
                return 0.0;
            }
            return _numberOfValues / ActualSampleFrequency;
        }

        public void ValidateConfigurationValues()
        {
            if (_numberOfValues == 0)
            {
                throw new ApplicationException("Please enter a valid number of values.");    
            }
            if (ActualSampleFrequency == 0.0)
            {
                throw new ApplicationException("Please enter a valid sample frequency."); 
            }
        }

        public void InitValueBuffer()
        {
            if (_values == null || _values.Length != _numberOfValues)
            {
                _values = new uint[_numberOfValues];
            }
            _numberOfValuesRead = 0;
        }

        public void ReadValuesFromModule()
        {
            GCHandle gch = GCHandle.Alloc(_values, GCHandleType.Pinned);
            AdqError = AdqDriver.adqReadBuffer(_boardIndex, _moduleIndex,
                _numberOfValues, _values, out _numberOfValuesRead);
            gch.Free();
        }

        public uint Reset()
        {
            uint adqError = AdqDriver.adqResetModule(_boardIndex, _moduleIndex);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                InfoTextChanged(string.Format("Reset Board {0}, Module {1}",
                        _boardIndex, _moduleIndex));
            }
            else
            {
                ErrorTextChanged("adqResetModule", adqError);
            }
            return adqError;
        }

        public uint StopAcquisition()
        {
            return AdqDriver.adqStop(_boardIndex, _moduleIndex);
        }

        public uint GetModuleStatus(out bool isRunning)
        {
            uint adqError = AdqDriver.adqReadContinuousStatus(_boardIndex, _moduleIndex, false, out uint numValuesAvailable, out isRunning);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ErrorTextChanged("adqReadContinuousStatus", adqError);
            }

            return adqError;
        }

        public void ClearValueBuffer()
        {
            _values = null;
        }

        protected void InfoTextChanged(string infoText)
        {
            var args = new InfoTextChangedEventArgs
            {
                InfoText = infoText
            };
            OnModuleInfoTextChanged(args);
        }

        protected void ErrorTextChanged(string functionName, uint adqError)
        {
            var args = new InfoTextChangedEventArgs
            {
                InfoText = string.Format("Function {0} returned Error number: 0x{1:X}", functionName, adqError)
            };
            OnModuleInfoTextChanged(args);
        }

        protected virtual void OnModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            ModuleInfoTextChanged?.Invoke(this, e);
        }
    }
}
