﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using ReadBuffer.ALLDAQ;
using ReadBuffer.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace ReadBuffer
{
    public partial class FormMain : Form
    {
        private readonly Color _gridLineColor = Color.DarkSlateGray;
        private readonly Color _labelColor = Color.LightSkyBlue;
        private const string _fmtStringPhysValues = "{0:0.000000}";
        private const string _fmtStringDigitalValues = "{0}";
        private const int _idxChartTypeLine = 0;
        private const int _idxChartTypeStepLine = 1;
        private const int _idxChartTypeLineAndMarkers = 2;
        private const int _idxChartTypeStepLineAndMarkers = 3;
        private const int _idxXAxisLabelsTime = 0;

        private ModuleLocator _moduleLocator;
        private readonly List<IInputModule> _inputModules = new List<IInputModule>();
        private IInputModule _selectedAdqModule;
        private bool _isModuleBusy;
        private bool _isFormClosing;
        private string _selectedUnitSymbol;

        private int _oldChartTypeIndex = -1;
        private int _oldNumberOfChartAreas = 0;
        private bool _oldChartAlignmentIsAuto;

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            coBoxChartType.SelectedIndex = _idxChartTypeLine;
            coBoxXAxisLabels.SelectedIndex = _idxXAxisLabelsTime;
            EnableUI(false);
            btnStartStop.Enabled = false;
            InitChartXAxisTitle();
            try
            {
                List<uint> moduleTypes = InitModuleTypesToFind();
                _moduleLocator = new ModuleLocator(_inputModules, moduleTypes, OnInfoTextChanged);
                _moduleLocator.FindInputModules();

                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                textBoxInfo.AppendText(string.Format("Number of input modules found: {0}" 
                    + Environment.NewLine, _inputModules.Count));
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The device driver is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private List<uint> InitModuleTypesToFind()
        {
            List<uint> moduleTypes = new List<uint>
            {
                AdqConstants.ADQ_MODULE_TYPE_DI,
                AdqConstants.ADQ_MODULE_TYPE_DIO,
                AdqConstants.ADQ_MODULE_TYPE_AI,
                AdqConstants.ADQ_MODULE_TYPE_CNTR
            };
            return moduleTypes;
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (IInputModule inputModule in _inputModules)
            {
                coBoxModules.Items.Add(inputModule.ModuleName);
            }
        }

        private void OnInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            textBoxInfo.AppendText(e.InfoText + Environment.NewLine);
        }

        private void DisplayErrorText(string functionName, uint adqError)
        {
            textBoxInfo.AppendText(
                string.Format("Function {0} returned Error number: 0x{1:X}" + Environment.NewLine,
                    functionName, 
                    adqError));
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_selectedAdqModule != null)
            {
                _selectedAdqModule.ClearValueBuffer();
                _selectedAdqModule.ModuleInfoTextChanged -= OnInfoTextChanged;
            }

            ClearDisplayedValues();
            EnableUI(coBoxModules.SelectedIndex >= 0);
            if (coBoxModules.SelectedIndex < 0)
            {
                _selectedAdqModule = null;
                btnStartStop.Enabled = false;
                return;
            }

            _selectedAdqModule = _inputModules[coBoxModules.SelectedIndex];
            _selectedAdqModule.ModuleInfoTextChanged += OnInfoTextChanged;
            textBoxSampleFrequency.Text = string.Format("{0}", _selectedAdqModule.DesiredSampleFrequency);
            coBoxNumOfValues.Text = _selectedAdqModule.NumberOfValues.ToString();
            UpdateActualFrequency(_selectedAdqModule);
            UpdateTimeToWait(_selectedAdqModule);

            if (_selectedAdqModule is AnalogInputModule)
            {
                tabControl1.SelectedTab = tabPageAnalogInput;
                ShowAnalogInputConfiguration((AnalogInputModule)_selectedAdqModule);
            }
            else if (_selectedAdqModule is CounterModule)
            {
                tabControl1.SelectedTab = tabPageCounter;
                ShowCounterConfiguration((CounterModule)_selectedAdqModule);
            }
            else
            {
                tabControl1.SelectedTab = tabPageDigital;
            }
            btnStartStop.Enabled = true;
        }

        private void ShowCounterConfiguration(CounterModule cntrModule)
        {
            FillCounterClockSources(cntrModule);
            coBoxClockSource.SelectedIndex = cntrModule.SelectedClockSourceIndex;
            coBoxCntrModes.SelectedIndex = cntrModule.SelectedCounterModeIndex;
            coBoxCntrStartValue.Text = cntrModule.SelectedStartValue.ToString();
        }

        private void FillCounterClockSources(CounterModule cntrModule)
        {
            coBoxClockSource.Items.Clear();
            foreach (ClockSourceItem item in cntrModule.ClockSourceItems)
            {
                coBoxClockSource.Items.Add(item.ClockSourceName);
            }
        }

        private void ShowAnalogInputConfiguration(AnalogInputModule aiModule)
        {
            FillChannels(aiModule);
            FillRanges(aiModule);
        }

        private void FillChannels(AnalogInputModule aiModule)
        {
            if (coBoxChannels.Items.Count != aiModule.NumberOfChannels)
            {
                coBoxChannels.Items.Clear();
                for (int i = 0; i < aiModule.NumberOfChannels; i++)
                {
                    coBoxChannels.Items.Add(i.ToString());
                }
            }
            coBoxChannels.SelectedIndex = (int)aiModule.SelectedChannelIndex;
        }

        private void FillRanges(AnalogInputModule aiModule)
        {
            coBoxRanges.Items.Clear();
            for (int i = 0; i < aiModule.NumberOfRanges; i++)
            {
                var physicalRange = aiModule.GetRange(i);
                coBoxRanges.Items.Add(physicalRange.RangeText);
            }
            coBoxRanges.SelectedIndex = (int)aiModule.SelectedRangeIndex;
        }

        private void EnableUI(bool enabled)
        {
            coBoxModules.Enabled = enabled;
            textBoxSampleFrequency.Enabled = enabled;
            coBoxNumOfValues.Enabled = enabled;
            btnReset.Enabled = enabled;
            chkBoxDisplayTextValues.Enabled = enabled;
            coBoxXAxisLabels.Enabled = enabled;
            coBoxChannels.Enabled = enabled;
            coBoxRanges.Enabled = enabled;
            coBoxChartType.Enabled = enabled;
            coBoxCntrModes.Enabled = enabled;
            coBoxClockSource.Enabled = enabled;
            coBoxCntrStartValue.Enabled = enabled;
        }

        private void CoBoxNumOfValues_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAdqModule == null) return;

            _selectedAdqModule.NumberOfValues = ParseNumberOfValues(coBoxNumOfValues.Text);
            UpdateTimeToWait(_selectedAdqModule);
        }

        private void TxtBoxSampleFrequency_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAdqModule == null) return;

            _selectedAdqModule.DesiredSampleFrequency = ParseFrequency(textBoxSampleFrequency.Text);
            UpdateActualFrequency(_selectedAdqModule);
            UpdateTimeToWait(_selectedAdqModule);
        }

        private void UpdateActualFrequency(IInputModule adqModule)
        {
            textBoxActualFrequency.Text = AdqCommon.FormatFrequency(adqModule.ActualSampleFrequency);
        }

        private void UpdateTimeToWait(IInputModule adqModule)
        {
            lblDuration.Text = AdqCommon.FormatTimeToWait(adqModule.GetSecondsToWait());
        }

        private double ParseFrequency(string sampleFrequency)
        {
            bool isValid = double.TryParse(sampleFrequency, out double actualFrequency);
            if (!isValid || actualFrequency <= 0) return 0.0;

            return actualFrequency;
        }

        private uint ParseNumberOfValues(string numberOfValues)
        {
            bool isValid = uint.TryParse(numberOfValues, out uint numValues);
            if (!isValid) return 0;

            return numValues;
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            _selectedAdqModule.Reset();
        }

        private void BtnStartStop_Click(object sender, EventArgs e)
        {
            if (_isModuleBusy)
            {
                textBoxInfo.AppendText("Acquisition stopped by user.");
                StopRunningAcquisition();
                btnStartStop.Text = "Start";
                return;
            }

            UpdateChart(coBoxChartType.SelectedIndex, _selectedAdqModule);
            try
            {
                _selectedAdqModule.ValidateConfigurationValues();
                uint adqError = ConfigureModule(_selectedAdqModule);
                if (adqError != AdqErrors.ADQ_ERROR_NONE) return;
    
                EnableUI(false);
                btnStartStop.Text = "Stop";
                _isModuleBusy = true;
                _selectedAdqModule.InitValueBuffer();
                _selectedAdqModule.AdqError = AdqErrors.ADQ_ERROR_NONE;	
                backgroundWorker1.RunWorkerAsync(_selectedAdqModule);
            }
            catch (Exception ex)
            {
                _isModuleBusy = false;
                EnableUI(true);
                btnStartStop.Text = "Start";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);	
            }
        }

        private uint ConfigureModule(IInputModule adqModule)
        {
            uint adqError = adqModule.SetSampleFrequency();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                return adqError;
            }

            if (adqModule is AnalogInputModule)
            {
                return ConfigureAiModule((AnalogInputModule)adqModule);
            }
            else if (adqModule is DigitalInputModule)
            {
                return ConfigureDiModule((DigitalInputModule)adqModule);    
            }
            else if (adqModule is CounterModule)
            {
                return ConfigureCounterModule((CounterModule)adqModule);
            }
            return AdqErrors.ADQ_ERROR_UNSUPPORTED;
        }

        private uint ConfigureAiModule(AnalogInputModule aiModule)
        {
            return aiModule.ConfigureModule((uint)coBoxChannels.SelectedIndex, (uint)coBoxRanges.SelectedIndex);
        }

        private uint ConfigureDiModule(DigitalInputModule diModule)
        {
            return diModule.ConfigureModule();
        }

        private uint ConfigureCounterModule(CounterModule cntrModule)
        {
            uint startValue = ParseNumberOfValues(coBoxCntrStartValue.Text);
            return cntrModule.ConfigureModule(coBoxClockSource.SelectedIndex, coBoxCntrModes.SelectedIndex, startValue);
        }

        private void DisplayValues(IInputModule adqModule)
        {
            if (adqModule is DigitalInputModule )
            {
                DisplayDigitalValuesAsText(adqModule);
                DisplayBitValuesAsChart((DigitalInputModule)adqModule);
            }
            else if (adqModule is CounterModule)
            { 
                DisplayDigitalValuesAsText(adqModule);
                DisplayDigitalValuesAsChart(adqModule);
            }
            else if (adqModule is AnalogInputModule)
            {
                DisplayAnalogValuesAsText((AnalogInputModule)adqModule);
                DisplayAnalogValuesAsChart((AnalogInputModule)adqModule);
            }
        }

        private void DisplayDigitalValuesAsText(IInputModule adqModule)
        {
            if (!chkBoxDisplayTextValues.Checked) return;
            UseWaitCursor = true;
            try
            {
                double time = 0.0;
                var sb = new StringBuilder();
                for (int i = 0; i < adqModule.NumberOfValuesRead; i++)
                {
                    sb.AppendLine(string.Format("{0:000}, {1:0.0000000}sec, Digital: {2} --> 0x{2:X8}", 
                        i, time, adqModule.Values[i]));
                    time += adqModule.SampleTime;
                }
                textBoxInfo.AppendText(sb.ToString());
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void DisplayAnalogValuesAsText(AnalogInputModule aiModule)
        {
            if (!chkBoxDisplayTextValues.Checked) return;

            UseWaitCursor = true;
            try
            {
                double time = 0.0;
                var sb = new StringBuilder();
                for (int i = 0; i < aiModule.NumberOfValuesRead; i++)
                {
                    uint digitalValue = aiModule.Values[i];
                    double physicalValue = aiModule.SelectedRange.DigitalToPhysical(digitalValue);
                    sb.AppendLine(string.Format("{0:000}, {1:0.0000000}sec: {2} --> {3:0.00000000}{4}",
                        i, time, digitalValue, physicalValue, aiModule.SelectedRange.UnitSymbol));
                    time += aiModule.SampleTime;
                }
                textBoxInfo.AppendText(sb.ToString());
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void InitChartXAxisTitle()
        {
            chart1.Titles.Clear();
            Title title = new Title(coBoxXAxisLabels.Text);
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 95;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.BottomCenter;
            title.Docking = Docking.Bottom;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
            chart1.Titles.Add(title);
        }

        private void UpdateChart(int chartTypeIndex, IInputModule adqModule)
        {
            if (adqModule is DigitalInputModule)
            {
                AdjustChartAreasAndSeries(chartTypeIndex, (int)((DigitalInputModule)adqModule).PortWidthBits);
            }
            else
            {
                AdjustChartAreasAndSeries(chartTypeIndex, 1);
            }
        }

        private void AdjustChartAreasAndSeries(int chartTypeIndex, int numberOfChartAreas)
        {
            if (_oldChartAlignmentIsAuto != rbAlignAuto.Checked)
            {
                _oldChartAlignmentIsAuto = rbAlignAuto.Checked;
                _oldNumberOfChartAreas = 0;
            }

            if (_oldNumberOfChartAreas != numberOfChartAreas)
            {
                _oldChartTypeIndex = -1;
                _oldNumberOfChartAreas = numberOfChartAreas;
                InitChartAreas(numberOfChartAreas);
            }

            if (_oldChartTypeIndex != chartTypeIndex)
            {
                _oldChartTypeIndex = chartTypeIndex;
                textBoxInfo.Clear();
                InitSeries(chartTypeIndex, numberOfChartAreas);
            }
            else
            {
                ClearDisplayedValues();
            }
        }

        private void InitChartAreas(int numberOfChartAreas)
        {
            chart1.BackColor = Color.Black;
            chart1.ChartAreas.Clear();

            if (numberOfChartAreas > 1)
            {
                float areaHeight = (100.0f / numberOfChartAreas);

                float areaPositionY = 0;
                for (int i = 0; i < numberOfChartAreas; i++)
                {
                    var chartArea = new ChartArea("ChartArea" + i.ToString());
                    InitChartArea(chartArea);

                    chartArea.Position.Auto = rbAlignAuto.Checked;
                    if (rbAlignVertical.Checked)
                    {
                        chartArea.Position.X = 0;
                        chartArea.Position.Width = 99;
                        chartArea.Position.Y = areaPositionY;
                        chartArea.Position.Height = areaHeight;
                        areaPositionY += areaHeight;
                    }
                    chartArea.AxisY.Minimum = -0.05;
                    chartArea.AxisY.Maximum = 1.05;
                    chartArea.AxisY.Interval = 1;
                    chartArea.AxisY.IntervalOffset = 0.05;
                    chart1.ChartAreas.Add(chartArea);
                }
            }
            else
            {
                var chartArea = new ChartArea("ChartArea0");
                InitChartArea(chartArea);
                chartArea.Position.Auto = false;
                chartArea.Position.X = 0;
                chartArea.Position.Width = 99f;
                chartArea.Position.Y = 0.25f;
                chartArea.Position.Height = 99.5f;
                chartArea.AxisY.Maximum = double.NaN;
                chartArea.AxisY.Minimum = double.NaN;

                chart1.ChartAreas.Add(chartArea);
            }
        }

        private void InitChartArea(ChartArea chartArea)
        {
            chartArea.BackColor = Color.Black;

            chartArea.AxisX.MajorGrid.LineColor = _gridLineColor;
            chartArea.AxisX.MajorTickMark.LineColor = _gridLineColor;
            chartArea.AxisX.MinorGrid.LineColor = _gridLineColor;
            chartArea.AxisX.MinorTickMark.LineColor = _gridLineColor;
            chartArea.AxisX.LineColor = _gridLineColor;
            chartArea.AxisX.LineWidth = 1;
            chartArea.AxisX.LabelStyle.ForeColor = _labelColor;
            chartArea.AxisX.TitleForeColor = _labelColor;
            chartArea.AxisX.ScrollBar.ButtonColor = Color.DarkGray;
            chartArea.AxisX.ScrollBar.BackColor = Color.Gray;
            chartArea.CursorX.IsUserSelectionEnabled = true;

            chartArea.AxisY.MajorGrid.LineColor = _gridLineColor;
            chartArea.AxisY.MajorTickMark.LineColor = _gridLineColor;
            chartArea.AxisY.MinorGrid.LineColor = _gridLineColor;
            chartArea.AxisY.MinorTickMark.LineColor = _gridLineColor;
            chartArea.AxisY.LineColor = _gridLineColor;
            chartArea.AxisY.LineWidth = 1;
            chartArea.AxisY.LabelStyle.ForeColor = _labelColor;
            chartArea.AxisY.TitleForeColor = _labelColor;
            chartArea.AxisY.ScrollBar.ButtonColor = Color.DarkGray;
            chartArea.AxisY.ScrollBar.BackColor = Color.Gray;
            chartArea.CursorY.IsUserSelectionEnabled = true;
        }

        private void InitSeries(int chartTypeIndex, int numberOfChartAreas)
        {
            chart1.GetToolTipText -= Chart1_GetToolTipText;
            chart1.Series.Clear();
            chart1.Legends.Clear();
            for (int i = 0; i < numberOfChartAreas; i++)
            {
                var series = new Series("Series" + i.ToString());
                if (chartTypeIndex == _idxChartTypeLine)
                {
                    series.ChartType = SeriesChartType.Line;
                }
                else if (chartTypeIndex == _idxChartTypeStepLine)
                {
                    series.ChartType = SeriesChartType.StepLine;
                }
                else if (chartTypeIndex == _idxChartTypeLineAndMarkers)
                {
                    series.ChartType = SeriesChartType.Line;
                    series.MarkerStyle = MarkerStyle.Circle;
                }
                else if (chartTypeIndex == _idxChartTypeStepLineAndMarkers)
                {
                    series.ChartType = SeriesChartType.StepLine;
                    series.MarkerStyle = MarkerStyle.Circle;
                }
                else
                {
                    series.ChartType = SeriesChartType.Point;
                }
                series.XValueType = ChartValueType.UInt32;
                series.IsXValueIndexed = false;
                series.ChartArea = chart1.ChartAreas[i].Name;

                var legend = new Legend("Legend" + i.ToString())
                {
                    ForeColor = Color.BlanchedAlmond,
                    BackColor = Color.Transparent,
                    DockedToChartArea = series.ChartArea
                };
                chart1.Legends.Add(legend);
                series.Legend = legend.Name;
                chart1.Series.Add(series);
            }
            chart1.GetToolTipText += Chart1_GetToolTipText;
        }

        private void Chart1_GetToolTipText(object sender, ToolTipEventArgs e)
        {
            if (e.HitTestResult.ChartElementType == ChartElementType.DataPoint)
            {
                Series series = e.HitTestResult.Series;
                if (series != null)
                {
                    DataPoint dp = series.Points[e.HitTestResult.PointIndex];
                    e.Text = FormatTooltipText(dp);
                }
            }
            else if (e.HitTestResult.ChartElementType == ChartElementType.Axis
                || e.HitTestResult.ChartElementType == ChartElementType.TickMarks)
            {
                if (sender is Chart)
                {
                    HitTestResult result = ((Chart)sender).HitTest(e.X, e.Y, ChartElementType.DataPoint);
                    if (result.ChartElementType == ChartElementType.DataPoint)
                    {
                        DataPoint dp = result.Series.Points[result.PointIndex];
                        e.Text = FormatTooltipText(dp);
                    }
                }
            }
            
        }

        private string FormatTooltipText(DataPoint dp)
        {
            if (_selectedAdqModule is AnalogInputModule)
            {
                return string.Format("X={0}, Y={1}",
                            dp.XValue,
                            AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]) + _selectedUnitSymbol);
            }
            return string.Format("X={0}, Y={1}",
                            dp.XValue, dp.YValues[0]);
        }

        private void DisplayBitValuesAsChart(DigitalInputModule diModule)
        {
            UseWaitCursor = true;
            try
            {
                DataPointCollection points0 = null;
                chart1.Titles[0].Text = coBoxXAxisLabels.Text;
                for (int bit = 0; bit < diModule.PortWidthBits; bit++)
                {
                    Series series = chart1.Series[bit];
                    ChartArea ca = chart1.ChartAreas[bit];
                    UpdateChartArea(ca, 0.25, string.Empty, _fmtStringDigitalValues);
                    UpdateLegendText(series, "Bit " + bit.ToString());
                    series.Points.SuspendUpdates();
                    double xValue = 0.0;
                    double xValueDelta = GetXValueDelta();
                    for (int valueIdx = 0; valueIdx < diModule.NumberOfValuesRead; valueIdx++)
                    {
                        var dp = new DataPoint(valueIdx, (diModule.Values[valueIdx] >> bit) & 1);
                        if (points0 == null)
                        {
                            dp.AxisLabel = GetXAsixLabel(xValue);
                            xValue += xValueDelta;
                        }
                        else
                        {
                            dp.AxisLabel = points0[valueIdx].AxisLabel;
                        }
                        series.Points.Add(dp);
                    }
                    if (points0 == null)
                    {
                        points0 = series.Points;
                    }
                    series.Points.ResumeUpdates();
                }
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void DisplayDigitalValuesAsChart(IInputModule adqModule)
        {
            UseWaitCursor = true;
            try
            {
                Series series = chart1.Series[0];
                ChartArea ca = chart1.ChartAreas[0];
                chart1.Titles[0].Text = coBoxXAxisLabels.Text;
                UpdateChartArea(ca, 0.25, "Digital value", _fmtStringDigitalValues);
                UpdateLegendText(series, "Module " + adqModule.ModuleIndex.ToString());
                series.Points.SuspendUpdates();
                double xValue = 0.0;
                double xValueDelta = GetXValueDelta();
                for (int valueIdx = 0; valueIdx < adqModule.NumberOfValuesRead; valueIdx++)
                {
                    var dp = new DataPoint(valueIdx, adqModule.Values[valueIdx])
                    {
                        AxisLabel = GetXAsixLabel(xValue)
                    };
                    series.Points.Add(dp);
                    xValue += xValueDelta;
                }
                ca.RecalculateAxesScale();
                series.Points.ResumeUpdates();
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void DisplayAnalogValuesAsChart(AnalogInputModule aiModule)
        {
            UseWaitCursor = true;
            try
            {
                Series series = chart1.Series[0];
                ChartArea ca = chart1.ChartAreas[0];
                chart1.Titles[0].Text = coBoxXAxisLabels.Text;
                var range = aiModule.SelectedRange;
                _selectedUnitSymbol = range.UnitSymbol;
                double scrollSize = range.PhysicalResolution / 4.0;
                UpdateChartArea(ca, scrollSize, range.UnitName, _fmtStringPhysValues);
                UpdateLegendText(series, "Channel " + aiModule.SelectedChannelIndex.ToString());
                double xValue = 0.0;
                double xValueDelta = GetXValueDelta();
                series.Points.SuspendUpdates();
                for (int i = 0; i < aiModule.NumberOfValuesRead; i++)
                {
                    var dp = new DataPoint(i, range.DigitalToPhysical(aiModule.Values[i]))
                    {
                        AxisLabel = GetXAsixLabel(xValue)
                    };
                    series.Points.Add(dp);
                    xValue += xValueDelta;
                }
                ca.RecalculateAxesScale();
                series.Points.ResumeUpdates();
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void UpdateChartArea(ChartArea ca, double yScrollSize, string yAxisTitle, string yAxisLabelFormat)
        {
            ca.CursorY.Interval = yScrollSize;
            ca.AxisY.ScaleView.SmallScrollMinSize = yScrollSize;
            ca.AxisY.ScaleView.SmallScrollSize = yScrollSize;
            ca.AxisY.Title = yAxisTitle;
            ca.AxisY.LabelStyle.Format = yAxisLabelFormat;
        }

        private void UpdateLegendText(Series series, string legendText)
        {
            series.LegendText = legendText;
            series.IsVisibleInLegend = true;
        }

        private string GetXAsixLabel(double xValue)
        {
            if (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime)
            {
                return AdqCommon.FormatSampleTime(xValue);
            }
            return ((int)xValue).ToString();
        }

        private double GetXValueDelta()
        {
            return (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? _selectedAdqModule.SampleTime : 1.0;
        }

        private void RemoveAllSeriesPoints()
        {
            for (int i = 0; i < chart1.Series.Count; i++)
            {
                RemovePoints(chart1.Series[i]);
            }
        }

        private void RemovePoints(Series series)
        {
            series.IsVisibleInLegend = false;
            series.Points.SuspendUpdates();
            while(series.Points.Count > 0)
            {
                series.Points.RemoveAt(series.Points.Count - 1);
            }
            series.Points.ResumeUpdates();
            series.Points.Clear();
        }

        private void BackgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            IInputModule adqModule = e.Argument as IInputModule;
            adqModule.ReadValuesFromModule();
            e.Result = adqModule;
        }

        private void BackgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (_isFormClosing) return;

            IInputModule adqModule = e.Result as IInputModule;
            _isModuleBusy = false;
            if (adqModule.AdqError == AdqErrors.ADQ_ERROR_NONE)
            {
                DisplayValues(adqModule);
            }
            else
            {
                DisplayErrorText("adqReadBuffer", adqModule.AdqError);
            }
            EnableUI(true);
            btnStartStop.Text = "Start";
        }

        private void StopRunningAcquisition()
        {
            _selectedAdqModule.GetModuleStatus(out bool isRunning);
            if (isRunning)
            {
                _selectedAdqModule.StopAcquisition();
            }
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            CloseAdqDriver();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void ClearDisplayedValues()
        {
            RemoveAllSeriesPoints();
            textBoxInfo.Clear();
        }

        private void TextBoxInfo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Modifiers == Keys.Control && e.KeyCode == Keys.A)
            {
                textBoxInfo.SelectionStart = 0;
                textBoxInfo.SelectionLength = textBoxInfo.Text.Length;
            }
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_isModuleBusy )
            {
                DialogResult result = MessageBox.Show("The acquisition is still running."
                    + Environment.NewLine
                    + "Do you want to close the program?", 
                    "Module busy", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.Yes)
                {
                    _isFormClosing = true;
                    StopRunningAcquisition();
                    e.Cancel = false;
                }
                else
                {
                    e.Cancel = true;
                }
            }
        }

        private void TsMenuItemSaveGraphic_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetImageFileName(_selectedAdqModule, ".png");
            saveFileDialog1.DefaultExt = "png";
            saveFileDialog1.ShowDialog();
        }

        private string GetImageFileName(IInputModule adqModule, string fileExt)
        {
            string s = string.Empty;
            if (adqModule != null)
            {
                s = string.Format("{0}, Board {1}, Module {2}, ",
                    adqModule.ModelName,
                    adqModule.BoardIndex,
                    adqModule.ModuleIndex);
            }
            return s + DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss") + fileExt;
        }

        private ChartImageFormat GetImageFormat(int filterIndex)
        {
            ChartImageFormat imgFormat;
            switch (filterIndex)
            {
                case 1:
                    imgFormat = ChartImageFormat.Png;
                    break;
                case 2:
                    imgFormat = ChartImageFormat.Gif;
                    break;
                case 3:
                    imgFormat = ChartImageFormat.Bmp;
                    break;
                case 4:
                    imgFormat = ChartImageFormat.Tiff;
                    break;
                case 5:
                    imgFormat = ChartImageFormat.Jpeg;
                    break;
                default:
                    imgFormat = ChartImageFormat.Png;
                    break;
            }
            return imgFormat;
        }

        private void SaveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            ChartImageFormat imgFormat = GetImageFormat(saveFileDialog1.FilterIndex);
            chart1.SaveImage(saveFileDialog1.FileName, imgFormat);
        }

        private void TsMnuItemCopyToClipboard_Click(object sender, EventArgs e)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                chart1.SaveImage(ms, ChartImageFormat.Png);
                var bitmap = new Bitmap(ms);
                Clipboard.SetImage(bitmap);
            }
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }
    }
}
