﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.ComponentModel;
using System.Runtime.InteropServices;
using WriteBufferDO.Utility;

namespace WriteBufferDO.Models
{
    public class DigitalModule : INotifyPropertyChanged
    {
        public uint[] Values;
        public uint PortWidthBits { get; private set; }
        public string ModuleName { get; private set; }
        public uint AdqError { get; private set; }
        public uint NumberOfValuesWritten { get; set; }
        public event PropertyChangedEventHandler PropertyChanged;

        public DigitalModule(string boardName, uint boardIndex, uint moduleIndex, ref AdqDriver.ADQGeneralModuleInfo moduleInfo, uint portWidthBits)
        {
            _numberOfValues = portWidthBits;
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            PortWidthBits = portWidthBits;
            ModuleType = moduleInfo.ModuleType;
            ClockType = moduleInfo.ClockType;
            MaxfrequencyHz = moduleInfo.ClockBaseFrequencyHz / moduleInfo.ClockMinimumDivider;
            _desiredOutputFrequency = MaxfrequencyHz;
            ActualOutputFrequency = MaxfrequencyHz;
            BuildModuleName(boardName);
        }

        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public uint ModuleType { get; }
        public uint ClockType { get; }
        public double MaxfrequencyHz { get; }

        double _desiredOutputFrequency;
        public double DesiredOutputFrequency
        {
            get { return _desiredOutputFrequency; }
            set
            {
                if (_desiredOutputFrequency != value)
                {
                    _desiredOutputFrequency = value;
                    NotifyPropertyChanged(nameof(DesiredOutputFrequency));
                    SetOutputFrequency();
                }
            }
        }

        private bool SetOutputFrequency()
        {
            double actualFrequency = 0.0;
            if (_desiredOutputFrequency > 0.0)
            {
                uint adqError = AdqDriver.adqSetFrequency(BoardIndex, ModuleIndex, _desiredOutputFrequency, out actualFrequency);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetErrorText("adqSetFrequency", adqError);
                    actualFrequency = 0.0;
                }

            }
            SetActualOutputFrequency(actualFrequency);
            return (actualFrequency > 0.0);
        }

        private void SetActualOutputFrequency(double value)
        {
            if (ActualOutputFrequency != value)
            {
                ActualOutputFrequency = value;
                NotifyPropertyChanged(nameof(ActualOutputFrequency));
            }
        }

        public double ActualOutputFrequency { get; private set; }

        public double GetSecondsToWait()
        {
            if (ActualOutputFrequency <= 0.0)
            {
                return 0.0;
            }
            return _numberOfValues / ActualOutputFrequency;
        }

        uint _numberOfValues;
        public uint NumberOfValues
        {
            get { return _numberOfValues; }
            set 
            {
                if (_numberOfValues != value)
                {
                    _numberOfValues = value;
                    NotifyPropertyChanged(nameof(NumberOfValues));
                }
            }
        }

        uint _repeatBufferCount = AdqCommon.DefaultRepeatBufferCount;
        public uint RepeatBufferCount
        {
            get { return _repeatBufferCount; }
            set
            {
                if (_repeatBufferCount != value)
                {
                    _repeatBufferCount = value;
                    NotifyPropertyChanged(nameof(RepeatBufferCount));
                }
            }
        }

        private void BuildModuleName(string boardName)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit, Max. {5}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeName(ModuleType),
                PortWidthBits,
                AdqCommon.FormatFrequency(MaxfrequencyHz));
        }

        public void ResetModule()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqResetModule", adqError);
                return;
            }
            ModuleInfoText = string.Format("Reset board {0}, module {1}", BoardIndex, ModuleIndex);
        }

        public bool ConfigureModule()
        {
            if(! SetOutputFrequency()) return false;

            return ConfigureDioModuleToOutput();
        }

        public void WriteBuffer()
        {
            NumberOfValuesWritten = 0;
            AdqError = AdqErrors.ADQ_ERROR_NONE;
            GCHandle gch = GCHandle.Alloc(Values, GCHandleType.Pinned);
            AdqError = AdqDriver.adqWriteBuffer(BoardIndex, ModuleIndex,
                NumberOfValues, Values, _repeatBufferCount, out uint numValuesWritten);
            NumberOfValuesWritten = numValuesWritten;
            gch.Free();
        }

        private bool ConfigureDioModuleToOutput()
        {
            if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
            {
                uint adqError = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_OUTPUT);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetErrorText("adqDIOSetDirection", adqError);
                    return false;
                }
            }
            return true;
        }

        public bool GetModuleStatus()
        {
            uint adqError = AdqDriver.adqWriteContinuousStatus(BoardIndex, ModuleIndex, false, out uint numAccept, out bool isModuleRunning);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqWriteContinuousStatus", adqError);
            }
            return isModuleRunning;
        }

        public uint StopOutput()
        {
            return AdqDriver.adqStop(BoardIndex, ModuleIndex);
        }

        public string ValidateDesiredOutputFrequency(double frequency)
        {
            string msg = string.Empty;
            if (frequency <= 0)
            {
                msg = "The output frequency must be greater than 0";
            }
            return msg;
        }

        public string ValidateNumberOfValues(uint numValues)
        {
            return ValidateValueIsPositive(numValues);
        }

        public string ValidateRepeatBufferCount(uint count)
        {
            return ValidateValueIsPositive(count);
        }

        private string ValidateValueIsPositive(uint value)
        {
            string msg = string.Empty;
            if (value == 0)
            {
                msg = "The number must be greater than 0";
            }
            return msg;
        }

        private void SetErrorText(string functionName, uint adqError)
        {
            ModuleInfoText = string.Format("Function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        string _infoText = string.Empty;
        public string ModuleInfoText
        {
            get { return _infoText; }
            set
            {
                _infoText = value;
                if (value != string.Empty)
                {
                    NotifyPropertyChanged(nameof(ModuleInfoText));
                }
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
