﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using WriteBufferDO.Utility;

namespace WriteBufferDO.Models
{
    public class ModuleLocator : IDisposable, INotifyPropertyChanged
    {
        private bool _disposed = false;
        private uint _adqOpenError = AdqCommon.NotOpen;

        public event PropertyChangedEventHandler PropertyChanged;
        public List<DigitalModule> DigitalModules { get; private set; }

        public ModuleLocator(PropertyChangedEventHandler handler)
        {
            PropertyChanged += handler;
        }

        public void InitModuleLocator()
        {
            try
            {
                _adqOpenError = OpenAdqDriver();
                if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE) return;

                DigitalModules = new List<DigitalModule>();
                List<uint> moduleTypes = InitModuleTypesToFind();
                FindAdqModules(moduleTypes);
                if (DigitalModules.Count == 0)
                {
                    InfoText = AdqCommon.FormatOutputText("No DI or DIO modules found.");
                }
            }
            catch (DllNotFoundException ex)
            {
                InfoText = AdqCommon.FormatOutputText("The device driver is not properly installed." + Environment.NewLine
                             + ex.Message);
            }
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        private List<uint> InitModuleTypesToFind()
        {
            List<uint> moduleTypes = new List<uint>
            {
                AdqConstants.ADQ_MODULE_TYPE_DO,
                AdqConstants.ADQ_MODULE_TYPE_DIO
            };
            return moduleTypes;
        }

        private void FindAdqModules(List<uint> moduleTypes)
        {
            uint numberOfBoards = GetNumberOfBoards();
            for (uint boardIndex = 0; boardIndex < numberOfBoards; boardIndex++)
            {
                uint adqError = AdqDriver.adqGetNumberOfModules(boardIndex, out uint numberOfModules);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                    continue;
                }

                adqError = GetModuleTypes(boardIndex, numberOfModules, moduleTypes);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    continue;
                }
            }
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private uint GetModuleTypes(uint boardIndex, uint numberOfModules, List<uint> moduleTypes)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleTypes.Contains(moduleType))
                {
                    adqError = GetModuleInfo(boardIndex, moduleIdx, moduleType);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
            return adqError;
        }

        private uint GetModuleInfo(uint boardIndex, uint moduleIdx, uint moduleType)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;

            if (moduleType == AdqConstants.ADQ_MODULE_TYPE_DO)
            {
                adqError = GetDOModuleInfo(boardIndex, moduleIdx);
            }
            else if (moduleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
            {
                adqError = GetDIOModuleInfo(boardIndex, moduleIdx);
            }
            return adqError;
        }

        private uint GetDOModuleInfo(uint boardIndex, uint moduleIndex)
        {
            uint adqError;
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqDoModuleInfo = new AdqDriver.ADQDOModuleInfo(AdqDriver.ADQDOModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqDoModuleInfo.Size);
            Marshal.StructureToPtr(adqDoModuleInfo, moduleInfo, false);

            adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, ref adqGeneralModuleInfo, adqDoModuleInfo.Size, moduleInfo, out uint infoSize);
            adqDoModuleInfo = (AdqDriver.ADQDOModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQDOModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqDoModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqDoModuleInfo.Size, infoSize));

            if (IsClockControlledModule(adqGeneralModuleInfo.ClockType))
            {
                Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, 
                    string.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex));

                string boardName = GetBoardName(boardIndex);
                var doModule = new DigitalModule(boardName, boardIndex, moduleIndex, ref adqGeneralModuleInfo, adqDoModuleInfo.PortWidthBits);
                DigitalModules.Add(doModule);
            }
            return adqError;
        }

        private uint GetDIOModuleInfo(uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqDioModuleInfo = new AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqDioModuleInfo.Size);
            Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, ref adqGeneralModuleInfo, adqDioModuleInfo.Size, moduleInfo, out uint infoSize);
            adqDioModuleInfo = (AdqDriver.ADQDIOModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQDIOModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqDioModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize));

            if (IsClockControlledModule(adqGeneralModuleInfo.ClockType))
            {
                Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, 
                    string.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex));

                string boardName = GetBoardName(boardIndex);
                var dioModule = new DigitalModule(boardName, boardIndex, moduleIndex, ref adqGeneralModuleInfo, adqDioModuleInfo.PortWidthBits);
                DigitalModules.Add(dioModule);
            }
            return adqError;
        }

        private bool IsClockControlledModule(uint clockType)
        {
            return clockType == AdqConstants.ADQ_CLOCK_TYPE_HARDWARE || clockType == AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE;
        }

        private string GetBoardName(uint boardIndex)
        {
            string s = string.Empty;
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                s = boardInfo.BoardName;
            }
            return s;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = AdqCommon.FormatOutputText(string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError));
        }

        private string _infoText;
        public string InfoText
        {
            get { return _infoText; }
            private set
            {
                _infoText = value;
                NotifyPropertyChanged(nameof(InfoText));
            }
        }

        private void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        ~ModuleLocator()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                _disposed = true;
                CloseAdqDriver();
            }
        }
    }
}
