﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;

namespace WriteBufferDO.Utility
{
    static class AdqCommon
    {
        public static readonly uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_FATAL | AdqErrors.ADQ_ERROR_NOT_OPEN;
        public static readonly uint DefaultRepeatBufferCount = 10;
        public static readonly string TimeFmt = "HH:mm:ss.fff";

        public static string FormatOutputText(string text)
        {
            return "[" + DateTime.Now.ToString("HH:mm:ss.fff") + "] " + text;
        }

        public static string GetModuleTypeName(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeNames, moduleType);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _moduleTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        };

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatFrequency(double frequency)
        {
            string s;
            if (frequency >= 1000000)
            {
                s = string.Format("{0:0.######}MHz", frequency / 1000000);
            }
            else if (frequency >= 1000)
            {
                s = string.Format("{0:0.######}kHz", frequency / 1000);
            }
            else
            {
                s = string.Format("{0:0.########}Hz", frequency);
            }
            return s;
        }

        public static string FormatOutputTime(double seconds)
        {
            string s;
            try
            {
                var ts = TimeSpan.FromSeconds(seconds);

                s = string.Concat(ts.Days >= 1 ? string.Concat(ts.Days, " d ") : string.Empty,
                                  ts.Hours >= 1 ? string.Concat(ts.Hours, " h ") : string.Empty,
                                  ts.Minutes >= 1 ? string.Concat(ts.Minutes, " min ") : string.Empty,
                                  ts.Seconds >= 1 ? string.Concat(ts.Seconds, " sec ") : string.Empty,
                                  ts.Milliseconds >= 1 ? string.Concat(ts.Milliseconds, " ms") : string.Empty);
                if (string.IsNullOrEmpty(s) && seconds > 0)
                {
                    s = "< 1ms";
                }
            }
            catch (OverflowException)
            {
                s = string.Empty;
            }
            return s;
        }
    }
}
