﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;
using WriteBufferDO.Models;
using WriteBufferDO.Utility;

namespace WriteBufferDO.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged, IDataErrorInfo
    {
        private const string _fmtStr = "{0:0.###}";

        private readonly ModuleLocator _moduleLocator;
        private BackgroundWorker _worker;
        private bool _isAppClosing;
        private DispatcherTimer _dispatcherTimer;
        private readonly Dictionary<string, string> _errorInfo = new Dictionary<string, string>();

        public RelayCommand ResetModuleCommand { get; set; }
        public RelayCommand ClearInfoTextCommand { get; set; }
        public RelayCommand StartStopWriteBufferCommand { get; set; }
        public RelayCommand CloseCommand { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public MainViewModel()
        {
            LastValueZeroChecked = true;
            InitCommands();
            InitValueGenerators();
            _moduleLocator = new ModuleLocator(MainViewModel_PropertyChanged);
            _moduleLocator.InitModuleLocator();
            RaisePropertyChanged(nameof(CanSelectModule));
            InitDispatcherTimer();
        }

        private void InitDispatcherTimer()
        {
            _dispatcherTimer = new DispatcherTimer();
            _dispatcherTimer.Tick += new EventHandler(DispatcherTimer_Tick);
            _dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 500);
        }

        private void InitValueGenerators()
        {
            ValueGenerators = new List<IValueGenerator>
            {
                new BitShiftLeftValueGenerator(),
                new BitShiftRightValueGenerator(),
                new CountUpValueGenerator(),
                new RandomValueGenerator()
            };
        }

        private void InitCommands()
        {
            ResetModuleCommand = new RelayCommand(ResetModuleExecute, CanResetModuleExecute);
            ClearInfoTextCommand = new RelayCommand(ClearInfoTextExecute);
            StartStopWriteBufferCommand = new RelayCommand(StartStopWriteBufferExecute, CanStartStopWriteBufferExecute);
            CloseCommand = new RelayCommand(CloseApplicationExecute);
        }

        public List<DigitalModule> DigitalModules
        {
            get { return _moduleLocator.DigitalModules; }
        }

        public List<IValueGenerator> ValueGenerators { get; private set; }

        IValueGenerator _selectedValueGenerator;
        public IValueGenerator SelectedGenerator
        {
            get { return _selectedValueGenerator; }
            set
            {
                if (_selectedValueGenerator != value)
                {
                    _selectedValueGenerator = value;
                    RaisePropertyChanged(nameof(SelectedGenerator));
                }
            }
        }

        private DigitalModule _selectedModule;
        public DigitalModule SelectedModule
        {
            get { return _selectedModule; }
            set
            {
                if (_selectedModule != value)
                {
                    if (_selectedModule != null)
                    {
                        _selectedModule.PropertyChanged -= MainViewModel_PropertyChanged;
                    }

                    _selectedModule = value;
                    IsModuleIdle = _selectedModule != null;
                    if (_selectedModule != null)
                    {
                        _selectedModule.PropertyChanged += MainViewModel_PropertyChanged;
                        RepeatBufferCount = _selectedModule.RepeatBufferCount.ToString();
                        DesiredOutputFrequency = string.Format(_fmtStr, _selectedModule.DesiredOutputFrequency);
                        NumberOfValues = _selectedModule.NumberOfValues.ToString();
                    }
                    else
                    {
                        RepeatBufferCount = "1";
                        DesiredOutputFrequency = "1000";
                        NumberOfValues = "8";
                    }

                    RaisePropertyChanged(nameof(SelectedModule));
                    RaisePropertyChanged(nameof(OutputTimeFormatted));
                    RaisePropertyChanged(nameof(ActualOutputFrequency));
                }
            }
        }

        private string _desiredOutputFrequency;
        public string DesiredOutputFrequency
        {
            get { return _desiredOutputFrequency; }
            set
            {
                if (_desiredOutputFrequency != value)
                {
                    _desiredOutputFrequency = value;
                    if (_selectedModule == null) return;

                    string msg = ValidateDoubleValue(_desiredOutputFrequency, out double frequency);
                    if (string.IsNullOrEmpty(msg))
                    {
                        msg = _selectedModule.ValidateDesiredOutputFrequency(frequency);
                    }

                    UpdateErrorInfo(nameof(DesiredOutputFrequency), msg);
                    if (string.IsNullOrEmpty(msg))
                    {
                        _selectedModule.DesiredOutputFrequency = frequency;
                    }
                    RaisePropertyChanged(nameof(DesiredOutputFrequency));
                    StartStopWriteBufferCommand.RaiseCanExecuteChanged();
                }
            }
        }

        public double ActualOutputFrequency
        {
            get
            {
                if (_selectedModule != null)
                {
                    return _selectedModule.ActualOutputFrequency;
                }
                return 0.0;
            }
        }

        private string _numberOfValues;
        public string NumberOfValues
        {
            get { return _numberOfValues; }
            set
            {
                if (_numberOfValues != value)
                {
                    _numberOfValues = value;

                    if (_selectedModule == null) return;

                    string msg = ValidateUintTextInput(_numberOfValues, out uint numValues, _selectedModule.ValidateNumberOfValues);
                    UpdateErrorInfo(nameof(NumberOfValues), msg);

                    if (string.IsNullOrEmpty(msg))
                    {
                        _selectedModule.NumberOfValues = numValues;
                    }

                    RaisePropertyChanged(nameof(NumberOfValues));
                    StartStopWriteBufferCommand.RaiseCanExecuteChanged();
                }
            }
        }

        public bool LastValueZeroChecked { get; set; }


        private string _repeatBufferCount;
        public string RepeatBufferCount
        {
            get { return _repeatBufferCount; }
            set
            {
                if (_repeatBufferCount != value)
                {
                    _repeatBufferCount = value;

                    if (_selectedModule == null) return;

                    string msg = ValidateUintTextInput(_repeatBufferCount, out uint count, _selectedModule.ValidateRepeatBufferCount);
                    UpdateErrorInfo(nameof(RepeatBufferCount), msg);

                    if (string.IsNullOrEmpty(msg))
                    {
                        _selectedModule.RepeatBufferCount = count;
                    }

                    RaisePropertyChanged(nameof(RepeatBufferCount));
                    StartStopWriteBufferCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private static string ValidateDoubleValue(string textValue, out double doubleValue)
        {
            doubleValue = 0.0;
            string msg = null;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else if (!double.TryParse(textValue, out doubleValue))
            {
                msg = "The entered text \"" + textValue + "\" is not a valid number";
            }
            return msg;
        }

        private string ValidateUintTextInput(string textValue, out uint value, Func<uint, string> moduleValidationMethod)
        {
            value = 0;
            string msg;

            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else
            {
                if (!uint.TryParse(textValue, out value))
                {
                    msg = "The entered text \"" + textValue + "\" is not a valid number";
                }
                else
                {
                    msg = moduleValidationMethod(value);
                }
            }
            return msg;
        }

        private void UpdateErrorInfo(string propertyName, string msg)
        {
            if (string.IsNullOrEmpty(msg))
            {
                _errorInfo.Remove(propertyName);
            }
            else
            {
                _errorInfo.AddOrUpdate(propertyName, msg);
            }
        }

        public string OutputTimeFormatted
        {
            get
            {
                if (_selectedModule == null) return string.Empty;

                double seconds = _selectedModule.GetSecondsToWait();
                string txt = "Buffer: " + AdqCommon.FormatOutputTime(seconds);
                if (_selectedModule.RepeatBufferCount > 1)
                {
                    txt += ", Total: " + AdqCommon.FormatOutputTime(seconds * _selectedModule.RepeatBufferCount);
                }
                return txt;
            }
        }

        private void AppendInfoText(string text)
        {
            if (InfoText != string.Empty)
            {
                InfoText += Environment.NewLine;
            }
            InfoText += text;
            //InfoText += "[" + DateTime.Now.ToString("HH:mm:ss.fff") + "] " + text;
            RaisePropertyChanged(nameof(InfoText));
        }

        public string InfoText { get; private set; } = string.Empty;

        private bool CanResetModuleExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void ResetModuleExecute(object obj)
        {
            _selectedModule.ResetModule();
        }

        private void CloseApplicationExecute(object obj)
        {
            if (!_isModuleIdle)
            {
                _isAppClosing = true;
                _selectedModule.StopOutput();
            }
            Application.Current.MainWindow.Close();
        }

        private void ClearInfoTextExecute(object obj)
        {
            InfoText = string.Empty;
            RaisePropertyChanged(nameof(InfoText));
        }

        private bool CanStartStopWriteBufferExecute(object obj)
        {
            return _selectedModule != null
                && _errorInfo.Count == 0;
        }

        private void StartStopWriteBufferExecute(object obj)
        {
            if (_isModuleIdle)
            {
                bool isConfigured = _selectedModule.ConfigureModule();
                if (isConfigured)
                {
                    _selectedModule.Values = _selectedValueGenerator.GenerateValues(_selectedModule.PortWidthBits, _selectedModule.NumberOfValues, LastValueZeroChecked);
                    StartBackgroundWorker();
                }
            }
            else
            {
                _selectedModule.StopOutput();
            }
        }

        private bool _isModuleIdle;
        public bool IsModuleIdle
        {
            get { return _isModuleIdle; }
            set
            {
                if (_isModuleIdle != value)
                {
                    _isModuleIdle = value;

                    ButtonContent = _isModuleIdle ? "Start" : "Stop";
                    RaisePropertyChanged(nameof(IsModuleIdle));
                    RaisePropertyChanged(nameof(CanSelectModule));
                }
            }
        }

        public bool CanSelectModule
        {
            get { return _moduleLocator.DigitalModules.Count > 0 && _isModuleIdle; }
        }


        string _buttonContent = "Start";
        public string ButtonContent
        {
            get { return _buttonContent; }
            set
            {
                _buttonContent = value;
                RaisePropertyChanged(nameof(ButtonContent));
            }
        }

        public bool IsInputValid
        {
            get
            {
                return _errorInfo.Count == 0;
            }
        }

        public string Error => "MainViewModel.Error";

        public string this[string propertyName]
        {
            get
            {
                return _errorInfo.ContainsKey(propertyName) ? _errorInfo[propertyName] : null;
            }
        }

        private void StartBackgroundWorker()
        {
            IsModuleIdle = false;
            AppendInfoStartWriting();
            _worker = new BackgroundWorker
            {
                WorkerReportsProgress = false,
                WorkerSupportsCancellation = false
            };
            _worker.DoWork += Worker_DoWork;
            _worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            CommandManager.InvalidateRequerySuggested();
            _worker.RunWorkerAsync(_selectedModule);
        }

        private void AppendInfoStartWriting()
        {
            string s = string.Format("Started writing to Board {0} Module {1}, {2}, values: {3}, repeat: {4}, frequency: {5}",
                                _selectedModule.BoardIndex, _selectedModule.ModuleIndex,
                                _selectedValueGenerator.GeneratorName,
                                _selectedModule.NumberOfValues, _selectedModule.RepeatBufferCount,
                                AdqCommon.FormatFrequency(_selectedModule.ActualOutputFrequency));
            AppendInfoText(AdqCommon.FormatOutputText(s));
        }

        private void AppendInfoValuesWritten(uint adqError)
        {
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                AppendInfoText(AdqCommon.FormatOutputText("Values written to the internal data buffer"));
            }
            else
            {
                AppendInfoText(AdqCommon.FormatOutputText(string.Format("Error: Function adqWriteBuffer returned error number: 0x{0:X}", adqError)));
            }
        }

        private void AppendInfoOutputEnded()
        { 
            AppendInfoText(AdqCommon.FormatOutputText(string.Format("Finished writing values, number of values written: {0}",
                                _selectedModule.NumberOfValuesWritten)));
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            DigitalModule adqModule = e.Argument as DigitalModule;
            adqModule.WriteBuffer();

            e.Result = adqModule;
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (_isAppClosing) return;

            AppendInfoValuesWritten(_selectedModule.AdqError);
            _dispatcherTimer.Start();
        }

        private void DispatcherTimer_Tick(object sender, EventArgs e)
        {
            CheckModuleStatus();
        }

        private void CheckModuleStatus()
        {
            bool isModuleRunning = _selectedModule.GetModuleStatus();
            if (!isModuleRunning)
            {
                _dispatcherTimer.Stop();
                IsModuleIdle = true;
                AppendInfoOutputEnded();
                CommandManager.InvalidateRequerySuggested();
            }
        }

        void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        void MainViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(ActualOutputFrequency))
            {
                RaisePropertyChanged(nameof(ActualOutputFrequency));
                RaisePropertyChanged(nameof(OutputTimeFormatted));
            }
            else if (e.PropertyName == nameof(NumberOfValues))
            {
                RaisePropertyChanged(nameof(OutputTimeFormatted));
            }
            else if (e.PropertyName == nameof(RepeatBufferCount))
            {
                RaisePropertyChanged(nameof(OutputTimeFormatted));
            }
            else if (e.PropertyName == nameof(ModuleLocator.InfoText))
            {
                AppendInfoText(_moduleLocator.InfoText);
            }
            else if (e.PropertyName == nameof(DigitalModule.ModuleInfoText))
            {
                AppendInfoText(_selectedModule.ModuleInfoText);
            }
        }
    }
}
