using System.Runtime.InteropServices;
/// <summary>
/// This namespace has defines for constants and function prototypes used by the ALLDAQ Utility library.
/// Native import library:
/// - 32bit: adqUtility.dll
/// - 64bit: adqUtility_64.dll
/// </summary>
namespace ALLDAQ_DLL.Utilities
{
    /// <summary>
    /// Import function prototypes, error codes returned by the API functions.
    /// </summary>
    public sealed class AdqUtility
    {
#if WIN64
        public const string ADQUTILITY_DLL = "adqUtility_64.dll";
#else
        public const string ADQUTILITY_DLL = "adqUtility.dll";
#endif

#pragma warning disable IDE1006 // Naming Styles
        /// <summary>
        /// Calculate the RMS value. The buffer can contain values of several channels interleaved.
        /// </summary>
        /// <param name="buffer">Buffer with values [in].</param>
        /// <param name="offset">Offset of the first value in buffer [in].</param>
        /// <param name="step">Step to the next value of the same channel [in].</param>
        /// <param name="numberOfValues">Number of values of the channel [in].</param>
        /// <param name="digitalMin">Digital minimum value [in].</param>
        /// <param name="digitalMax">Digital maximum value [in].</param>
        /// <param name="physicalMin">Physical minimum value of the signal [in].</param>
        /// <param name="physicalMax">Physical maximum value of the signal [in].</param>
        /// <param name="RMSValue">The calculated RMS value [out].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityCalculateRMSValue(
            uint[] buffer,
            uint offset,
            uint step,
            uint numberOfValues,
            uint digitalMin,
            uint digitalMax,
            double physicalMin,
            double physicalMax,
            out double RMSValue
        );


        /// <summary>
        /// Calculate minimum, maximum, average, and P2P value. The buffer can contain values of several channels interleaved.
        /// </summary>
        /// <param name="buffer">Buffer with values [in].</param>
        /// <param name="offset">Offset of the first value in buffer [in].</param>
        /// <param name="step">Step to the next value of the same channel [in].</param>
        /// <param name="numberOfValues">Number of values of the channel [in].</param>
        /// <param name="digitalMin">Digital minimum value [in].</param>
        /// <param name="digitalMax">Digital maximum value [in].</param>
        /// <param name="physicalMin">Physical minimum value of the signal [in].</param>
        /// <param name="physicalMax">Physical maximum value of the signal [in].</param>
        /// <param name="minValue">The minimum value of the signal in physical units [out].</param>
        /// <param name="maxValue">The maximum value of the signal in physical units [out].</param>
        /// <param name="averageValue">The arithmetic mean value of the signal in physical units [out].</param>
        /// <param name="P2PValue">The peak to peak value of the signal, the difference of maxValue and minValue [out].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityCalculateMinMaxAverageAndP2PValue(
            uint[] buffer,
            uint offset,
            uint step,
            uint numberOfValues,
            uint digitalMin,
            uint digitalMax,
            double physicalMin,
            double physicalMax,
            out double minValue,
            out double maxValue,
            out double averageValue,
            out double P2PValue
        );


        /// <summary>
        /// Calculate the frequency spectrum of an input signal. The buffer can contain values of several channels interleaved.
        /// </summary>
        /// <param name="samplingRateHz">Sampling rate of the acquired values in Hertz [in].</param>
        /// <param name="buffer">Buffer with values [in].</param>
        /// <param name="offset">Offset of the first value in buffer [in].</param>
        /// <param name="step">Step to the next value of the same channel [in].</param>
        /// <param name="numberOfValues">Number of values of the channel [in].</param>
        /// <param name="digitalMin">Digital minimum value [in].</param>
        /// <param name="digitalMax">Digital maximum value [in].</param>
        /// <param name="physicalMin">Physical minimum value of the signal [in].</param>
        /// <param name="physicalMax">Physical maximum value of the signal [in].</param>
        /// <param name="frequencySpectrum">Array of frequencies [out].</param>
        /// <param name="frequencyStep">The frequency step of the frequency spectrum. Steps always start with 0 [out].</param>
        /// <param name="frequencySpectrumSize">The number of values of the array frequencySpectrum [in].</param>
        /// <param name="windowFunction">One of the defined constants FFT_WINDOW_FUNCTION [in].</param>
        /// <param name="overlap">The amount of overlapping of the window for the following calculation. The value can be from 0 to 0.98 [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFrequencySpectrum(
            double samplingRateHz,
            uint[] buffer,
            uint offset,
            uint step,
            uint numberOfValues,
            uint digitalMin,
            uint digitalMax,
            double physicalMin,
            double physicalMax,
            double[] frequencySpectrum,
            out double frequencyStep,
            uint frequencySpectrumSize,
            uint windowFunction,
            double overlap
        );


        /// <summary>
        /// Calculate the frequency spectrum of a known sine signal. The buffer can contain values of several channels interleaved.
        /// </summary>
        /// <param name="samplingRateHz">Sampling rate of the acquired values in Hertz [in].</param>
        /// <param name="sinusFrequencyHz">The value of the known sine frequency in Hertz [in].</param>
        /// <param name="buffer">Buffer with values [in].</param>
        /// <param name="offset">Offset of the first value in buffer [in].</param>
        /// <param name="step">Step to the next value of the same channel [in].</param>
        /// <param name="numberOfValues">Number of values of the channel [in].</param>
        /// <param name="digitalMin">Digital minimum value [in].</param>
        /// <param name="digitalMax">Digital maximum value [in].</param>
        /// <param name="physicalMin">Physical minimum value of the signal [in].</param>
        /// <param name="physicalMax">Physical maximum value of the signal [in].</param>
        /// <param name="frequencySpectrum">Array of frequencies [out].</param>
        /// <param name="frequencyStep">The frequency step of the frequency spectrum. Steps always start with 0 [out].</param>
        /// <param name="frequencySpectrumSize">The number of values of the array frequencySpectrum [in].</param>
        /// <param name="windowFunction">One of the defined constants FFT_WINDOW_FUNCTION [in].</param>
        /// <param name="overlap">The amount of overlapping of the window for the following calculation. The value can be from 0 to 0.98 [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFrequencySpectrumCoherentSampling(
            double samplingRateHz,
            double sinusFrequencyHz,
            uint[] buffer,
            uint offset,
            uint step,
            uint numberOfValues,
            uint digitalMin,
            uint digitalMax,
            double physicalMin,
            double physicalMax,
            double[] frequencySpectrum,
            out double frequencyStep,
            uint frequencySpectrumSize,
            uint windowFunction,
            double overlap
        );


        /// <summary>
        /// Calculate characteristic signal values of a given frequency spectrum (main frequency, THD, THDr+N, THDf+N).
        /// </summary>
        /// <param name="frequencySpectrum">Array of Frequencies, calculated with function adqUtilityFrequencySpectrum [in].</param>
        /// <param name="frequencyStep">The frequency step of the frequency spectrum [in].</param>
        /// <param name="frequencySpectrumSize">Size of the array frequencySpectrum in values [in].</param>
        /// <param name="frequencyIgnoreBelow">Filter lower frequencies up to this value [in].</param>
        /// <param name="frequencyIgnoreAbove">Filter higher frequencies from this value [in].</param>
        /// <param name="mainFrequencyHz">The main frequency of the signal in Hertz [out].</param>
        /// <param name="proportionMainFrequency">The proportion of the main frequency as normalized value from 0 to 1.0 [out].</param>
        /// <param name="THD">The total harmonic distortion [out].</param>
        /// <param name="THDrPlusNoise">The total harmonic distortion + noise (root mean square) [out].</param>
        /// <param name="THDfPlusNoise">The total harmonic distortion + noise (fundamental) [out].</param>
        /// <param name="RMSValue">The calculated RMS value [out].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityCalculateCharacteristicValuesFromFrequencySpectrum(
            double[] frequencySpectrum,
            double frequencyStep,
            uint frequencySpectrumSize,
            double frequencyIgnoreBelow,
            double frequencyIgnoreAbove,
            out double mainFrequencyHz,
            out double proportionMainFrequency,
            out double THD,
            out double THDrPlusNoise,
            out double THDfPlusNoise,
            out double RMSValue
        );


        /// <summary>
        /// Apply a digital filter on acquired values using generated filter coefficients.
        /// </summary>
        /// <param name="buffer">Buffer with values [in, out].</param>
        /// <param name="offset">Offset of the first value in buffer [in].</param>
        /// <param name="step">Step to the next value of the same channel [in].</param>
        /// <param name="numberOfValues">Number of values of the channel [in].</param>
        /// <param name="digitalMin">Digital minimum value [in].</param>
        /// <param name="digitalMax">Digital maximum value [in].</param>
        /// <param name="physicalMin">Physical minimum value of the signal [in].</param>
        /// <param name="physicalMax">Physical maximum value of the signal [in].</param>
        /// <param name="firCoefficients">Array of filter coefficients generated with one of the four filter functions [in].</param>
        /// <param name="firOrder">The number of coefficients in firCoefficients [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIR(
            uint[] buffer,
            uint offset,
            uint step,
            uint numberOfValues,
            uint digitalMin,
            uint digitalMax,
            double physicalMin,
            double physicalMax,
            double[] firCoefficients,
            uint firOrder
        );


        /// <summary>
        /// Generate filter coefficients for a high pass filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterOutToHz">The frequency of the high pass filter in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateHighPassCoefficients(
            double samplingRateHz,
            double filterOutToHz,
            double[] filterCoefficients,
            uint filterOrder
        );


        /// <summary>
        /// Generate filter coefficients for a low pass filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterOutFromHz">The frequency of the low pass filter in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateLowPassCoefficients(
            double samplingRateHz,
            double filterOutFromHz,
            double[] filterCoefficients,
            uint filterOrder
        );


        /// <summary>
        /// Generate filter coefficients for a band-pass filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterOutToHz">The lower frequency of the band-pass filter in Hertz [in].</param>
        /// <param name="filterOutFromHz">The upper frequency of the band-pass filter in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateBandPassCoefficients(
            double samplingRateHz,
            double filterOutToHz,
            double filterOutFromHz,
            double[] filterCoefficients,
            uint filterOrder
        );


        /// <summary>
        /// Generate filter coefficients for a band-stop filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterOutFromHz">The lower frequency of the band-stop filter in Hertz [in].</param>
        /// <param name="filterOutToHz">The upper frequency of the band-stop filter in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateBandStopCoefficients(
            double samplingRateHz,
            double filterOutFromHz,
            double filterOutToHz,
            double[] filterCoefficients,
            uint filterOrder
        );


        /// <summary>
        /// Generate filter coefficients for an A-weighted filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateAWeightedCoefficients(
            double samplingRateHz,
            double[] filterCoefficients,
            uint filterOrder
        );


        /// <summary>
        /// Generate filter coefficients for a ITUR-468 weighted filter.
        /// </summary>
        /// <param name="samplingRateHz">The sampling rate of the signal in Hertz [in].</param>
        /// <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out].</param>
        /// <param name="filterOrder">The number of coefficients to generate [in].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityFilterFIRGenerateITUR468WeightedCoefficients(
            double samplingRateHz,
            double[] filterCoefficients,
            uint filterOrder
        );

        /// <summary>
        /// Calculate the ratio in decibel for a linear input value as 20 * log(linearValue / referenceValue).
        /// </summary>
        /// <param name="referenceValue">Reference value, e.g. for dBu 0.775, for dbV 1.0 [in].</param>
        /// <param name="linearValue">The linear value, e.g. calculated by one of the FFT functions [in].</param>
        /// <param name="dBuValue">The calculated value in decibel [out].</param>
        /// <returns>Error code.</returns>
        [DllImport(ADQUTILITY_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqUtilityConvertLinearTodBu(
            double referenceValue,
            double linearValue,
            out double dBuValue
        );

#pragma warning restore IDE1006 // Naming Styles
    }
    /// <summary>
    /// Constants used by the API.
    /// </summary>
    public sealed class AdqUtilityConstants
    {
        /// <summary>
        /// FFT window function rectangular.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_RECTANGULAR = 0;
        /// <summary>
        /// FFT window function hann.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_HANN = 1;
        /// <summary>
        /// FFT window function hamming.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_HAMMING = 2;
        /// <summary>
        /// FFT window function bartlett.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_BARTLETT = 3;
        /// <summary>
        /// FFT window function flat top.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_FLAT_TOP = 4;
        /// <summary>
        /// FFT window function harris nutall.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_HARRIS_NUTALL = 5;
        /// <summary>
        /// FFT window function blackman harris 7 term.
        /// </summary>
        public const uint FFT_WINDOW_FUNCTION_BLACKMAN_HARRIS_7_TERM = 6;
    }
    /// <summary>
    /// Error codes returned by the API functions.
    /// </summary>
    public sealed class AdqUtilityErrors
    {
        /// <summary>
        /// Error none.
        /// </summary>
        public const uint ADQ_UTILITY_ERROR_NONE = 0;
        /// <summary>
        /// Error digital limits invalid.
        /// </summary>
        public const uint ADQ_UTILITY_ERROR_DIGITAL_LIMITS_INVALID = 1;
        /// <summary>
        /// Error invalid parameter 1.
        /// </summary>
        public const uint ADQ_UTILITY_ERROR_INVALID_PARAMETER_1 = 100;
        /// <summary>
        /// Error invalid parameter 2.
        /// </summary>
        public const uint ADQ_UTILITY_ERROR_INVALID_PARAMETER_2 = 101;
    }
}

