﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ADQInfo project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports ADQInfo.ALLDAQ.Boards

Public Class FormMain
    Private Const _picoSecondsPerSecond As Double = 1000000000000.0
    Dim _adqOpenError As UInteger = AdqCommon.NotOpen

    Private Sub FormMain_Shown(sender As Object, e As EventArgs) Handles MyBase.Shown
        Try
            Dim trNode = New TreeNode("ALLDAQ hardware")
            tvInfo.Nodes.Add(trNode)

            _adqOpenError = OpenAdqDriver()
            If (_adqOpenError <> AdqErrors.ADQ_ERROR_NONE) Then
                Return
            End If

            GetSystemInfo(trNode)
            tvInfo.ExpandAll()
            tvInfo.Nodes(0).EnsureVisible()

            CloseAdqDriver()
        Catch ex As DllNotFoundException
            Dim trNode As TreeNode
            trNode = New TreeNode(String.Format("DllNotFoundException: {0}", ex.Message))
            tvInfo.Nodes.Add(trNode)
        End Try
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            Dim trNode As TreeNode = New TreeNode(FormatErrorText("adqOpen", adqError))
            tvInfo.Nodes.Add(trNode)
        End If
        Return adqError
    End Function

    Private Sub GetSystemInfo(parentTreeNode As TreeNode)
        Dim trNode As TreeNode
        Dim systemInfo = New AdqDriver.ADQSystemInfo(AdqDriver.ADQSystemInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetSystemInfo(systemInfo)
        If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
            trNode = New TreeNode("Library version: " & AdqCommon.FormatVersion(systemInfo.DLLVersion))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Number of boards: {0}", systemInfo.NumberOfBoards))
            parentTreeNode.Nodes.Add(trNode)

            GetBoardInfo(systemInfo.NumberOfBoards, parentTreeNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetSystemInfo", adqError)
        End If
    End Sub

    Private Sub GetBoardInfo(numberOfBoards As UInteger, parentTreeNode As TreeNode)
        Dim boardNode As TreeNode
        Dim trNode As TreeNode
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)
        Dim boardIdx As UInteger = 0
        While boardIdx < numberOfBoards
            Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIdx, boardInfo)
            If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
                Dim model As String = AdqCommon.GetModelName(boardInfo.BoardName)
                boardNode = New TreeNode(
                String.Format("Board number: {0} - {1}", boardIdx, model))
                parentTreeNode.Nodes.Add(boardNode)

                trNode = New TreeNode(
                    String.Format("Name: {0}", boardInfo.BoardName))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Vendor ID: &H{0:X4}", boardInfo.VendorID))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Device ID: &H{0:X4}", boardInfo.DeviceID))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Serial number: &H{0:X8}", boardInfo.SerialNumber))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Firmware version: {0}", boardInfo.FirmwareVersion))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Description: {0}", boardInfo.BoardDescription))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode(
                    String.Format("Driver name: {0}", boardInfo.DriverName))
                boardNode.Nodes.Add(trNode)

                trNode = New TreeNode("Driver version: " & AdqCommon.FormatVersion(boardInfo.DriverVersion))
                boardNode.Nodes.Add(trNode)

                If (boardInfo.TTLVoltageLevel > 0) Then
                    AddTTLVoltageLevels(boardNode, boardInfo.TTLVoltageLevel)
                    AddCurrentTTLVoltageLevel(boardIdx, boardNode)
                End If

                trNode = New TreeNode("Bus type: " & AdqCommon.GetBusTypeName(boardInfo.BusType))
                boardNode.Nodes.Add(trNode)

                If boardInfo.BusType = AdqConstants.ADQ_BUS_TYPE_PCI Then
                    AddBoardPCIInfo(boardIdx, trNode)
                End If

                trNode = New TreeNode(
                    String.Format("Installed location: {0}", If(boardInfo.Remote, "Remote", "Local")))
                boardNode.Nodes.Add(trNode)

                If (boardInfo.Remote) Then
                    trNode = New TreeNode(
                        String.Format("Remote IP address: {0}", boardInfo.RemoteIPAddressOrHostName))
                    boardNode.Nodes.Add(trNode)
                End If

                trNode = New TreeNode(
                    String.Format("Number of modules: {0}", boardInfo.NumberOfModules))
                boardNode.Nodes.Add(trNode)

                GetModuleInfo(boardIdx, boardInfo.NumberOfModules, boardNode)
            Else
                trNode = New TreeNode(String.Format("Board {0}: ", boardIdx) & FormatErrorText("adqGetBoardInfo", adqError))
                parentTreeNode.Nodes.Add(trNode)
            End If

            boardIdx += 1UI
        End While
    End Sub

    Private Sub AddTTLVoltageLevels(parentTreeNode As TreeNode, voltageLevels As UInteger)
        Dim voltageLevelsTreeNode = New TreeNode("TTL voltage levels")
        parentTreeNode.Nodes.Add(voltageLevelsTreeNode)

        AddTTLVoltageLevelTreeNode(voltageLevelsTreeNode, voltageLevels, AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS)
        AddTTLVoltageLevelTreeNode(voltageLevelsTreeNode, voltageLevels, AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS)
    End Sub

    Private Sub AddTTLVoltageLevelTreeNode(parentTreeNode As TreeNode, voltageLevels As UInteger, voltageConstant As UInteger)
        Dim trNode As TreeNode
        If (voltageLevels And voltageConstant) = voltageConstant Then
            trNode = New TreeNode(AdqCommon.GetTTLVoltageLevelText(voltageConstant))
            parentTreeNode.Nodes.Add(trNode)
        End If
    End Sub

    Private Sub AddCurrentTTLVoltageLevel(boardIndex As UInteger, parentTreeNode As TreeNode)
        Dim voltage As UInteger
        Dim trNode As TreeNode
        Dim adqError As UInteger = AdqDriver.adqGetBoardTTLVoltage(boardIndex, voltage)
        If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
            trNode = New TreeNode(
                String.Format("Current TTL voltage level: {0}", AdqCommon.GetTTLVoltageLevelText(voltage)))
            parentTreeNode.Nodes.Add(trNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetBoardTTLVoltage", adqError)
        End If
    End Sub

    Private Sub AddBoardPCIInfo(boardIndex As UInteger, parentTreeNode As TreeNode)
        Dim trNode As TreeNode
        Dim boardPciInfo = New AdqDriver.ADQBoardPCIInfo(AdqDriver.ADQBoardPCIInfo_ID)
        Dim adqError As UInteger = AdqDriver.adqGetBoardPCIInfo(boardIndex, boardPciInfo)
        If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
            trNode = New TreeNode(String.Format("PCI Bus number: {0}", boardPciInfo.BusNumber))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(String.Format("PCI Device number: {0}", boardPciInfo.DeviceNumber))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(String.Format("PCI Function number: {0}", boardPciInfo.FunctionNumber))
            parentTreeNode.Nodes.Add(trNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetBoardPCIInfo", adqError)
        End If
    End Sub

    Private Sub GetModuleInfo(boardIndex As UInteger, numberOfModules As UInteger, parentTreeNode As TreeNode)
        Dim moduleNode As TreeNode
        Dim moduleType As UInteger
        Dim moduleIdx As UInteger = 0
        While moduleIdx < numberOfModules
            moduleNode = New TreeNode(
                String.Format("Module number: {0}", moduleIdx))
            parentTreeNode.Nodes.Add(moduleNode)

            Dim adqError As UInteger = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
                Dim moduleTypeName As String = AdqCommon.GetModuleTypeName(moduleType)
                moduleNode.Text &= " - " + moduleTypeName

                Dim trNode As TreeNode = New TreeNode(
                    String.Format("Module type: {0}", moduleTypeName))
                moduleNode.Nodes.Add(trNode)

                If (moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO) Then
                    GetDIOModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_DO) Then
                    GetDOModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_DI) Then
                    GetDIModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_CNTR) Then
                    GetCounterModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_EXTINT) Then
                    GetExternalIrqInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_AI) Then
                    GetAIModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_AO) Then
                    GetAOModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_I2C) Then
                    GetI2CModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_PWM) Then
                    GetPWMModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_FIN) Then
                    GetFINModuleInfo(boardIndex, moduleIdx, moduleNode)
                ElseIf (moduleType = AdqConstants.ADQ_MODULE_TYPE_I2SO) Then
                    GetI2SOModuleInfo(boardIndex, moduleIdx, moduleNode)
                End If
            Else
                AddErrorTreeNode(moduleNode, "adqGetModuleType", adqError)
            End If

            moduleIdx += 1UI
        End While
    End Sub

    Private Sub GetDIOModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDioModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
            AddDigitalModuleCommonAttributes(parentTreeNode, adqDioModuleInfo.PortWidthBits, adqDioModuleInfo.IsOptoIsolated, adqDioModuleInfo.IsTTL)

            AddCurrentDioDirection(boardIndex, moduleIndex, parentTreeNode)
            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddMaximumSampleFrequency(boardIndex, moduleIndex, adqGeneralModuleInfo.ClockType, parentTreeNode)
            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub AddCurrentDioDirection(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim dioDirection As UInteger
        Dim adqError = AdqDriver.adqDIOGetDirection(boardIndex, moduleIndex, dioDirection)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim childNode = New TreeNode(String.Format("Current DIO direction: {0}", AdqCommon.GetDioDirectionText(dioDirection)))
            parentTreeNode.Nodes.Add(childNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqDIOGetDirection", adqError)
        End If
    End Sub

    Private Sub GetDOModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDoModuleInfo = New AdqDriver.ADQDOModuleInfo(AdqDriver.ADQDOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDoModuleInfo.Size))
        Marshal.StructureToPtr(adqDoModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDoModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDoModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDOModuleInfo)), AdqDriver.ADQDOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
            Debug.Assert(adqDoModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDoModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
            AddDigitalModuleCommonAttributes(parentTreeNode, adqDoModuleInfo.PortWidthBits, adqDoModuleInfo.IsOptoIsolated, adqDoModuleInfo.IsTTL)

            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            If (adqGeneralModuleInfo.IsIRQSource) Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub GetDIModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDiModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
            AddDigitalModuleCommonAttributes(parentTreeNode, adqDiModuleInfo.PortWidthBits, adqDiModuleInfo.IsOptoIsolated, adqDiModuleInfo.IsTTL)

            Dim trNode As TreeNode = New TreeNode(
                String.Format("Number of digital filters: {0}", adqDiModuleInfo.NumberOfFilters))
            parentTreeNode.Nodes.Add(trNode)

            If adqDiModuleInfo.NumberOfFilters > 0 Then
                AddDigitalFilters(parentTreeNode, adqDiModuleInfo)
                AddCurrentDigitalFilter(boardIndex, moduleIndex, parentTreeNode)
            End If

            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddMaximumSampleFrequency(boardIndex, moduleIndex, adqGeneralModuleInfo.ClockType, parentTreeNode)
            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub AddDigitalModuleCommonAttributes(parentTreeNode As TreeNode, portWidthBits As UInteger, isOptoIsolated As Boolean, isTTL As Boolean)
        Dim treeNode = New TreeNode(
                String.Format("Port width: {0} Bit", portWidthBits))
        parentTreeNode.Nodes.Add(treeNode)

        treeNode = New TreeNode(
                String.Format("Is optoisolated: {0}", isOptoIsolated))
        parentTreeNode.Nodes.Add(treeNode)

        treeNode = New TreeNode(
                String.Format("Is TTL: {0}", isTTL))
        parentTreeNode.Nodes.Add(treeNode)
    End Sub

    Private Sub AddDigitalFilters(parentTreeNode As TreeNode, ByRef diModuleInfo As AdqDriver.ADQDIModuleInfo)
        Dim filtersTreeNode = New TreeNode("Digital filters")
        parentTreeNode.Nodes.Add(filtersTreeNode)

        Dim trNode As TreeNode
        Dim filterIdx As Integer = 0
        While filterIdx < diModuleInfo.NumberOfFilters
            trNode = New TreeNode(
                String.Format("Filter {0}: {1}", filterIdx,
                              AdqCommon.FormatDoubleValue(diModuleInfo.FilterTimePicoseconds(filterIdx) / _picoSecondsPerSecond, "s")))
            filtersTreeNode.Nodes.Add(trNode)
            filterIdx += 1
        End While
    End Sub

    Private Sub AddCurrentDigitalFilter(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim filterValue As ULong
        Dim trNode As TreeNode
        Dim adqError As UInteger = AdqDriver.adqDIOGetFilterPicoseconds(boardIndex, moduleIndex, filterValue)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            trNode = New TreeNode(
                String.Format("Current filter value: {0}", AdqCommon.FormatDoubleValue(filterValue / _picoSecondsPerSecond, "s")))
            parentTreeNode.Nodes.Add(trNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqDIOGetFilterPicoseconds", adqError)
        End If
    End Sub

    Private Sub AddGeneralModuleInfo(parentTreeNode As TreeNode, adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo)
        Dim trNode As TreeNode
        trNode = New TreeNode(
            String.Format("Description: {0}", adqGeneralModuleInfo.ModuleDescription))
        parentTreeNode.Nodes.Add(trNode)

        trNode = New TreeNode(
            String.Format("Clock type: {0}", AdqCommon.GetClockTypeName(adqGeneralModuleInfo.ClockType)))
        parentTreeNode.Nodes.Add(trNode)

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, "The ClockMinimumDivider of the module is zero")

            trNode = New TreeNode(
                    String.Format("Base frequency: {0}", AdqCommon.FormatDoubleValue(adqGeneralModuleInfo.ClockBaseFrequencyHz, "Hz")))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Clock minimum divider: {0}", adqGeneralModuleInfo.ClockMinimumDivider))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(String.Format("Maximum frequency: {0}",
                AdqCommon.FormatDoubleValue((adqGeneralModuleInfo.ClockBaseFrequencyHz / adqGeneralModuleInfo.ClockMinimumDivider), "Hz")))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Internal data buffer size: {0} values", adqGeneralModuleInfo.InternalDataBufferSizeValues))
            parentTreeNode.Nodes.Add(trNode)
        End If

        trNode = New TreeNode(
                String.Format("Is interrupt source: {0}", adqGeneralModuleInfo.IsIRQSource))
        parentTreeNode.Nodes.Add(trNode)
    End Sub

    Private Sub AddInterruptInfo(boardIndex As UInteger, moduleIndex As UInteger, parentNode As TreeNode)
        Dim trNode As TreeNode
        Dim adqIrqInfo = New AdqDriver.ADQIRQInfo(AdqDriver.ADQIRQInfo_ID)
        Dim adqError = AdqDriver.adqGetIRQInfo(boardIndex, moduleIndex, adqIrqInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim irqInfoTreeNode As TreeNode = New TreeNode("Interrupt Info")
            parentNode.Nodes.Add(irqInfoTreeNode)

            trNode = New TreeNode(
                String.Format("Description: {0}", adqIrqInfo.IRQDescription))
            irqInfoTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Supports bit-change: {0}", adqIrqInfo.SupportsBitChange))
            irqInfoTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Supports bit-pattern compare: {0}", adqIrqInfo.SupportsBitPatternCompare))
            irqInfoTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Port status error: &H{0:X8}", adqIrqInfo.SupportsPortStatusError))
            irqInfoTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Fault recovery: &H{0:X8}", adqIrqInfo.SupportsFaultRecovery))
            irqInfoTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("External interrupt: &H{0:X8}", adqIrqInfo.SupportsExternal))
            irqInfoTreeNode.Nodes.Add(trNode)
        Else
            AddErrorTreeNode(parentNode, "adqGetIRQInfo", adqError)
        End If
    End Sub

    Private Sub GetCounterModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim trNode As TreeNode
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqCntrModuleInfo = New AdqDriver.ADQCNTRModuleInfo(AdqDriver.ADQCNTRModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqCntrModuleInfo.Size))
        Marshal.StructureToPtr(adqCntrModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqCntrModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqCntrModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQCNTRModuleInfo)), AdqDriver.ADQCNTRModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqCntrModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqCntrModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)

            trNode = New TreeNode(
                String.Format("Counter type: {0}", AdqCommon.GetCounterTypeName(adqCntrModuleInfo.CounterType)))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Counter width: {0} Bit, max. start value: {1}, hex &H{1:X}",
                              adqCntrModuleInfo.WidthBits, GetCounterMaxStartValue(adqCntrModuleInfo.WidthBits)))
            parentTreeNode.Nodes.Add(trNode)

            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If

            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddMaximumSampleFrequency(boardIndex, moduleIndex, adqGeneralModuleInfo.ClockType, parentTreeNode)
            AddCounterClockSources(parentTreeNode, adqCntrModuleInfo.ClockSources)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub AddMaximumSampleFrequency(boardIndex As UInteger, moduleIndex As UInteger, clockType As UInteger, parentTreeNode As TreeNode)
        If Not IsClockControlledModule(clockType) Then Return

        Dim maxSampleFrequency As Double
        Dim adqError As UInteger = AdqDriver.adqGetMaximumSampleFrequency(boardIndex, moduleIndex, 1, maxSampleFrequency)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim childNode = New TreeNode(String.Format("Maximum sample frequency: {0}", AdqCommon.FormatDoubleValue(maxSampleFrequency, "Hz")))
            parentTreeNode.Nodes.Add(childNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetMaximumSampleFrequency", adqError)
        End If
    End Sub

    Private Function GetCounterMaxStartValue(widthBits As UInteger) As UInteger
        Return CUInt((1UL << CInt(widthBits)) - 1UL)
    End Function

    Private Function IsClockControlledModule(clockType As UInteger) As Boolean
        Return clockType = AdqConstants.ADQ_CLOCK_TYPE_HARDWARE OrElse clockType = AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE
    End Function

    Private Sub AddCounterClockSources(parentTreeNode As TreeNode, clockSources As UInteger)
        Dim clockSourcesTreeNode = New TreeNode("Counter clock sources")
        parentTreeNode.Nodes.Add(clockSourcesTreeNode)

        AddCounterClockSourceTreeNode(clockSourcesTreeNode, clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)
        AddCounterClockSourceTreeNode(clockSourcesTreeNode, clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)
        AddCounterClockSourceTreeNode(clockSourcesTreeNode, clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ)
    End Sub

    Private Sub AddCounterClockSourceTreeNode(parentTreeNode As TreeNode, clockSources As UInteger, clockSourceConstant As UInteger)
        If (clockSources And clockSourceConstant) = clockSourceConstant Then
            Dim trNode As TreeNode = New TreeNode(AdqCommon.GetCounterClockSourceName(clockSourceConstant))
            parentTreeNode.Nodes.Add(trNode)
        End If
    End Sub

    Private Sub GetExternalIrqInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)

        Dim infoSize As UInteger
        Dim adqError As UInteger
        adqError = AdqDriver.adqGetModuleInfo(
                                            boardIndex,
                                            moduleIndex,
                                            adqGeneralModuleInfo,
                                            0,
                                            IntPtr.Zero,
                                            infoSize)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub GetAIModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqAInputModuleInfo = New AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqAInputModuleInfo.Size))
        Marshal.StructureToPtr(adqAInputModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqAInputModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqAInputModuleInfo = CType(
            Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQAIModuleInfo)), AdqDriver.ADQAIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqAInputModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqAInputModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)

            If (adqGeneralModuleInfo.IsIRQSource) Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If

            Dim trNode As TreeNode = New TreeNode(
                String.Format("Shared data rate: {0}",
                              If(adqAInputModuleInfo.SharedDataRate, "Yes", "No")))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                String.Format("Number of channels: {0}", adqAInputModuleInfo.NumberOfChannels))
            parentTreeNode.Nodes.Add(trNode)
            AddCurrentHardwareRange(boardIndex, moduleIndex, adqAInputModuleInfo.NumberOfChannels, trNode)
            AddSelectedCalibration(boardIndex, moduleIndex, parentTreeNode)
            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddMaximumSampleFrequencyList(boardIndex, moduleIndex, adqAInputModuleInfo.NumberOfChannels, parentTreeNode)

            If adqAInputModuleInfo.RangeInfoPerChannel Then
                trNode = New TreeNode("Variable ranges per channel encountered." _
                                        & " Please use a newer version of this program.")
                parentTreeNode.Nodes.Add(trNode)
                Return
            End If

            AddAiRangesInfo(parentTreeNode, adqAInputModuleInfo)
        End If
    End Sub

    Private Sub AddSelectedCalibration(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim selectedCalibration As UInteger
        Dim adqError = AdqDriver.adqAnalogGetSelectedCalibration(boardIndex, moduleIndex, selectedCalibration)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim childNode = New TreeNode(String.Format("Selected calibration: {0}", AdqCommon.GetCalibrationText(selectedCalibration)))
            parentTreeNode.Nodes.Add(childNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqAnalogGetSelectedCalibration", adqError)
        End If
    End Sub

    Private Sub AddHardwareTriggerInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim hwTriggerInfo = New AdqDriver.ADQHWTriggerInfo(AdqDriver.ADQHWTriggerInfo_ID)
        Dim adqError As UInteger = AdqDriver.adqGetHWTriggerInfo(boardIndex, moduleIndex, hwTriggerInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim hwTriggerTreeNode = New TreeNode("Hardware trigger info")
            parentTreeNode.Nodes.Add(hwTriggerTreeNode)

            If hwTriggerInfo.HWTriggerStartSource = 0 _
                AndAlso hwTriggerInfo.HWTriggerStopSource = 0 _
                AndAlso Not hwTriggerInfo.SupportsChannelListPerTrigger Then

                hwTriggerTreeNode.Nodes.Add(New TreeNode("No hardware trigger supported"))
                Return
            End If

            Dim trNode = New TreeNode(
                    String.Format("Hardware trigger start source: &H{0:X8}", hwTriggerInfo.HWTriggerStartSource))
            hwTriggerTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Hardware trigger stop source: &H{0:X8}", hwTriggerInfo.HWTriggerStopSource))
            hwTriggerTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Supports channel list per trigger: {0}", If(hwTriggerInfo.SupportsChannelListPerTrigger, "Yes", "No")))
            hwTriggerTreeNode.Nodes.Add(trNode)
        End If
    End Sub

    Private Sub AddMaximumSampleFrequencyList(boardIndex As UInteger, moduleIndex As UInteger, numberOfChannels As UInteger, parentTreeNode As TreeNode)
        Dim trNode = New TreeNode("Maximum sample frequencies")
        parentTreeNode.Nodes.Add(trNode)

        Dim plural As String = String.Empty
        Dim childNode As TreeNode
        Dim numChannels As UInteger = 1
        While numChannels <= numberOfChannels
            Dim maxSampleFrequency As Double
            Dim adqError As UInteger = AdqDriver.adqGetMaximumSampleFrequency(boardIndex, moduleIndex, numChannels, maxSampleFrequency)
            If adqError = AdqErrors.ADQ_ERROR_NONE Then
                childNode = New TreeNode(String.Format("{0} channel{1}: {2}",
                                                       numChannels, plural, AdqCommon.FormatDoubleValue(maxSampleFrequency, "Hz")))
                trNode.Nodes.Add(childNode)
                plural = "s"
            Else
                AddErrorTreeNode(trNode, "adqGetMaximumSampleFrequency", adqError)
                Return
            End If

            numChannels += 1UI
        End While
    End Sub

    Private Sub AddAiRangesInfo(parentTreeNode As TreeNode, adqAiModuleInfo As AdqDriver.ADQAIModuleInfo)
        Dim trNode As TreeNode = New TreeNode(
            String.Format("Number of ranges: {0}", adqAiModuleInfo.NumberOfRanges))
        parentTreeNode.Nodes.Add(trNode)

        Dim aiRange As AdqDriver.ADQAIRange
        Dim rangeIdx As Integer = 0
        While rangeIdx < adqAiModuleInfo.NumberOfRanges
            aiRange = adqAiModuleInfo.Ranges(rangeIdx)
            AddRangeInfo(parentTreeNode, rangeIdx, aiRange.DigitalMin, aiRange.DigitalMax, aiRange.PhysicalUnit, aiRange.PhysicalMin, aiRange.PhysicalMax)
            rangeIdx += 1
        End While
    End Sub

    Private Sub AddCurrentHardwareRange(boardIndex As UInteger, moduleIndex As UInteger, numberOfChannels As UInteger, parentTreeNode As TreeNode)
        Dim rangeIndex As UInteger
        Dim channelIdx As UInteger = 0
        While channelIdx < numberOfChannels
            Dim adqError As UInteger = AdqDriver.adqAIGetChannelHardwareRange(boardIndex, moduleIndex, channelIdx, rangeIndex)
            If adqError = AdqErrors.ADQ_ERROR_NONE Then
                Dim TreeNode = New TreeNode(String.Format("Channel {0} current range: {1}", channelIdx, rangeIndex))
                parentTreeNode.Nodes.Add(TreeNode)
            Else
                'The API-function adqAIGetChannelHardwareRange is only supported for AI modules with safety switching on excessive range overflow, e.g. ADQ-412
                If Not IsErrorUnsuppored(adqError) Then
                    AddErrorTreeNode(parentTreeNode, "adqAIGetChannelHardwareRange", adqError)
                End If
                Return
            End If

            channelIdx += 1UI
        End While
    End Sub

    Private Function IsErrorUnsuppored(adqError As UInteger) As Boolean
        Return adqError = (AdqErrors.ADQ_ERROR_TYPE_ERROR Or AdqErrors.ADQ_ERROR_PARAMETER_2 Or AdqErrors.ADQ_ERROR_UNSUPPORTED)
    End Function

    Private Sub GetAOModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqAoModuleInfo = New AdqDriver.ADQAOModuleInfo(AdqDriver.ADQAOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqAoModuleInfo.Size))
        Marshal.StructureToPtr(adqAoModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqAoModuleInfo.Size,
            moduleInfo,
            infoSize)

        adqAoModuleInfo = CType(
            Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQAOModuleInfo)), AdqDriver.ADQAOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqAoModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqAoModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
            AddSelectedCalibration(boardIndex, moduleIndex, parentTreeNode)

            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If

            AddHardwareTriggerInfo(boardIndex, moduleIndex, parentTreeNode)
            AddAoRangesInfo(parentTreeNode, adqAoModuleInfo)
        End If
    End Sub

    Private Sub AddAoRangesInfo(parentTreeNode As TreeNode, ByRef adqAoModuleInfo As AdqDriver.ADQAOModuleInfo)
        Dim trNode As TreeNode = New TreeNode(
            String.Format("Number of ranges: {0}", adqAoModuleInfo.NumberOfRanges))
        parentTreeNode.Nodes.Add(trNode)

        Dim aoRange As AdqDriver.ADQAORange
        Dim rangeIndex As Integer = 0
        While rangeIndex < adqAoModuleInfo.NumberOfRanges
            aoRange = adqAoModuleInfo.Ranges(rangeIndex)
            AddRangeInfo(parentTreeNode, rangeIndex, aoRange.DigitalMin, aoRange.DigitalMax, aoRange.PhysicalUnit, aoRange.PhysicalMin, aoRange.PhysicalMax)
            rangeIndex += 1
        End While
    End Sub

    Private Sub AddRangeInfo(parentTreeNode As TreeNode,
                            rangeIndex As Integer,
                            digitalMin As UInteger,
                            digitalMax As UInteger,
                            physicalUnit As UInteger,
                            physicalMin As Double,
                            physicalMax As Double)
        Dim rangeTreeNode As TreeNode
        Dim trNode As TreeNode
        Dim unitName As String
        unitName = AdqCommon.GetPhysicalUnitName(physicalUnit)

        rangeTreeNode = New TreeNode(String.Format("Range {0}", rangeIndex))
        parentTreeNode.Nodes.Add(rangeTreeNode)

        trNode = New TreeNode(
            String.Format("Physical unit: {0}", unitName))
        rangeTreeNode.Nodes.Add(trNode)

        trNode = New TreeNode(
            String.Format("Physical range: {0} ... {1} {2}",
                            physicalMin, physicalMax, unitName))
        rangeTreeNode.Nodes.Add(trNode)

        trNode = New TreeNode(
            String.Format("Digital range: {0} ... {1}, hex &H{0:X} ... &H{1:X}",
                            digitalMin, digitalMax))
        rangeTreeNode.Nodes.Add(trNode)

        Dim physResolution As Double = (physicalMax - physicalMin) / (digitalMax - digitalMin)
        trNode = New TreeNode("Physical resolution: " _
                + AdqCommon.FormatDoubleValue(physResolution, AdqCommon.GetPhysicalUnitSymbol(physicalUnit)))
        rangeTreeNode.Nodes.Add(trNode)
    End Sub

    Private Sub GetI2CModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqI2cModuleInfo = New AdqDriver.ADQI2CModuleInfo(AdqDriver.ADQI2CModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqI2cModuleInfo.Size))
        Marshal.StructureToPtr(adqI2cModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqI2cModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqI2cModuleInfo = CType(
            Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQI2CModuleInfo)), AdqDriver.ADQI2CModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqI2cModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqI2cModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)

            Dim trNode As TreeNode = New TreeNode(
                    String.Format("Data transfer rate: {0}", AdqCommon.FormatBitsPerSecond(adqI2cModuleInfo.SpeedBitsPerSecond)))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Supports 7-bit addresses: {0}", If(adqI2cModuleInfo.Supports7BitAddresses, "Yes", "No")))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Supports 10-bit addresses: {0}", If(adqI2cModuleInfo.Supports10BitAddresses, "Yes", "No")))
            parentTreeNode.Nodes.Add(trNode)

            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If
        End If
    End Sub

    Private Sub GetPWMModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqPwmModuleInfo = New AdqDriver.ADQPWMModuleInfo(AdqDriver.ADQPWMModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqPwmModuleInfo.Size))
        Marshal.StructureToPtr(adqPwmModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqPwmModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqPwmModuleInfo = CType(
            Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQPWMModuleInfo)), AdqDriver.ADQPWMModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Debug.Assert(adqPwmModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqPwmModuleInfo.Size, infoSize))

            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)

            Dim trNode As TreeNode = New TreeNode(
                    String.Format("Base frequency: {0}", AdqCommon.FormatDoubleValue(adqPwmModuleInfo.BaseFrequencyHz, "Hz")))
            parentTreeNode.Nodes.Add(trNode)

            trNode = New TreeNode(
                    String.Format("Minimum divider: {0}", adqPwmModuleInfo.MinimumDivider))
            parentTreeNode.Nodes.Add(trNode)

            AddPWMMaximumFrequency(boardIndex, moduleIndex, parentTreeNode)

            If adqGeneralModuleInfo.IsIRQSource Then
                AddInterruptInfo(boardIndex, moduleIndex, parentTreeNode)
            End If
        End If
    End Sub

    Private Sub AddPWMMaximumFrequency(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim maxFrequency As Double
        Dim adqError As UInteger = AdqDriver.adqPWMGetMaximumFrequency(boardIndex, moduleIndex, maxFrequency)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            Dim trNode As TreeNode = New TreeNode(
                    String.Format("Max frequency: {0}", AdqCommon.FormatDoubleValue(maxFrequency, "Hz")))
            parentTreeNode.Nodes.Add(trNode)
        Else
            AddErrorTreeNode(parentTreeNode, "adqPWMGetMaximumFrequency", adqError)
        End If
    End Sub

    Private Sub GetFINModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        GetGeneralModuleInfo(boardIndex, moduleIndex, parentTreeNode)
    End Sub

    Private Sub GetI2SOModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        GetGeneralModuleInfo(boardIndex, moduleIndex, parentTreeNode)
    End Sub

    Private Sub GetGeneralModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, parentTreeNode As TreeNode)
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, 0, IntPtr.Zero, infoSize)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            AddGeneralModuleInfo(parentTreeNode, adqGeneralModuleInfo)
        Else
            AddErrorTreeNode(parentTreeNode, "adqGetModuleInfo", adqError)
        End If
    End Sub

    Private Sub AddErrorTreeNode(parentTreeNode As TreeNode, functionName As String, adqError As UInteger)
        parentTreeNode.Nodes.Add(New TreeNode(FormatErrorText(functionName, adqError)))
    End Sub

    Private Function FormatErrorText(functionName As String, adqError As UInteger) As String
        Return String.Format("API-function {0} returned error number &H{1:X}", functionName, adqError)
    End Function

    Private Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Sub FormMain_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        CloseAdqDriver()
    End Sub

    Private Sub BtnClose_Click(sender As Object, e As EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub BtnInfo_Click(sender As Object, e As EventArgs) Handles btnInfo.Click
        AboutBox.ShowDialog()
    End Sub

    Private Sub CopyToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles CopyToolStripMenuItem.Click
        If tvInfo.SelectedNode Is Nothing Then Return

        Clipboard.SetText(tvInfo.SelectedNode.Text)
    End Sub

    Private Sub TvInfo_MouseUp(sender As Object, e As MouseEventArgs) Handles tvInfo.MouseUp
        If e.Button <> MouseButtons.Right Then Return

        Dim clickedNode = tvInfo.GetNodeAt(e.X, e.Y)
        If clickedNode Is Nothing Then Return

        tvInfo.SelectedNode = clickedNode
        ContextMenuStrip1.Show(tvInfo, New Point(e.X, e.Y))
    End Sub
End Class

