﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports AISingle.ALLDAQ.Boards

Public Class AnalogInputModule
    Private Const _notInitialized As UInteger = UInteger.MaxValue
    Private _selectedRangeIndex As UInteger = _notInitialized
    Private _selectedChannelIndex As UInteger = _notInitialized
    Private ReadOnly _ranges As List(Of AnalogRange) = New List(Of AnalogRange)

    Public ReadOnly Property NumberOfChannels As UInteger
    Public ReadOnly Property Description As String
    Public Event InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalModuleInfo As AdqDriver.ADQGeneralModuleInfo, ByRef aiModuleInfo As AdqDriver.ADQAIModuleInfo)
        Factor = 1.0
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        NumberOfChannels = aiModuleInfo.NumberOfChannels
        NumberOfRanges = aiModuleInfo.NumberOfRanges
        Description = generalModuleInfo.ModuleDescription
        BuildModuleName(boardName)
        AddRanges(aiModuleInfo)
    End Sub

    Private Sub AddRanges(ByRef aiModuleInfo As AdqDriver.ADQAIModuleInfo)
        Dim rangeIdx As Integer = 0
        While rangeIdx < aiModuleInfo.NumberOfRanges
            Dim range As AdqDriver.ADQAIRange = aiModuleInfo.Ranges(rangeIdx)

            Dim physRange = New AnalogRange(
                range.PhysicalUnit,
                range.DigitalMin,
                range.DigitalMax,
                range.PhysicalMin,
                range.PhysicalMax)
            _ranges.Add(physRange)
            rangeIdx += 1
        End While
    End Sub

    Private _moduleName As String
    Public ReadOnly Property ModuleName As String
        Get
            Return _moduleName
        End Get
    End Property

    ReadOnly Property BoardIndex As UInteger
    ReadOnly Property ModuleIndex As UInteger

    Public Function GetRange(index As UInteger) As AnalogRange
        Return _ranges(CInt(index))
    End Function

    Private _selectedRange As AnalogRange
    ReadOnly Property SelectedRange() As AnalogRange
        Get
            Return _selectedRange
        End Get
    End Property

    ReadOnly Property ChannelIndex() As UInteger
        Get
            Return _selectedChannelIndex
        End Get
    End Property

    Public Property Factor As Double
    Public Property Offset As Double

    Public Function SetChannelAndRange(channelIndex As UInteger, rangeIndex As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        If (IsChannelChanged(channelIndex) OrElse IsRangeChanged(rangeIndex)) Then
            adqError = AdqDriver.adqAISetChannelAndRange(BoardIndex, ModuleIndex, channelIndex, rangeIndex)
            If (adqError = AdqErrors.ADQ_ERROR_NONE) Then
                SetCurrentChannel(channelIndex)
                SetCurrentRange(rangeIndex)
                UpdateInfoText(String.Format("Board {0}, module {1}, channel {2}, range {3} configured",
                        BoardIndex,
                        ModuleIndex,
                        channelIndex,
                        _selectedRange.RangeText))
            Else
                SetFunctionErrorInfo("adqAISetChannelAndRange", adqError)
            End If
        End If
        Return adqError
    End Function

    Public Function ReadSingle(ByRef digitalValue As UInteger) As UInteger
        Dim adqError As UInteger = AdqDriver.adqReadSingle(BoardIndex, ModuleIndex, digitalValue)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqReadSingle", adqError)
        End If
        Return adqError
    End Function

    Public Function Reset() As UInteger
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
        Else
            UpdateInfoText(String.Format("Reset board {0} module {1}", BoardIndex, ModuleIndex))
        End If
        _selectedRangeIndex = _notInitialized
        _selectedChannelIndex = _notInitialized
        Return adqError
    End Function

    Private Function IsRangeChanged(rangeIndex As UInteger) As Boolean
        Return _selectedRangeIndex = _notInitialized OrElse _selectedRangeIndex <> rangeIndex
    End Function

    Public Sub SetCurrentRange(rangeIndex As UInteger)
        _selectedRange = If(rangeIndex < _ranges.Count, _ranges(CInt(rangeIndex)), Nothing)
        _selectedRangeIndex = rangeIndex
    End Sub

    Private Function IsChannelChanged(channelIndex As UInteger) As Boolean
        Return _selectedChannelIndex = _notInitialized OrElse _selectedChannelIndex <> channelIndex
    End Function

    Public Sub SetCurrentChannel(channelIndex As UInteger)
        _selectedChannelIndex = channelIndex
    End Sub

    ReadOnly Property NumberOfRanges As UInteger

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("Function {0} returned Error number: 0x{1:X}", functionName, adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent InfoTextChanged(Me, e)
    End Sub

    Private Sub BuildModuleName(boardName As String)
        _moduleName = String.Format("{0}, Board {1}, Module {2}, {3} {4}",
            AdqCommon.GetModelName(boardName),
            BoardIndex,
            ModuleIndex,
            NumberOfChannels,
            IIf(NumberOfChannels = 1, "Channel", "Channels"))
    End Sub
End Class
