﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports AISingle.ALLDAQ.Boards

Public Class ModuleLocator
    Dim adqOpenError As UInteger = AdqCommon.NotOpen
    Private ReadOnly _aiModules As ICollection(Of AnalogInputModule)
    Public Event InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(aiModules As ICollection(Of AnalogInputModule),
                   ByRef moduleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs))
        _aiModules = aiModules
        AddHandler InfoTextChanged, moduleInfoTextChanged
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Public Sub CloseAdqDriver()
        If adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Public Sub FindAdqModules()
        adqOpenError = OpenAdqDriver()
        If adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            Dim numberOfBoards As UInteger = GetNumberOfBoards()
            Dim boardIdx As UInteger = 0

            While boardIdx < numberOfBoards
                Dim numberOfModules As UInteger
                Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)
                If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                    boardIdx += 1UI
                    Continue While
                End If

                GetModuleTypes(boardIdx, numberOfModules)
                boardIdx += 1UI
            End While
        End If
    End Sub

    Private Function GetModuleTypes(boardIndex As UInteger, numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0

        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If moduleType = AdqConstants.ADQ_MODULE_TYPE_AI Then
                GetAnalogInputModuleInfo(boardIndex, moduleIdx)
            End If

            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetAnalogInputModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqAiModuleInfo As AdqDriver.ADQAIModuleInfo = New AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqAiModuleInfo.Size))
        Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex,
                                              moduleIndex,
                                              adqGeneralModuleInfo,
                                              adqAiModuleInfo.Size,
                                              moduleInfo,
                                              infoSize)
        adqAiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQAIModuleInfo)), AdqDriver.ADQAIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqAiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize))

        Dim boardName As String = GetBoardName(boardIndex)
        If adqAiModuleInfo.RangeInfoPerChannel Then
            UpdateInfoText(String.Format("Board {0}, {1}, Module {2} has individual ranges." _
                & Environment.NewLine _
                & "Please update to a newer version of this program.",
                boardIndex,
                boardName,
                moduleIndex))
        Else
            Dim aiModule As AnalogInputModule = New AnalogInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqAiModuleInfo)
            _aiModules.Add(aiModule)
        End If
        Return adqError
    End Function

    Private Function GetBoardName(boardIndex As UInteger) As String
        Dim s As String = ""
        Dim boardInfo As AdqDriver.ADQBoardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            s = boardInfo.BoardName
        End If
        Return s
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("Function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args As InfoTextChangedEventArgs = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent InfoTextChanged(Me, e)
    End Sub
End Class
