﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file contains a programming example for ALLDAQ Data Acquisition Boards
'       with modules of type 
'       - Analog Input
'       Read a single value from the module 
' 
'-----------------------------------------------------------------------------

Imports AISingle.ALLDAQ.Boards

Public Class FormMain
    Private _factorIsValid As Boolean
    Private _offsetIsValid As Boolean
    Private _moduleLocator As ModuleLocator
    Private ReadOnly _aiModules As List(Of AnalogInputModule) = New List(Of AnalogInputModule)
    Private _selectedModule As AnalogInputModule

    Private Sub FrmMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Try
            EnableUI(False)
            _moduleLocator = New ModuleLocator(_aiModules, AddressOf OnModuleInfoTextChanged)
            _moduleLocator.FindAdqModules()
            FillModuleCombobox()
            SelectFirstComboboxItem(coBoxModules)
            AddInfoTextLine("Number of analog input modules found: " & _aiModules.Count.ToString())
        Catch ex As DllNotFoundException
            AddInfoTextLine("The device driver is not properly installed." & Environment.NewLine & ex.Message)
        End Try
    End Sub

    Private Sub FillModuleCombobox()
        coBoxModules.Items.Clear()
        For Each aiModule As AnalogInputModule In _aiModules
            coBoxModules.Items.Add(aiModule.ModuleName)
        Next
    End Sub

    Private Sub OnModuleInfoTextChanged(sender As Object, e As InfoTextChangedEventArgs)
        AddInfoTextLine(e.InfoText)
    End Sub

    Private Sub CoBoxModules_SelectedIndexChanged(sender As Object, e As EventArgs) Handles coBoxModules.SelectedIndexChanged
        EnableUI(coBoxModules.SelectedIndex >= 0)
        If _selectedModule IsNot Nothing Then
            RemoveHandler _selectedModule.InfoTextChanged, AddressOf OnModuleInfoTextChanged
        End If

        If coBoxModules.SelectedIndex < 0 Then
            _selectedModule = Nothing
        Else
            _selectedModule = _aiModules(coBoxModules.SelectedIndex)
            AddHandler _selectedModule.InfoTextChanged, AddressOf OnModuleInfoTextChanged
        End If
        UpdateModuleProperties(_selectedModule)
        FillChannels(_selectedModule)
        FillRanges(_selectedModule)
    End Sub

    Private Sub FillChannels(aiModule As AnalogInputModule)
        coBoxChannels.Items.Clear()
        If aiModule Is Nothing Then Return

        Dim chanIdx As UInteger = 0
        While chanIdx < aiModule.NumberOfChannels
            coBoxChannels.Items.Add(chanIdx.ToString())
            chanIdx += 1UI
        End While
        SelectFirstComboboxItem(coBoxChannels)
    End Sub

    Private Sub UpdateModuleProperties(adqModule As AnalogInputModule)
        If adqModule Is Nothing Then
            TextBoxDescription.Clear()
            TextBoxFactor.Clear()
            TextBoxOffset.Clear()
            Return
        End If

        TextBoxDescription.Text = adqModule.Description
        TextBoxFactor.Text = adqModule.Factor.ToString()
        TextBoxOffset.Text = adqModule.Offset.ToString()
        _factorIsValid = True
        _offsetIsValid = True
    End Sub

    Private Sub FillRanges(aiModule As AnalogInputModule)
        coBoxRanges.Items.Clear()
        If aiModule Is Nothing Then Return

        Dim rangeIdx As UInteger = 0
        While rangeIdx < aiModule.NumberOfRanges
            Dim PhysicalRange As AnalogRange = aiModule.GetRange(rangeIdx)
            coBoxRanges.Items.Add(PhysicalRange.RangeText)
            rangeIdx += 1UI
        End While
        SelectFirstComboboxItem(coBoxRanges)
    End Sub

    Private Sub TextBoxFactor_Leave(sender As Object, e As EventArgs) Handles TextBoxFactor.Leave
        If _selectedModule Is Nothing Then Return

        Dim value As Double
        _factorIsValid = Double.TryParse(TextBoxFactor.Text, value)
        If Not _factorIsValid Then
            AddInfoTextLine("The entered text at Factor is not a valid number: """ & TextBoxFactor.Text & """")
            Return
        End If
        _selectedModule.Factor = value
    End Sub

    Private Sub TextBoxOffset_Leave(sender As Object, e As EventArgs) Handles TextBoxOffset.Leave
        If _selectedModule Is Nothing Then Return

        Dim value As Double
        _offsetIsValid = Double.TryParse(TextBoxOffset.Text, value)
        If Not _offsetIsValid Then
            AddInfoTextLine("The entered text at Offset is not a valid number: """ & TextBoxOffset.Text & """")
            Return
        End If
        _selectedModule.Offset = value
    End Sub

    Private Sub BtnReadValue_Click(sender As Object, e As EventArgs) Handles btnReadValue.Click
        If (Not _factorIsValid) OrElse (Not _offsetIsValid) Then
            AddInfoTextLine("Please correct the entered Factor and Offset.")
            Return
        End If

        ReadAnalogValue(_selectedModule, CUInt(coBoxChannels.SelectedIndex), CUInt(coBoxRanges.SelectedIndex))
    End Sub

    Private Sub ReadAnalogValue(aiModule As AnalogInputModule, channelIndex As UInteger, rangeIndex As UInteger)
        If aiModule Is Nothing Then Return

        Dim adqError As UInteger = aiModule.SetChannelAndRange(channelIndex, rangeIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            Return
        End If

        ReadSingleValue(aiModule)
    End Sub

    Private Sub ReadSingleValue(aiModule As AnalogInputModule)
        Dim digitalValue As UInteger
        Dim adqError As UInteger = aiModule.ReadSingle(digitalValue)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            DisplayValue(aiModule, digitalValue)
        End If
    End Sub

    Private Sub DisplayValue(aiModule As AnalogInputModule, digitalValue As UInteger)
        Dim physicalValue As Double = aiModule.SelectedRange.DigitalToPhysical(digitalValue)

        Dim s As String = String.Format("[{0}] Board {1}, Module {2}, Channel {3}",
                                        Date.Now.ToString("HH:mm:ss.fff"),
                                        aiModule.BoardIndex,
                                        aiModule.ModuleIndex,
                                        aiModule.ChannelIndex) _
            & String.Format(" --> digital: {0}, physical: {1:0.000000000}{2}",
                digitalValue,
                physicalValue,
                aiModule.SelectedRange.UnitSymbol)

        If aiModule.Factor <> 1.0 OrElse aiModule.Offset <> 0.0 Then
            physicalValue = physicalValue * aiModule.Factor + aiModule.Offset
            s &= String.Format("; Factor: {0}, Offset: {1} --> {2:0.000000000}{3}",
                    aiModule.Factor, aiModule.Offset, physicalValue, aiModule.SelectedRange.UnitSymbol)
        End If
        s &= Environment.NewLine
        textBoxInfo.AppendText(s)
    End Sub

    Private Sub AddInfoTextLine(text As String)
        textBoxInfo.AppendText(String.Format("[{0}] {1}" & Environment.NewLine,
                    Date.Now.ToString("HH:mm:ss.fff"), text))
    End Sub

    Private Sub SelectFirstComboboxItem(coBox As ComboBox)
        If coBox.Items.Count > 0 Then
            coBox.SelectedIndex = 0
        End If
    End Sub

    Private Sub CloseAdqDriver()
        If _moduleLocator IsNot Nothing Then
            _moduleLocator.CloseAdqDriver()
        End If
    End Sub

    Private Sub EnableUI(enable As Boolean)
        btnReadValue.Enabled = enable
        btnReset.Enabled = enable
        TextBoxFactor.Enabled = enable
        TextBoxOffset.Enabled = enable
    End Sub

    Private Sub BtnReset_Click(sender As Object, e As EventArgs) Handles btnReset.Click
        ResetModule(_selectedModule)
    End Sub

    Private Sub ResetModule(aiModule As AnalogInputModule)
        aiModule.Reset()
    End Sub

    Private Sub FrmMain_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        CloseAdqDriver()
    End Sub

    Private Sub BtnClose_Click(sender As Object, e As EventArgs) Handles btnClose.Click
        Close()
    End Sub

    Private Sub BtnClear_Click(sender As Object, e As EventArgs) Handles btnClear.Click
        ClearInfoText()
    End Sub

    Private Sub ClearInfoText()
        textBoxInfo.Clear()
    End Sub

    Private Sub BtnInfo_Click(sender As Object, e As EventArgs) Handles btnInfo.Click
        AboutBox.ShowDialog()
    End Sub
End Class
