﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports CommonSettings.ALLDAQ.Boards

Public Class ModuleLocator
    Private _adqOpenError As UInteger = AdqCommon.NotOpen
    Private ReadOnly _aiModules As ICollection(Of AnalogModule)
    Private ReadOnly _dioBoards As ICollection(Of Board)
    Private ReadOnly _diModules As ICollection(Of DigitalInputModule)

    Public Event InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(aiModules As ICollection(Of AnalogModule), diModules As ICollection(Of DigitalInputModule),
                   dioBoards As ICollection(Of Board), ByRef InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs))
        _aiModules = aiModules
        _diModules = diModules
        _dioBoards = dioBoards
        AddHandler InfoTextChanged, InfoTextChanged
    End Sub

    Public Sub InitModuleLocator()
        _adqOpenError = OpenAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            Dim numberOfBoards As UInteger = GetNumberOfBoards()
            Dim boardIdx As UInteger = 0
            While boardIdx < numberOfBoards
                Dim boardInfo As AdqDriver.ADQBoardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)
                Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIdx, boardInfo)
                If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                    SetFunctionErrorInfo("adqGetBoardInfo", adqError)
                    boardIdx += 1UI
                    Continue While
                End If

                If SupportsTTLVoltageSelection(boardInfo.TTLVoltageLevel) Then
                    _dioBoards.Add(New Board(boardInfo.BoardName, boardIdx, boardInfo))
                End If

                Dim numberOfModules As UInteger
                adqError = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)
                If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                    boardIdx += 1UI
                    Continue While
                End If
                GetModuleTypes(boardInfo.BoardName, boardIdx, numberOfModules)
                boardIdx += 1UI
            End While
        End If
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Private Function GetModuleTypes(boardName As String, boardIndex As UInteger, numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0
        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If moduleType = AdqConstants.ADQ_MODULE_TYPE_AI OrElse moduleType = AdqConstants.ADQ_MODULE_TYPE_AO Then
                AddAnalogModule(boardName, boardIndex, moduleIdx, moduleType)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DI Then
                adqError = GetDiModuleInfo(boardName, boardIndex, moduleIdx)
                If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                    moduleIdx += 1UI
                    Continue While
                End If
            End If
            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Sub AddAnalogModule(boardName As String, boardIndex As UInteger, moduleIndex As UInteger, moduleType As UInteger)
        _aiModules.Add(New AnalogModule(boardName, boardIndex, moduleIndex, moduleType))
    End Sub

    Private Function GetDiModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo As AdqDriver.ADQDIModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)
        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.Size, moduleInfo, infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

        If adqDiModuleInfo.NumberOfFilters > 0 Then
            Dim diModule = New DigitalInputModule(boardName, boardIndex, moduleIndex, adqDiModuleInfo)
            _diModules.Add(diModule)
        End If
        Return adqError
    End Function

    Private Function SupportsTTLVoltageSelection(value As UInteger) As Boolean
        If value = 0 Then
            Return False
        End If
        Return (value And (value - 1)) <> 0
    End Function

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Public Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        RaiseOnInfoTextChanged(String.Format("API-function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Protected Sub RaiseOnInfoTextChanged(infoText As String)
        If InfoTextChangedEvent IsNot Nothing Then
            Dim args = New InfoTextChangedEventArgs(infoText)
            RaiseEvent InfoTextChanged(Me, args)
        End If
    End Sub
End Class
