﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports DIReadContinuous.ALLDAQ.Boards
Imports DIReadContinuous.DIReadContinuous.ALLDAQ
Imports System.Runtime.InteropServices

Class ModuleLocator
    Private _adqOpenError As UInteger = AdqCommon.NotOpen
    Private ReadOnly _modules As ICollection(Of DigitalInputModule)
    Public Event ModuleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(ByVal Modules As ICollection(Of DigitalInputModule), ByVal moduleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs))
        _modules = Modules
        AddHandler ModuleInfoTextChanged, moduleInfoTextChanged
    End Sub

    Public Sub FindModules()
        _adqOpenError = OpenAdqDriver()
        If _adqOpenError <> AdqErrors.ADQ_ERROR_NONE Then Return

        Dim numberOfBoards As UInteger = GetNumberOfBoards()
        Dim boardIdx As UInteger = 0
        While boardIdx < numberOfBoards
            Dim numberOfModules As UInteger
            Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)

            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                boardIdx += 1UI
                Continue While
            End If

            GetModuleTypes(boardIdx, numberOfModules)
            boardIdx += 1UI
        End While
    End Sub

    Private Function GetModuleTypes(ByVal boardIndex As UInteger, ByVal numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0
        Dim boardName As String = Nothing

        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)

            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If moduleType = AdqConstants.ADQ_MODULE_TYPE_DI OrElse moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
                If boardName Is Nothing Then
                    boardName = GetBoardName(boardIndex)
                End If

                If moduleType = AdqConstants.ADQ_MODULE_TYPE_DI Then
                    adqError = GetDIModuleInfo(boardName, boardIndex, moduleIdx)
                    If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                        moduleIdx += 1UI
                        Continue While
                    End If
                ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
                    adqError = GetDIOModuleInfo(boardName, boardIndex, moduleIdx)
                    If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                        moduleIdx += 1UI
                        Continue While
                    End If
                End If
            End If

            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetDIModuleInfo(ByVal boardName As String, ByVal boardIndex As UInteger, ByVal moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)
        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)
        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.Size, moduleInfo, infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0,
                         String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            _modules.Add(New DigitalInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.PortWidthBits))
        End If
        Return adqError
    End Function

    Private Function GetDIOModuleInfo(ByVal boardName As String, ByVal boardIndex As UInteger, ByVal moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)
        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, False)
        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.Size, moduleInfo, infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))
        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0,
                         String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            _modules.Add(New DigitalInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.PortWidthBits))
        End If
        Return adqError
    End Function

    Private Function IsClockControlledModule(clockType As UInteger) As Boolean
        Return clockType = AdqConstants.ADQ_CLOCK_TYPE_HARDWARE OrElse clockType = AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE
    End Function

    Private Function GetBoardName(ByVal boardIndex As UInteger) As String
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)
        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetBoardInfo", adqError)
            Return ""
        End If

        Return boardInfo.BoardName
    End Function

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If

        Return adqError
    End Function

    Public Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If

        Return numBoards
    End Function

    Private Sub SetFunctionErrorInfo(ByVal functionName As String, ByVal adqError As UInteger)
        UpdateInfoText(String.Format("API-function {0} returned Error number: &H{1:X}", functionName, adqError))
    End Sub

    Private Sub UpdateInfoText(ByVal text As String)
        Dim args As InfoTextChangedEventArgs = New InfoTextChangedEventArgs With
        {
            .InfoText = text
        }
        RaiseEvent ModuleInfoTextChanged(Me, args)
    End Sub
End Class
