﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Globalization
Imports DIReadContinuous.ALLDAQ.Boards

Public Class AdqCommon
    Private Const _oneMicrosecond As Double = 0.000001
    Private Const _oneMillisecond As Double = 0.001
    Private Const _oneSecond As Double = 1.0
    Private Const _oneMinute As Double = 60
    Private Const _oneHour As Double = 3600

    Public Shared DefaultDisplayNumberOfValues As UInteger = 1000
    Public Shared DefaultDesiredSampleFrequency As UInteger = 1000
    Public Shared DefaultMinNotifyAfterValues As UInteger = 1

    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN

    Public Enum PropertyType As Integer
        ActualSampleFrequency
        DisplayBuffer
        UpdateValuesToDisplay
        UpdateNumberOfValues
        AcquisitionStarting
        AcquisitionStarted
        AcquisitionEnded
        AcquisitionErrorOccurred
    End Enum

    Public Shared BitColors As Color() = {
        Color.SkyBlue,
        Color.LimeGreen,
        Color.MediumOrchid,
        Color.LightCoral,
        Color.SteelBlue,
        Color.YellowGreen,
        Color.Turquoise,
        Color.HotPink,
        Color.Khaki,
        Color.Tan,
        Color.DarkSeaGreen,
        Color.CornflowerBlue,
        Color.Plum,
        Color.CadetBlue,
        Color.PeachPuff,
        Color.LightSalmon}

    Public Shared DefaultBackgroundColor As Color = Color.Black
    Public Shared DefaultAxisColor As Color = Color.DarkSlateGray
    Public Shared DefaultLabelColor As Color = Color.LightSkyBlue
    Public Shared DefaultScrollBarColor As Color = Color.LightGray

    Public Shared Function GetModuleTypeName(moduleType As UInteger) As String
        Return GetDictionaryValue(_moduleTypeNames, moduleType)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If (Not found) Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _moduleTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        }

    Public Shared Function GetModelName(ByVal boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then posADQ = 0
        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)

        If posEnd < 0 Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function

    Public Shared Function FormatFrequency(ByVal frequency As Double) As String
        Dim s As String

        If frequency >= 1000000 Then
            s = String.Format("{0:0.######}MHz", frequency / 1000000)
        ElseIf frequency >= 1000 Then
            s = String.Format("{0:0.######}kHz", frequency / 1000)
        Else
            s = String.Format("{0:0.########}Hz", frequency)
        End If

        Return s
    End Function

    Public Shared Function FormatSampleTime(ByVal seconds As Double) As String
        If seconds = 0.0 Then Return "0"
        Dim s As String

        If seconds < _oneMicrosecond Then
            s = String.Format("{0:0##}ns", seconds * 1000000000)
        ElseIf seconds < _oneMillisecond Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.#######}µs", seconds * 1000000)
        ElseIf seconds < _oneSecond Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.#######}ms", seconds * 1000)
        ElseIf seconds < _oneMinute Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.#######}s", seconds)
        ElseIf seconds < _oneHour Then
            Dim minutes As Integer = CInt((seconds / _oneMinute))
            Dim sec As Double = seconds Mod _oneMinute
            s = String.Format(CultureInfo.InvariantCulture, "{0}:{1:0.#######}m", minutes, sec)
        Else
            Dim hours As Integer = CInt((seconds / _oneHour))
            Dim minutes As Integer = CInt((seconds Mod _oneHour))
            Dim sec As Double = seconds Mod _oneMinute
            s = String.Format(CultureInfo.InvariantCulture, "{0}:{1}:{2:0.#######}h", hours, minutes, sec)
        End If
        Return s
    End Function
End Class
