﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the FINRead project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports FINRead.ALLDAQ.Boards

Public Class FinModule
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        TimeoutMilliseconds = AdqCommon.DefaultTimeoutMilliseconds
        ModuleName = GetModuleName(boardName)
    End Sub

    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ModuleName As String
    Public Property TimeoutMilliseconds As UInteger

    Public Async Function FinReadAsync() As Task(Of FinReadData)
        IsFinReading = True
        Dim finData As FinReadData = Nothing
        Await Task.Run(Sub()
                           finData = FinRead(BoardIndex, ModuleIndex, TimeoutMilliseconds)
                       End Sub)
        IsFinReading = False
        Return finData
    End Function

    Private Shared Function FinRead(boardIndex As UInteger, moduleIndex As UInteger, timeoutMilliseconds As UInteger) As FinReadData
        Dim frequency As Double
        Dim onTime As Double
        Dim adqError As UInteger

        adqError = AdqDriver.adqFINRead(boardIndex, moduleIndex, frequency, onTime, timeoutMilliseconds)

        Dim finData As FinReadData = New FinReadData With {
            .TimeOfMeasurement = Date.Now,
            .HasError = adqError <> AdqErrors.ADQ_ERROR_NONE,
            .Frequency = frequency,
            .HighTime = onTime
        }
        If finData.HasError Then
            finData.ErrorText = GetFinReadErrorText(adqError, timeoutMilliseconds)
        End If
        Return finData
    End Function

    Private Shared Function GetFinReadErrorText(adqError As UInteger, timeoutMilliseconds As UInteger) As String
        Dim s As String = AdqCommon.GetFunctionErrorText("adqFINRead", adqError)
        If adqError = AdqErrors.ADQ_ERROR_TIMEOUT Then
            s &= Environment.NewLine & String.Format("Time-out reached after {0} milliseconds", timeoutMilliseconds)
        End If
        Return s
    End Function

    Private _isFinReading As Boolean
    Public Property IsFinReading As Boolean
        Get
            Return _isFinReading
        End Get
        Private Set
            _isFinReading = Value
            NotifyPropertyChanged(NameOf(IsFinReading))
        End Set
    End Property

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(_BoardIndex, _ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
            Return False
        End If
        Return True
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        ModuleInfoText = AdqCommon.GetFunctionErrorText(functionName, adqError)
    End Sub

    Private _infoText As String = String.Empty
    Public Property ModuleInfoText As String
        Get
            Return _infoText
        End Get
        Set
            _infoText = Value
            NotifyPropertyChanged(NameOf(ModuleInfoText))
        End Set
    End Property

    Private Sub NotifyPropertyChanged(propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Function GetModuleName(boardName As String) As String
        Return AdqCommon.GetModelName(boardName) _
                & String.Format(", Board {0}, Module {1}", BoardIndex, ModuleIndex)
    End Function
End Class
