﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the FINRead project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel

Public Class MainViewModel
    Implements INotifyPropertyChanged, IDataErrorInfo

    Private ReadOnly _errorInfo As Dictionary(Of String, String) = New Dictionary(Of String, String)
    Private _moduleLocator As ModuleLocator
    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Property ResetModuleCommand As RelayCommand
    Public Property FinReadCommand As RelayCommand
    Public Property ClearInfoTextCommand As RelayCommand
    Public Property CloseCommand As RelayCommand

    Public Sub New()
        InitCommands()
    End Sub

    Private Sub InitCommands()
        ResetModuleCommand = New RelayCommand(AddressOf ResetModuleExecute, AddressOf CanResetModuleExecute)
        FinReadCommand = New RelayCommand(AddressOf FinReadExecute, AddressOf CanFinReadExecute)
        ClearInfoTextCommand = New RelayCommand(AddressOf ClearInfoTextExecute)
        CloseCommand = New RelayCommand(AddressOf CloseApplicationExecute)
    End Sub

    Friend Sub Init()
        _moduleLocator = New ModuleLocator(AddressOf MainViewModel_PropertyChanged)
        _moduleLocator.InitModuleLocator()
        If FinModules.Count > 0 Then
            SelectedModule = FinModules(0)
        End If
    End Sub
    Public ReadOnly Property FinModules As List(Of FinModule)
        Get
            Return _moduleLocator.FinModules
        End Get
    End Property

    Private _selectedModule As FinModule
    Public Property SelectedModule As FinModule
        Get
            Return _selectedModule
        End Get
        Set
            If _selectedModule Is Value Then Return
            If _selectedModule IsNot Nothing Then
                RemoveHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
            End If

            _selectedModule = Value
            If _selectedModule IsNot Nothing Then
                AddHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
                FinTimeoutText = _selectedModule.TimeoutMilliseconds.ToString()
            End If
            ResetModuleCommand.RaiseCanExecuteChanged()
            FinReadCommand.RaiseCanExecuteChanged()
            NotifyPropertyChanged(NameOf(SelectedModule))
        End Set
    End Property

    Private _finTimeoutText As String = String.Empty
    Public Property FinTimeoutText As String
        Get
            Return _finTimeoutText
        End Get
        Set
            If _selectedModule Is Nothing Then Return
            If _finTimeoutText <> Value Then
                _finTimeoutText = Value
                Dim timeoutValue As UInteger
                Dim msg As String = ValidateTimeoutValue(_finTimeoutText, timeoutValue)
                If Not String.IsNullOrEmpty(msg) Then
                    _errorInfo.AddOrUpdate(NameOf(FinTimeoutText), msg)
                Else
                    _selectedModule.TimeoutMilliseconds = timeoutValue
                    _errorInfo.Remove(NameOf(FinTimeoutText))
                End If

                NotifyPropertyChanged(NameOf(FinTimeoutText))
                FinReadCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private Function ValidateTimeoutValue(textValue As String, ByRef timeout As UInteger) As String
        timeout = 0
        Dim msg As String = String.Empty
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        Else
            If Not UInteger.TryParse(textValue, timeout) Then
                msg = "The entered text """ & textValue & """ is not a valid number"
            End If
        End If
        Return msg
    End Function

    Private Function CanFinReadExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing AndAlso Not _selectedModule.IsFinReading AndAlso _errorInfo.Count = 0
    End Function

    Private Async Sub FinReadExecute(obj As Object)
        AppendInfoText(String.Format("Starting FIN Read operation, time-out {0} milliseconds...", _selectedModule.TimeoutMilliseconds))
        Dim finData As FinReadData = Await _selectedModule.FinReadAsync()
        UpdateFinReadMeasuredValues(finData)
    End Sub

    Private Sub UpdateFinReadMeasuredValues(finData As FinReadData)
        If finData IsNot Nothing Then
            If finData.HasError Then
                AppendInfoText(finData.ErrorText)
            Else
                Dim s As String = "FIN Read measured values:" _
                        & Environment.NewLine _
                        & String.Format("- Time of measurement: " & finData.TimeOfMeasurement.ToString("HH:mm:ss.fff")) _
                        & Environment.NewLine _
                        & String.Format("- Frequency: {0:0.#########} Hz", finData.Frequency)
                If finData.Frequency > 0 Then
                    Dim period As Double = 1 / finData.Frequency
                    s &= Environment.NewLine
                    s &= String.Format("- Period: {0:0.########} sec", period)
                    s &= Environment.NewLine
                    s &= String.Format("- High-time: {0:0.########} sec", finData.HighTime)
                    s &= Environment.NewLine
                    s &= String.Format("- Low-time: {0:0.########} sec", period - finData.HighTime)
                    s &= Environment.NewLine
                    s &= String.Format("- Duty cycle: {0:0.########} %", finData.HighTime / period * 100.0)
                End If
                AppendInfoText(s)
            End If
        End If
    End Sub

    Private Function CanResetModuleExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub ResetModuleExecute(obj As Object)
        Dim success As Boolean = _selectedModule.ResetModule()
        If success Then
            AppendInfoText(String.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex))
        End If
    End Sub

    Public ReadOnly Property CanSelectFinModule As Boolean
        Get
            Return (_selectedModule Is Nothing AndAlso FinModules.Count > 0) _
                OrElse (_selectedModule IsNot Nothing AndAlso Not _selectedModule.IsFinReading)
        End Get
    End Property

    Private Sub ClearInfoTextExecute(obj As Object)
        InfoText = String.Empty
    End Sub

    Private Sub AppendInfoText(text As String)
        If InfoText <> String.Empty Then
            InfoText &= Environment.NewLine
        End If
        InfoText &= Date.Now.ToString("[HH:mm:ss.fff] ") & text
    End Sub

    Dim _infoText As String = String.Empty
    Public Property InfoText As String
        Get
            Return _infoText
        End Get
        Set(value As String)
            _infoText = value
            NotifyPropertyChanged(NameOf(InfoText))
        End Set
    End Property

    Default Public ReadOnly Property Item(propertyName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim value As String = String.Empty
            If _errorInfo.TryGetValue(propertyName, value) Then
                Return value
            End If
            Return Nothing
        End Get
    End Property

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return "MainViewModel.Error"
        End Get
    End Property

    Protected Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub MainViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(InfoText) Then
            AppendInfoText(_moduleLocator.InfoText)
        ElseIf e.PropertyName = NameOf(_selectedModule.ModuleInfoText) Then
            AppendInfoText(_selectedModule.ModuleInfoText)
        ElseIf e.PropertyName = NameOf(_selectedModule.IsFinReading) Then
            FinReadCommand.RaiseCanExecuteChanged()
            NotifyPropertyChanged(NameOf(CanSelectFinModule))
        End If
    End Sub

    Private Sub CloseApplicationExecute(obj As Object)
        System.Windows.Application.Current.MainWindow.Close()
    End Sub

    Friend Sub OnWindowClosing(sender As Object, e As CancelEventArgs)
        If _selectedModule IsNot Nothing AndAlso _selectedModule.IsFinReading Then
            RemoveHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
            _selectedModule.ResetModule()
        End If
    End Sub
End Class
