﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the I2CReadWrite project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports I2CReadWrite.ALLDAQ.Boards

Public Class I2cModule
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef moduleInfo As AdqDriver.ADQGeneralModuleInfo,
                   ByRef i2cModuleInfo As AdqDriver.ADQI2CModuleInfo)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex

        ModuleName = GetModuleName(boardName)
        Description = GetDescription(moduleInfo.ModuleDescription, i2cModuleInfo)
    End Sub

    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ModuleName As String
    Public ReadOnly Property Description As String

    Public Function SetSlaveAddress(address As UInteger) As Boolean
        Dim adqError As UInteger = AdqDriver.adqI2CSetSlaveAddress(BoardIndex, ModuleIndex, address)
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqI2CSetSlaveAddress", adqError)
            Return False
        End If
        Return True
    End Function

    Public Function Read(numberOfBytesToRead As UInteger) As Boolean
        If (BytesRead Is Nothing OrElse BytesRead.Length <> numberOfBytesToRead) Then
            BytesRead = New Byte(CInt(numberOfBytesToRead - 1)) {}
        End If

        Dim numberOfBytesReturned As UInteger
        Dim adqError As UInteger = AdqDriver.adqI2CRead(BoardIndex, ModuleIndex, numberOfBytesToRead, BytesRead, numberOfBytesReturned)
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqI2CRead", adqError)
            Return False
        End If
        Return True
    End Function

    Private newPropertyValue As Byte()
    Public Property BytesRead As Byte()
        Get
            Return newPropertyValue
        End Get
        Private Set(ByVal value As Byte())
            newPropertyValue = value
        End Set
    End Property

    Public Function Write(bytes As Byte()) As Boolean
        Dim numberOfBytesWritten As UInteger
        Dim adqError As UInteger = AdqDriver.adqI2CWrite(BoardIndex, ModuleIndex, CUInt(bytes.Length), bytes, numberOfBytesWritten)
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqI2CWrite", adqError)
            Return False
        End If
        Return True
    End Function

    Public Function WriteThenRead(bytes As Byte(), numberOfBytesToRead As UInteger) As Boolean
        If (BytesRead Is Nothing OrElse BytesRead.Length <> numberOfBytesToRead) Then
            BytesRead = New Byte(CInt(numberOfBytesToRead - 1)) {}
        End If

        Dim numberOfBytesWritten As UInteger
        Dim numberOfBytesReturned As UInteger
        Dim adqError As UInteger = AdqDriver.adqI2CWriteThenRead(BoardIndex, ModuleIndex, CUInt(bytes.Length), bytes,
            numberOfBytesWritten, numberOfBytesToRead, BytesRead, numberOfBytesReturned)
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqI2CWriteThenRead", adqError)
            Return False
        End If
        Return True
    End Function

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqResetModule", adqError)
            Return False
        End If
        Return True
    End Function

    Private Function GetModuleName(boardName As String) As String
        Return AdqCommon.GetModelName(boardName) _
            & String.Format(", Board {0}, Module {1}", BoardIndex, ModuleIndex)
    End Function

    Private Function GetDescription(moduleDescription As String, ByRef i2cModuleInfo As AdqDriver.ADQI2CModuleInfo) As String
        Return moduleDescription & Environment.NewLine _
            & "- Speed bits per second: " & i2cModuleInfo.SpeedBitsPerSecond.ToString() & Environment.NewLine _
            & "- Supports 7-Bit addresses: " & i2cModuleInfo.Supports7BitAddresses.ToString() & Environment.NewLine _
            & "- Supports 10-Bit addresses: " & i2cModuleInfo.Supports10BitAddresses.ToString()
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        ModuleInfoText = String.Format("Function {0} returned error number: &H{1:X}", functionName, adqError)
    End Sub

    Private _moduleInfoText As String = String.Empty
    Public Property ModuleInfoText As String
        Get
            Return _moduleInfoText
        End Get
        Set
            _moduleInfoText = Value
            NotifyPropertyChanged("ModuleInfoText")
        End Set
    End Property

    Protected Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
End Class
