﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the I2CReadWrite project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Globalization

Public Class MainViewModel
    Implements INotifyPropertyChanged

    Private _moduleLocator As ModuleLocator
    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Property ResetModuleCommand As RelayCommand
    Public Property SetI2cSlaveAddressCommand As RelayCommand
    Public Property I2cReadCommand As RelayCommand
    Public Property I2cWriteCommand As RelayCommand
    Public Property I2cWriteThenReadCommand As RelayCommand
    Public Property ClearInfoTextCommand As RelayCommand
    Public Property CloseCommand As RelayCommand

    Public Sub New()
        InitCommands()
    End Sub

    Private Sub InitCommands()
        ResetModuleCommand = New RelayCommand(AddressOf ResetModuleExecute, AddressOf CanResetModuleExecute)
        SetI2cSlaveAddressCommand = New RelayCommand(AddressOf SetI2cSlaveAddressExecute, AddressOf CanSetI2cSlaveAddressExecute)
        I2cReadCommand = New RelayCommand(AddressOf I2cReadBytesExecute, AddressOf CanI2cReadBytesExecute)
        I2cWriteCommand = New RelayCommand(AddressOf I2cWriteBytesExecute, AddressOf CanI2cWriteBytesExecute)
        I2cWriteThenReadCommand = New RelayCommand(AddressOf I2cWriteThenReadExecute, AddressOf CanI2cWriteThenReadExecute)
        ClearInfoTextCommand = New RelayCommand(AddressOf ClearInfoTextExecute)
        CloseCommand = New RelayCommand(AddressOf CloseApplicationExecute)
    End Sub

    Public Sub Init()
        _moduleLocator = New ModuleLocator(AddressOf MainViewModel_PropertyChanged)
        _moduleLocator.InitModuleLocator()
        If IsI2cModuleAvailable Then
            SelectedModule = I2cModules(0)
        End If
    End Sub

    Public ReadOnly Property I2cModules As List(Of I2cModule)
        Get
            Return _moduleLocator.I2cModules
        End Get
    End Property

    Private _selectedModule As I2cModule
    Public Property SelectedModule As I2cModule
        Get
            Return _selectedModule
        End Get
        Set
            If _selectedModule Is Value Then Return
            If _selectedModule IsNot Nothing Then
                RemoveHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
            End If

            _selectedModule = Value
            If _selectedModule IsNot Nothing Then
                AddHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
            End If
            ResetModuleCommand.RaiseCanExecuteChanged()
            SetI2cSlaveAddressCommand.RaiseCanExecuteChanged()
            I2cReadCommand.RaiseCanExecuteChanged()
            I2cWriteCommand.RaiseCanExecuteChanged()
            I2cWriteThenReadCommand.RaiseCanExecuteChanged()
            NotifyPropertyChanged(NameOf(SelectedModule))
            NotifyPropertyChanged(NameOf(ModuleDescription))
        End Set
    End Property

    Private Function CanResetModuleExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub ResetModuleExecute(obj As Object)
        Dim success As Boolean = _selectedModule.ResetModule()
        If success Then
            AppendInfoText(String.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex))
        End If
    End Sub

    Private Function CanSetI2cSlaveAddressExecute(obj As Object) As Boolean
        Return CanUseI2cModule
    End Function

    Private Sub SetI2cSlaveAddressExecute(obj As Object)
        Try
            Dim i2cAddressValue As UInteger = SetNewSlaveAddressValue()
            Dim success As Boolean = _selectedModule.SetSlaveAddress(i2cAddressValue)
            If success Then
                AppendInfoText(String.Format("Board {0}, module {1}, I2C slave address set to &H{2:X}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex, i2cAddressValue))
            End If
        Catch ex As Exception
            AppendInfoText("Error setting slave address:" & Environment.NewLine & ex.Message)
        End Try
    End Sub

    Private Function SetNewSlaveAddressValue() As UInteger
        Dim newI2cAddressValue As UInteger = UInteger.Parse(_newI2cAddress, NumberStyles.AllowHexSpecifier)
        Dim i2cAddress As String = newI2cAddressValue.ToString("X")
        If Not _i2cAddresses.Contains(i2cAddress) Then
            _i2cAddresses.Add(i2cAddress)
        End If
        SelectedI2cAddress = i2cAddress
        Return newI2cAddressValue
    End Function

    Private ReadOnly _i2cAddresses As ObservableCollection(Of String) = New ObservableCollection(Of String)
    Public ReadOnly Property I2cAddresses As IEnumerable(Of String)
        Get
            Return _i2cAddresses
        End Get
    End Property

    Private _selectedI2cAddress As String
    Public Property SelectedI2cAddress As String
        Get
            Return _selectedI2cAddress
        End Get
        Set
            If _selectedI2cAddress <> Value Then
                _selectedI2cAddress = Value
                NotifyPropertyChanged(NameOf(SelectedI2cAddress))
            End If
        End Set
    End Property

    Private _newI2cAddress As String
    Public Property NewI2cAddress As String
        Get
            Return _selectedI2cAddress
        End Get
        Set
            If Not String.IsNullOrEmpty(Value) Then
                _newI2cAddress = Value
            End If
        End Set
    End Property

    Private Function CanI2cReadBytesExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub I2cReadBytesExecute(obj As Object)
        Dim success As Boolean = _selectedModule.Read(CUInt(_i2cReadNumBytesSelectedIndex + 1))
        If success Then
            DisplayBytes("Read", _selectedModule.BytesRead)
        End If
    End Sub

    Private Sub DisplayBytes(readOrWriteOperation As String, bytesRead As Byte())
        Dim s As String = GetOperationDescription(readOrWriteOperation, bytesRead.Length)
        For Each byteRead As Byte In bytesRead
            s &= String.Format("{0:X2} ", byteRead)
        Next
        AppendInfoText(s.TrimEnd())
    End Sub

    Private _i2cReadNumBytesSelectedIndex As Integer
    Public Property I2cReadNumBytesSelectedIndex As Integer
        Get
            Return _i2cReadNumBytesSelectedIndex
        End Get
        Set
            If _i2cReadNumBytesSelectedIndex <> Value Then
                _i2cReadNumBytesSelectedIndex = Value
                NotifyPropertyChanged(NameOf(I2cReadNumBytesSelectedIndex))
            End If
        End Set
    End Property

    Private Shared Function GetOperationDescription(operation As String, len As Integer) As String
        Return operation & " " & len.ToString() & If(len = 1, " byte:", " bytes:") & " hex "
    End Function

    Private ReadOnly _bytesToWrite As ObservableCollection(Of String) = New ObservableCollection(Of String)
    Public ReadOnly Property BytesToWrite As IEnumerable(Of String)
        Get
            Return _bytesToWrite
        End Get
    End Property

    Private _selectedBytesToWriteText As String
    Public Property SelectedBytesToWrite As String
        Get
            Return _selectedBytesToWriteText
        End Get
        Set
            If _selectedBytesToWriteText <> Value Then
                _selectedBytesToWriteText = Value
                NotifyPropertyChanged(NameOf(SelectedBytesToWrite))
            End If
        End Set
    End Property

    Private _newBytesToWrite As String
    Public Property NewBytesToWrite As String
        Get
            Return _selectedBytesToWriteText
        End Get
        Set
            _newBytesToWrite = Value
        End Set
    End Property

    Private Function CanI2cWriteBytesExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub I2cWriteBytesExecute(obj As Object)
        Dim bytesToWrite As String = _newBytesToWrite.Trim()

        If String.IsNullOrEmpty(bytesToWrite) Then
            AppendInfoText("Please enter bytes to write.")
            Return
        End If

        Dim bytes As Byte() = GetByteArrayToWrite(bytesToWrite)
        If bytes IsNot Nothing Then
            WriteBytesToModule(bytesToWrite, bytes)
        End If
    End Sub

    Private Function GetByteArrayToWrite(bytesToWrite As String) As Byte()
        Dim strBytes As String() = bytesToWrite.Trim().Split(TryCast(Nothing, Char()), StringSplitOptions.RemoveEmptyEntries)

        Dim len As Integer = strBytes.Length
        If len = 0 OrElse len > AdqCommon.MaxBytesReadWrite Then
            AppendInfoText(String.Format("Input error: {0} elements found. Only 1...{1} bytes can be written.", len, AdqCommon.MaxBytesReadWrite))
            Return Nothing
        End If

        Dim bytes = New Byte(len - 1) {}

        For i As Integer = 0 To len - 1
            Dim succeeded As Boolean = Byte.TryParse(strBytes(i), NumberStyles.AllowHexSpecifier, CultureInfo.InvariantCulture, bytes(i))
            If Not succeeded Then
                AppendInfoText(String.Format("Error parsing element '{0}' as byte.", strBytes(i)))
                Return Nothing
            End If
        Next
        Return bytes
    End Function

    Private Sub WriteBytesToModule(bytesToWrite As String, bytes As Byte())
        Dim succeeded As Boolean = _selectedModule.Write(bytes)
        If succeeded Then
            UpdateBytesToWriteProperties(bytesToWrite)
            DisplayBytes("Wrote", bytes)
        End If
    End Sub

    Private Sub UpdateBytesToWriteProperties(bytesToWrite As String)
        If Not _bytesToWrite.Contains(bytesToWrite) Then
            _bytesToWrite.Add(bytesToWrite)
        End If
        SelectedBytesToWrite = bytesToWrite
    End Sub

    Private ReadOnly _bytesToWriteBeforeRead As ObservableCollection(Of String) = New ObservableCollection(Of String)
    Public ReadOnly Property BytesToWriteBeforeRead As IEnumerable(Of String)
        Get
            Return _bytesToWriteBeforeRead
        End Get
    End Property

    Private Function CanI2cWriteThenReadExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub I2cWriteThenReadExecute(obj As Object)
        Dim bytesToWrite As String = _newBytesToWriteBeforeRead.Trim()

        If String.IsNullOrEmpty(bytesToWrite) Then
            AppendInfoText("Please enter bytes to write.")
            Return
        End If

        Dim bytes As Byte() = GetByteArrayToWrite(bytesToWrite)
        If bytes IsNot Nothing Then
            Dim succeeded As Boolean = _selectedModule.WriteThenRead(bytes, CUInt(_i2cNumBytesToReadAfterWriteSelectedIndex + 1))
            If succeeded Then
                UpdateBytesToWriteBeforeReadProperties(bytesToWrite)
                DisplayBytes("Wrote", bytes)
                DisplayBytes("Then read", _selectedModule.BytesRead)
            End If
        End If
    End Sub

    Private Sub UpdateBytesToWriteBeforeReadProperties(bytesToWrite As String)
        If Not _bytesToWriteBeforeRead.Contains(bytesToWrite) Then
            _bytesToWriteBeforeRead.Add(bytesToWrite)
        End If
        SelectedBytesToWriteBeforeRead = bytesToWrite
    End Sub

    Private _selectedBytesToWriteBeforeReadText As String
    Public Property SelectedBytesToWriteBeforeRead As String
        Get
            Return _selectedBytesToWriteBeforeReadText
        End Get
        Set
            If _selectedBytesToWriteBeforeReadText <> Value Then
                _selectedBytesToWriteBeforeReadText = Value
                NotifyPropertyChanged(NameOf(SelectedBytesToWriteBeforeRead))
            End If
        End Set
    End Property

    Private _newBytesToWriteBeforeRead As String
    Public Property NewBytesToWriteBeforeRead As String
        Get
            Return _selectedBytesToWriteBeforeReadText
        End Get

        Set
            _newBytesToWriteBeforeRead = Value
        End Set
    End Property

    Private _i2cNumBytesToReadAfterWriteSelectedIndex As Integer
    Public Property I2cNumBytesToReadAfterWriteSelectedIndex As Integer
        Get
            Return _i2cNumBytesToReadAfterWriteSelectedIndex
        End Get
        Set
            If _i2cNumBytesToReadAfterWriteSelectedIndex <> Value Then
                _i2cNumBytesToReadAfterWriteSelectedIndex = Value
                NotifyPropertyChanged(NameOf(I2cNumBytesToReadAfterWriteSelectedIndex))
            End If
        End Set
    End Property

    Public ReadOnly Property IsI2cModuleAvailable As Boolean
        Get
            Return _moduleLocator.I2cModules.Count > 0
        End Get
    End Property

    Public ReadOnly Property CanUseI2cModule As Boolean
        Get
            Return _selectedModule IsNot Nothing
        End Get
    End Property

    Public ReadOnly Property ModuleDescription As String
        Get
            Return If(_selectedModule IsNot Nothing, _selectedModule.Description, String.Empty)
        End Get
    End Property

    Private Sub ClearInfoTextExecute(obj As Object)
        InfoText = String.Empty
        NotifyPropertyChanged(NameOf(InfoText))
    End Sub

    Private Sub AppendInfoText(text As String)
        If InfoText <> String.Empty Then
            InfoText &= Environment.NewLine
        End If
        InfoText &= "[" & Date.Now.ToLongTimeString() & "] " & text
        NotifyPropertyChanged(NameOf(InfoText))
    End Sub

    Dim _infoText As String = String.Empty
    Public Property InfoText As String
        Get
            Return _infoText
        End Get
        Set(value As String)
            _infoText = value
            NotifyPropertyChanged(NameOf(InfoText))
        End Set
    End Property

    Protected Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub MainViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(_moduleLocator.InfoText) Then
            AppendInfoText(_moduleLocator.InfoText)
        ElseIf e.PropertyName = NameOf(_selectedModule.ModuleInfoText) Then
            AppendInfoText(_selectedModule.ModuleInfoText)
        End If
    End Sub

    Private Sub CloseApplicationExecute(obj As Object)
        System.Windows.Application.Current.MainWindow.Close()
    End Sub
End Class
