﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the Interrupt project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel

Public Class BitPatternCompareViewModel
    Inherits ErrorInfoViewModelBase

    Private _irqModule As IrqModule

    Public Sub UpdateIrqModule(irqModule As IrqModule)
        If _supportsIrqType Then
            RemoveHandler _irqModule.PortCompareStatusChanged, AddressOf PortCompareStatusChanged
            RemoveHandler _irqModule.BitChangeSettingsChanged, AddressOf BitChangeSettingsChanged
            RemoveHandler _irqModule.PropertyChanged, AddressOf BitPatternCompareViewModel_PropertyChanged
        End If

        _irqModule = irqModule
        _supportsIrqType = irqModule IsNot Nothing AndAlso irqModule.SupportsBitPatternCompare
        _enableCompare = irqModule IsNot Nothing AndAlso irqModule.PortBitPatternCompare

        If _supportsIrqType Then
            UpdatePortCompareValueText(irqModule.PortBitPatternValue)
            AddHandler _irqModule.PortCompareStatusChanged, AddressOf PortCompareStatusChanged
            AddHandler _irqModule.BitChangeSettingsChanged, AddressOf BitChangeSettingsChanged
            AddHandler _irqModule.PropertyChanged, AddressOf BitPatternCompareViewModel_PropertyChanged
        Else
            UpdatePortCompareValueText(0)
        End If
        ClearStatusIndicators()
        NotifyPropertyChanged(NameOf(CanUseBitPatternCompare))
        NotifyPropertyChanged(NameOf(EnableCompare))
        NotifyPropertyChanged(NameOf(SupportsIrqType))
    End Sub

    Private Sub BitPatternCompareViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(IrqModule.IsModuleIdle) Then
            UpdateModuleIdleStatus()
        End If
    End Sub

    Private Sub UpdateModuleIdleStatus()
        NotifyPropertyChanged(NameOf(CanUseBitPatternCompare))
    End Sub

    Private _supportsIrqType As Boolean
    Public ReadOnly Property SupportsIrqType As Boolean
        Get
            Return _supportsIrqType
        End Get
    End Property

    Public ReadOnly Property CanUseBitPatternCompare As Boolean
        Get
            Return _supportsIrqType AndAlso _irqModule IsNot Nothing AndAlso _irqModule.IsModuleIdle AndAlso Not _irqModule.BitChangeIsEnabled
        End Get
    End Property

    Private Sub BitChangeSettingsChanged(sender As Object, e As BitChangeSettingsChangedEventArgs)
        NotifyPropertyChanged(NameOf(CanUseBitPatternCompare))
    End Sub

    Public Sub ClearStatusIndicators()
        _statusCompareValueMatched = False
        _lastTimeCompareMatched = String.Empty
        NotifyPropertyChanged(NameOf(StatusCompareValueMatched))
        NotifyPropertyChanged(NameOf(LastTimeCompareMatched))
    End Sub

    Private Sub UpdatePortCompareValueText(compareValue As UInteger)
        _portCompareValueText = AdqCommon.FormatHexValue(compareValue)
        _errorInfo.Remove(NameOf(PortCompareValueText))
        NotifyPropertyChanged(NameOf(PortCompareValueText))
        NotifyPropertyChanged(NameOf(IsInputValid))
    End Sub

    Private _enableCompare As Boolean
    Public Property EnableCompare As Boolean
        Get
            Return _enableCompare
        End Get
        Set
            If _enableCompare <> Value Then
                _enableCompare = Value
                If _irqModule IsNot Nothing Then
                    _irqModule.PortBitPatternCompare = Value
                End If
                NotifyPropertyChanged(NameOf(EnableCompare))
            End If
        End Set
    End Property

    Private _portCompareValueText As String = "0"
    Public Property PortCompareValueText As String
        Get
            Return _portCompareValueText
        End Get
        Set
            If _portCompareValueText <> Value Then
                _portCompareValueText = Value

                Dim compareValue As UInteger
                Dim msg As String = InputValidation.ValidateHexValue(_portCompareValueText, compareValue)

                If Not String.IsNullOrEmpty(msg) Then
                    _errorInfo.AddOrUpdate(NameOf(PortCompareValueText), msg)
                Else
                    msg = _irqModule.ValidatePortMaxValue(compareValue)
                    If Not String.IsNullOrEmpty(msg) Then
                        _errorInfo.AddOrUpdate(NameOf(PortCompareValueText), msg)
                    Else
                        _errorInfo.Remove(NameOf(PortCompareValueText))
                        _irqModule.PortBitPatternValue = compareValue
                        _portCompareValueText = AdqCommon.FormatHexValue(compareValue)
                    End If

                    NotifyPropertyChanged(NameOf(PortCompareValueText))
                    NotifyPropertyChanged(NameOf(IsInputValid))
                End If
            End If
        End Set
    End Property

    Private Sub PortCompareStatusChanged(sender As Object, e As PortCompareStatusChangedEventArgs)
        _statusCompareValueMatched = e.StatusCompare
        _lastTimeCompareMatched = e.TimeCompareMatched.ToString("HH:mm:ss.fff")
        NotifyPropertyChanged(NameOf(StatusCompareValueMatched))
        NotifyPropertyChanged(NameOf(LastTimeCompareMatched))
    End Sub

    Private _statusCompareValueMatched As Boolean
    Public ReadOnly Property StatusCompareValueMatched As Boolean
        Get
            Return _statusCompareValueMatched
        End Get
    End Property

    Private _lastTimeCompareMatched As String
    Public ReadOnly Property LastTimeCompareMatched As String
        Get
            Return _lastTimeCompareMatched
        End Get
    End Property

End Class
