﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the Interrupt project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel

Public Class MainViewModel
    Inherits ErrorInfoViewModelBase

    Private ReadOnly _moduleLocator As ModuleLocator

    Public Property ResetModuleCommand As RelayCommand
    Public Property ClearStatusIndicatorsCommand As RelayCommand
    Public Property CloseCommand As RelayCommand
    Public Property EnableDisableIrqCommand As RelayCommand

    Public Sub New()
        InitCommands()
        _moduleLocator = New ModuleLocator(AddressOf MainViewModel_PropertyChanged)
        _moduleLocator.InitModuleLocator()
        _irqWaitTimeoutText = AdqCommon.DefaultIrqWaitTimeoutMilliseconds.ToString()
        InitViewModels()
    End Sub

    Private Sub InitCommands()
        ResetModuleCommand = New RelayCommand(AddressOf ResetModuleExecute, AddressOf CanResetModuleExecute)
        ClearStatusIndicatorsCommand = New RelayCommand(AddressOf ClearStatusIndicatorsExecute)
        CloseCommand = New RelayCommand(AddressOf CloseApplicationExecute)
        EnableDisableIrqCommand = New RelayCommand(AddressOf EnableDisableIrqExecute, AddressOf CanEnableDisableIrqExecute)
    End Sub

    Private Function CanEnableDisableIrqExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing AndAlso IsInputValid()
    End Function

    Private Sub EnableDisableIrqExecute(obj As Object)
        If _selectedModule.IsModuleIdle Then
            ClearStatusIndicators()
            _selectedModule.EnableInterrupt()
        Else
            _selectedModule.DisableInterrupt()
        End If
    End Sub

    Private Sub ClearStatusIndicators()
        BitChangeViewModel.ClearStatusIndicators()
        PortStatusViewModel.ClearStatusIndicators()
        BitPatternCompareViewModel.ClearStatusIndicators()
        ExternalIrqViewModel.ClearStatusIndicators()
        StatusIrqCount = 0
        StatusTriggerValue = 0
    End Sub

    Private _enableButtonContent As String = "Enable IRQ"
    Public Property EnableButtonContent As String
        Get
            Return _enableButtonContent
        End Get
        Set
            _enableButtonContent = Value
            NotifyPropertyChanged(NameOf(EnableButtonContent))
        End Set
    End Property

    Private Sub UpdateModuleIdleStatus()
        EnableButtonContent = If(_selectedModule.IsModuleIdle, "Enable IRQ", "Disable IRQ")
        NotifyPropertyChanged(NameOf(IsModuleAvailable))
        NotifyPropertyChanged(NameOf(IsModuleSelectable))
    End Sub

    Private Function CanResetModuleExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub ResetModuleExecute(obj As Object)
        Dim success As Boolean = _selectedModule.ResetModule()
        If (success) Then
            AppendInfoText(String.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex))
        End If
    End Sub

    Public ReadOnly Property IrqModules As List(Of IrqModule)
        Get
            Return _moduleLocator.IrqModules
        End Get
    End Property

    Private _selectedModule As IrqModule
    Public Property SelectedModule As IrqModule
        Get
            Return _selectedModule
        End Get
        Set
            If _selectedModule Is Value Then Return
            If _selectedModule IsNot Nothing Then
                RemoveHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
                RemoveHandler _selectedModule.CommonIrqStatusChanged, AddressOf CommonIrqStatusChanged
            End If

            _selectedModule = Value
            If _selectedModule IsNot Nothing Then
                AddHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
                AddHandler _selectedModule.CommonIrqStatusChanged, AddressOf CommonIrqStatusChanged
                _resetIrqCount = _selectedModule.ResetIRQCount
                _irqWaitTimeoutText = _selectedModule.TimeoutMilliseconds.ToString()
                _useCallbackFunction = _selectedModule.UseIrqCallbackFunction
                _useIrqWaitFunction = Not _useCallbackFunction
            Else
                _irqWaitTimeoutText = AdqCommon.DefaultIrqWaitTimeoutMilliseconds.ToString()
            End If
            StatusIrqCount = 0
            StatusTriggerValue = 0

            NotifyPropertyChanged(NameOf(SelectedModule))
            NotifyPropertyChanged(NameOf(IrqDescription))
            NotifyPropertyChanged(NameOf(ResetIrqCount))
            NotifyPropertyChanged(NameOf(IsModuleAvailable))
            NotifyPropertyChanged(NameOf(IrqWaitTimeoutText))
            NotifyPropertyChanged(NameOf(UseCallbackFunction))
            NotifyPropertyChanged(NameOf(UseIrqWaitFunction))
            UpdateViewModels()
            ResetModuleCommand.RaiseCanExecuteChanged()
            EnableDisableIrqCommand.RaiseCanExecuteChanged()
            SelectFirstSupportedIrqTabItem()
        End Set
    End Property

    Private Sub UpdateViewModels()
        BitChangeViewModel.UpdateIrqModule(_selectedModule)
        BitPatternCompareViewModel.UpdateIrqModule(_selectedModule)
        PortStatusViewModel.UpdateIrqModule(_selectedModule)
        ExternalIrqViewModel.UpdateIrqModule(_selectedModule)
    End Sub

    Private Sub SelectFirstSupportedIrqTabItem()
        If _selectedModule IsNot Nothing Then
            If _selectedModule.SupportsBitChange Then
                SelectedTabItemIndex = 0
            ElseIf _selectedModule.SupportsBitPatternCompare Then
                SelectedTabItemIndex = 1
            ElseIf _selectedModule.SupportedPortStatusErrors <> 0 OrElse _selectedModule.SupportedFaultRecovery <> 0 Then
                SelectedTabItemIndex = 2
            Else
                SelectedTabItemIndex = 3
            End If
        Else
            SelectedTabItemIndex = 0
        End If
    End Sub

    Private _selectedTabItemIndex As Integer
    Public Property SelectedTabItemIndex As Integer
        Get
            Return _selectedTabItemIndex
        End Get
        Set(ByVal value As Integer)
            If _selectedTabItemIndex <> value Then
                _selectedTabItemIndex = value
                NotifyPropertyChanged(NameOf(SelectedTabItemIndex))
            End If
        End Set
    End Property

    Public ReadOnly Property IsModuleSelectable As Boolean
        Get
            Return (_moduleLocator.IrqModules.Count > 0 AndAlso _selectedModule Is Nothing) OrElse IsModuleAvailable
        End Get
    End Property

    Public ReadOnly Property IsModuleAvailable As Boolean
        Get
            Return _selectedModule IsNot Nothing AndAlso _selectedModule.IsModuleIdle
        End Get
    End Property

    Public ReadOnly Property IrqDescription As String
        Get
            Return If(_selectedModule IsNot Nothing, _selectedModule.IrqDescription, String.Empty)
        End Get
    End Property

    Private _useCallbackFunction As Boolean = True
    Public Property UseCallbackFunction As Boolean
        Get
            Return _useCallbackFunction
        End Get
        Set
            If _useCallbackFunction <> Value Then
                _useCallbackFunction = Value
                UseIrqWaitFunction = Not Value
                If _selectedModule IsNot Nothing Then
                    _selectedModule.UseIrqCallbackFunction = Value
                End If
                NotifyPropertyChanged(NameOf(UseCallbackFunction))
            End If
        End Set
    End Property

    Private _useIrqWaitFunction As Boolean
    Public Property UseIrqWaitFunction As Boolean
        Get
            Return Not _useCallbackFunction
        End Get
        Set
            If _useIrqWaitFunction <> Value Then
                _useIrqWaitFunction = Value
                UseCallbackFunction = Not Value
                If _selectedModule IsNot Nothing Then
                    _selectedModule.UseIrqCallbackFunction = Not Value
                End If
                NotifyPropertyChanged(NameOf(UseIrqWaitFunction))
            End If
        End Set
    End Property

    Private _resetIrqCount As Boolean
    Public Property ResetIrqCount As Boolean
        Get
            Return _resetIrqCount
        End Get
        Set
            If _resetIrqCount <> Value Then
                _resetIrqCount = Value
                If _selectedModule IsNot Nothing Then
                    _selectedModule.ResetIRQCount = Value
                End If
                NotifyPropertyChanged(NameOf(ResetIrqCount))
            End If
        End Set
    End Property

    Private _irqWaitTimeoutText As String
    Public Property IrqWaitTimeoutText As String
        Get
            Return _irqWaitTimeoutText
        End Get
        Set
            If _selectedModule Is Nothing Then Return
            If _irqWaitTimeoutText <> Value Then
                _irqWaitTimeoutText = Value
                Dim timeoutValue As UInteger
                Dim msg As String = ValidateTimeoutValue(_irqWaitTimeoutText, timeoutValue)
                If Not String.IsNullOrEmpty(msg) Then
                    _errorInfo.AddOrUpdate(NameOf(IrqWaitTimeoutText), msg)
                Else
                    _selectedModule.TimeoutMilliseconds = timeoutValue
                    _errorInfo.Remove(NameOf(IrqWaitTimeoutText))
                End If

                NotifyPropertyChanged(NameOf(IrqWaitTimeoutText))
                EnableDisableIrqCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private Function ValidateTimeoutValue(textValue As String, ByRef timeout As UInteger) As String
        timeout = 0
        Dim msg As String = String.Empty
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        Else
            If Not UInteger.TryParse(textValue, timeout) Then
                msg = "The entered text """ & textValue & """ is not a valid number"
            End If
        End If
        Return msg
    End Function

    Private Sub InitViewModels()
        _bitChangeViewModel = New BitChangeViewModel()
        AddHandler BitChangeViewModel.PropertyChanged, AddressOf MainViewModel_PropertyChanged
        _bitPatternCompareViewModel = New BitPatternCompareViewModel()
        AddHandler BitPatternCompareViewModel.PropertyChanged, AddressOf MainViewModel_PropertyChanged
        _portStatusViewModel = New PortStatusViewModel()
        AddHandler PortStatusViewModel.PropertyChanged, AddressOf MainViewModel_PropertyChanged
        _externalIrqViewModel = New ExternalIrqViewModel()
        AddHandler ExternalIrqViewModel.PropertyChanged, AddressOf MainViewModel_PropertyChanged
    End Sub

    Private _bitChangeViewModel As BitChangeViewModel
    Public ReadOnly Property BitChangeViewModel As BitChangeViewModel
        Get
            Return _bitChangeViewModel
        End Get
    End Property

    Private _bitPatternCompareViewModel As BitPatternCompareViewModel
    Public ReadOnly Property BitPatternCompareViewModel As BitPatternCompareViewModel
        Get
            Return _bitPatternCompareViewModel
        End Get
    End Property

    Private _portStatusViewModel As PortStatusViewModel
    Public ReadOnly Property PortStatusViewModel As PortStatusViewModel
        Get
            Return _portStatusViewModel
        End Get
    End Property

    Private _externalIrqViewModel As ExternalIrqViewModel
    Public ReadOnly Property ExternalIrqViewModel As ExternalIrqViewModel
        Get
            Return _externalIrqViewModel
        End Get
    End Property

    Private Sub CommonIrqStatusChanged(sender As Object, e As CommonIrqStatusChangedEventArgs)
        StatusIrqCount = e.StatusIrqCount
        StatusTriggerValue = e.StatusTriggerValue
        AppendInfoText(e.InfoText)
    End Sub

    Private _statusIrqCount As UInteger
    Public Property StatusIrqCount As UInteger
        Get
            Return _statusIrqCount
        End Get
        Set
            If _statusIrqCount <> Value Then
                _statusIrqCount = Value
                NotifyPropertyChanged(NameOf(StatusIrqCount))
            End If
        End Set
    End Property

    Private _statusTriggerValue As UInteger
    Public Property StatusTriggerValue As UInteger
        Get
            Return _statusTriggerValue
        End Get
        Set
            If _statusTriggerValue <> Value Then
                _statusTriggerValue = Value
                NotifyPropertyChanged(NameOf(StatusTriggerValue))
            End If
        End Set
    End Property

    Dim _infoText As String = String.Empty
    Public Property InfoText As String
        Get
            Return _infoText
        End Get
        Set(value As String)
            _infoText = value
            NotifyPropertyChanged(NameOf(InfoText))
        End Set
    End Property

    Private Sub AppendInfoText(text As String)
        If _infoText <> String.Empty Then
            _infoText += Environment.NewLine
        End If
        _infoText += text
        NotifyPropertyChanged(NameOf(InfoText))
    End Sub

    Private Sub ClearStatusIndicatorsExecute(obj As Object)
        ClearStatusIndicators()
    End Sub

    Private Sub CloseApplicationExecute(obj As Object)
        If _selectedModule IsNot Nothing AndAlso Not _selectedModule.IsModuleIdle Then
            _selectedModule.DisableInterrupt()
        End If

        System.Windows.Application.Current.MainWindow.Close()
    End Sub

    Private Sub MainViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(_selectedModule.ModuleInfoText) Then
            AppendInfoText(_selectedModule.ModuleInfoText)
        ElseIf e.PropertyName = NameOf(_selectedModule.ModuleErrorText) Then
            AppendInfoText(_selectedModule.ModuleErrorText)
        ElseIf e.PropertyName = NameOf(_selectedModule.IsModuleIdle) Then
            UpdateModuleIdleStatus()
        ElseIf e.PropertyName = NameOf(IsInputValid) Then
            EnableDisableIrqCommand.RaiseCanExecuteChanged()
        End If
    End Sub

    Public Overrides Function IsInputValid() As Boolean
        If Not _bitChangeViewModel.IsInputValid() Then
            Return False
        End If

        If Not _bitPatternCompareViewModel.IsInputValid() Then
            Return False
        End If

        If Not _portStatusViewModel.IsInputValid() Then
            Return False
        End If

        If Not _externalIrqViewModel.IsInputValid() Then
            Return False
        End If
        Return _errorInfo.Count = 0
    End Function
End Class

