﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports ReadBuffer.ALLDAQ.Boards

Public Class ModuleLocator
    Private _adqOpenError As UInteger = AdqCommon.NotOpen
    Private ReadOnly _inputModules As ICollection(Of IInputModule)
    Private ReadOnly _moduleTypes As ICollection(Of UInteger)

    Public Event InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(inputModules As ICollection(Of IInputModule), moduleTypes As ICollection(Of UInteger),
                   ByRef moduleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs))
        _inputModules = inputModules
        _moduleTypes = moduleTypes
        AddHandler InfoTextChanged, moduleInfoTextChanged
    End Sub

    Public Sub FindAdqModules()
        _adqOpenError = OpenAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            Dim numberOfBoards As UInteger = GetNumberOfBoards()
            Dim boardIdx As UInteger = 0

            While boardIdx < numberOfBoards
                Dim numberOfModules As UInteger
                Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)
                If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                    boardIdx += 1UI
                    Continue While
                End If

                GetModuleTypes(boardIdx, numberOfModules)
                boardIdx += 1UI
            End While
        End If
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Public Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Private Function GetModuleTypes(boardIndex As UInteger, numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE

        Dim moduleIdx As UInteger = 0
        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If _moduleTypes.Contains(moduleType) Then
                GetModuleInfo(boardIndex, moduleIdx, moduleType)
            End If

            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetModuleInfo(boardIndex As UInteger, moduleIndex As UInteger, moduleType As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE

        If moduleType = AdqConstants.ADQ_MODULE_TYPE_DI Then
            adqError = GetDIModuleInfo(boardIndex, moduleIndex)
        ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            adqError = GetDIOModuleInfo(boardIndex, moduleIndex)
        ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_CNTR Then
            adqError = GetCounterModuleInfo(boardIndex, moduleIndex)
        ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_AI Then
            adqError = GetAIModuleInfo(boardIndex, moduleIndex)
        End If
        Return adqError
    End Function

    Private Function GetDIModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo As AdqDriver.ADQDIModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDiModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim boardName = GetBoardName(boardIndex)
            Dim adqModule = New DigitalInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.PortWidthBits)
            _inputModules.Add(adqModule)
        End If

        Return adqError
    End Function

    Private Function GetDIOModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo As AdqDriver.ADQDIOModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDioModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim boardName = GetBoardName(boardIndex)
            Dim adqModule = New DigitalInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.PortWidthBits)
            _inputModules.Add(adqModule)
        End If
        Return adqError
    End Function

    Private Function GetCounterModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqCntrModuleInfo As AdqDriver.ADQCNTRModuleInfo = New AdqDriver.ADQCNTRModuleInfo(AdqDriver.ADQCNTRModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqCntrModuleInfo.Size))
        Marshal.StructureToPtr(adqCntrModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqCntrModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqCntrModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQCNTRModuleInfo)), AdqDriver.ADQCNTRModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqCntrModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqCntrModuleInfo.Size, infoSize))

        Dim boardName = GetBoardName(boardIndex)
        If adqCntrModuleInfo.CounterType <> AdqConstants.ADQ_CNTR_TYPE_8254 Then
            UpdateInfoText(String.Format("Board {0}, {1}, Module {2} unsupported counter type found.",
                                  boardIndex, boardName, moduleIndex))
            Return AdqErrors.ADQ_ERROR_UNSUPPORTED
        End If

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim adqModule = New CounterModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqCntrModuleInfo)
            _inputModules.Add(adqModule)
        End If
        Return adqError
    End Function

    Private Function GetAIModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqAiModuleInfo As AdqDriver.ADQAIModuleInfo = New AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqAiModuleInfo.Size))
        Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex,
                                              moduleIndex,
                                              adqGeneralModuleInfo,
                                              adqAiModuleInfo.Size,
                                              moduleInfo,
                                              infoSize)
        adqAiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQAIModuleInfo)), AdqDriver.ADQAIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqAiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize))

        Dim boardName = GetBoardName(boardIndex)
        If adqAiModuleInfo.RangeInfoPerChannel Then
            UpdateInfoText(String.Format("Board {0}, {1}, Module {2} has individual ranges." _
                & Environment.NewLine _
                & "Please update to a newer version of this program.",
                boardIndex,
                boardName,
                moduleIndex))
            Return AdqErrors.ADQ_ERROR_UNSUPPORTED
        End If

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim aiModule = New AnalogInputModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqAiModuleInfo)
            _inputModules.Add(aiModule)
        End If

        Return adqError
    End Function

    Private Function IsClockControlledModule(clockType As UInteger) As Boolean
        Return clockType = AdqConstants.ADQ_CLOCK_TYPE_HARDWARE OrElse clockType = AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE
    End Function

    Private Function GetBoardName(boardIndex As UInteger) As String
        Dim s As String = ""
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            s = boardInfo.BoardName
        Else
            SetFunctionErrorInfo("adqGetBoardInfo", adqError)
        End If
        Return s
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("API-function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        OnModuleInfoTextChanged(args)
    End Sub

    Protected Sub OnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent InfoTextChanged(Me, e)
    End Sub
End Class
