﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports WriteBufferDO.ALLDAQ.Boards

Public Class ModuleLocator
    Implements INotifyPropertyChanged

    Private _disposed As Boolean = False
    Private _adqOpenError As UInteger = AdqCommon.NotOpen

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(handler As PropertyChangedEventHandler)
        AddHandler PropertyChanged, handler
    End Sub

    Public Sub InitModuleLocator()
        Try
            _adqOpenError = OpenAdqDriver()
            If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
                _digitalModules = New List(Of DigitalModule)
                Dim moduleTypes As List(Of UInteger) = InitModuleTypesToFind()
                FindAdqModules(moduleTypes)

                If _digitalModules.Count = 0 Then
                    InfoText = "No DI or DIO modules found."
                End If
            End If
        Catch ex As DllNotFoundException
            InfoText = "The device driver is not properly installed." & Environment.NewLine _
                & ex.Message
        End Try
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Private Function InitModuleTypesToFind() As List(Of UInteger)
        Dim moduleTypes = New List(Of UInteger) From {
            AdqConstants.ADQ_MODULE_TYPE_DO,
            AdqConstants.ADQ_MODULE_TYPE_DIO
        }
        Return moduleTypes
    End Function

    Private Sub FindAdqModules(moduleTypes As List(Of UInteger))
        Dim numberOfBoards As UInteger = GetNumberOfBoards()
        Dim boardIndex As UInteger = 0
        While boardIndex < numberOfBoards
            Dim numberOfModules As UInteger
            Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIndex, numberOfModules)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                boardIndex += 1UI
                Continue While
            End If

            GetModuleTypes(boardIndex, numberOfModules, moduleTypes)
            boardIndex += 1UI
        End While
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If _adqOpenError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Private Function GetModuleTypes(boardIndex As UInteger, numberOfModules As UInteger, moduleTypes As List(Of UInteger)) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0
        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If moduleTypes.Contains(moduleType) Then
                GetModuleInfo(boardIndex, moduleIdx, moduleType)
            End If

            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetModuleInfo(boardIndex As UInteger, moduleIdx As UInteger, moduleType As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE

        If moduleType = AdqConstants.ADQ_MODULE_TYPE_DO Then
            adqError = GetDoModuleInfo(boardIndex, moduleIdx)
        ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            adqError = GetDioModuleInfo(boardIndex, moduleIdx)
        End If
        Return adqError
    End Function

    Private Function GetDoModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDoModuleInfo As AdqDriver.ADQDOModuleInfo = New AdqDriver.ADQDOModuleInfo(AdqDriver.ADQDOModuleInfo_ID)

        Dim ModuleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDoModuleInfo.Size))
        Marshal.StructureToPtr(adqDoModuleInfo, ModuleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDoModuleInfo.Size, ModuleInfo, infoSize)
        adqDoModuleInfo = CType(Marshal.PtrToStructure(ModuleInfo, GetType(AdqDriver.ADQDOModuleInfo)), AdqDriver.ADQDOModuleInfo)
        Marshal.FreeHGlobal(ModuleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDoModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDoModuleInfo.Size, infoSize))

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0,
                         String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim boardName = GetBoardName(boardIndex)
            Dim doModule = New DigitalModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDoModuleInfo.PortWidthBits)
            _digitalModules.Add(doModule)
        End If
        Return adqError
    End Function

    Private Function GetDioModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo As AdqDriver.ADQDIOModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)

        Dim ModuleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, ModuleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.Size, ModuleInfo, infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(ModuleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(ModuleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))

        If IsClockControlledModule(adqGeneralModuleInfo.ClockType) Then
            Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0,
                         String.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex))

            Dim boardName = GetBoardName(boardIndex)
            Dim dioModule = New DigitalModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.PortWidthBits)
            _digitalModules.Add(dioModule)
        End If
        Return adqError
    End Function

    Private Function IsClockControlledModule(clockType As UInteger) As Boolean
        Return clockType = AdqConstants.ADQ_CLOCK_TYPE_HARDWARE OrElse clockType = AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE
    End Function

    Private Function GetBoardName(boardIndex As UInteger) As String
        Dim s As String = String.Empty
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            s = boardInfo.BoardName
        End If
        Return s
    End Function

    Dim _digitalModules As List(Of DigitalModule)
    Public ReadOnly Property DigitalModules As List(Of DigitalModule)
        Get
            Return _digitalModules
        End Get
    End Property

    Private Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        InfoText = AdqCommon.GetFunctionErrorText(functionName, adqError)
    End Sub

    Dim _infoText As String = String.Empty
    Public Property InfoText As String
        Get
            Return _infoText
        End Get
        Set(value As String)
            _infoText = value
            NotifyPropertyChanged(NameOf(InfoText))
        End Set
    End Property

    Public Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
    End Sub

    Public Sub Dispose()
        Dispose(True)
    End Sub

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not _disposed Then
            _disposed = True
            CloseAdqDriver()
        End If
    End Sub
End Class
