﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Windows.Threading
Imports WriteBufferDO.ALLDAQ.Boards

Public Class MainViewModel
    Implements INotifyPropertyChanged, IDataErrorInfo

    Private Const _fmtStr As String = "{0:0.###}"
    Private _moduleLocator As ModuleLocator
    Private _dispatcherTimer As DispatcherTimer
    Private _worker As BackgroundWorker
    Private _isAppClosing As Boolean
    Private _errorInfo As Dictionary(Of String, String) = New Dictionary(Of String, String)

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Property ResetModuleCommand As RelayCommand
    Public Property ClearInfoTextCommand As RelayCommand
    Public Property StartStopWriteBufferCommand As RelayCommand
    Public Property CloseCommand As RelayCommand

    Public Sub New()
        LastValueZeroChecked = True
        InitCommands()
        InitValueTypes()
        _moduleLocator = New ModuleLocator(AddressOf PropertyChanged_Handler)
        _moduleLocator.InitModuleLocator()
        RaisePropertyChanged(NameOf(CanSelectModule))
        InitDispatcherTimer()
    End Sub

    Private Sub InitDispatcherTimer()
        _dispatcherTimer = New DispatcherTimer With {
            .Interval = New TimeSpan(0, 0, 0, 0, 500)
        }
        AddHandler _dispatcherTimer.Tick, AddressOf DispatcherTimer_Tick
    End Sub

    Private Sub InitValueTypes()
        _valueGenerators = New List(Of IValueGenerator) From {
            New BitShiftLeftValueGenerator(),
            New BitShiftRightValueGenerator(),
            New CountUpValueGenerator(),
            New RandomValueGenerator()
        }
    End Sub

    Private Sub InitCommands()
        ResetModuleCommand = New RelayCommand(AddressOf ResetModuleExecute, AddressOf CanResetModuleExecute)
        ClearInfoTextCommand = New RelayCommand(AddressOf ClearInfoTextExecute)
        StartStopWriteBufferCommand = New RelayCommand(AddressOf StartStopWriteBufferExecute, AddressOf CanStartStopWriteBufferExecute)
        CloseCommand = New RelayCommand(AddressOf CloseApplicationExecute)
    End Sub

    Dim _valueGenerators As List(Of IValueGenerator)
    Public ReadOnly Property ValueGenerators As List(Of IValueGenerator)
        Get
            Return _valueGenerators
        End Get
    End Property

    Public ReadOnly Property DigitalModules As List(Of DigitalModule)
        Get
            Return _moduleLocator.DigitalModules
        End Get
    End Property

    Dim _selectedValueGenerator As IValueGenerator
    Public Property SelectedGenerator As IValueGenerator
        Get
            Return _selectedValueGenerator
        End Get
        Set
            If _selectedValueGenerator IsNot Value Then
                _selectedValueGenerator = Value
                RaisePropertyChanged(NameOf(SelectedGenerator))
            End If
        End Set
    End Property

    Dim _selectedModule As DigitalModule
    Public Property SelectedModule As DigitalModule
        Get
            Return _selectedModule
        End Get
        Set(ByVal value As DigitalModule)
            If value IsNot _selectedModule Then
                If _selectedModule IsNot Nothing Then
                    RemoveHandler _selectedModule.PropertyChanged, AddressOf PropertyChanged_Handler
                End If
                _selectedModule = value
                IsModuleIdle = _selectedModule IsNot Nothing
                If _selectedModule IsNot Nothing Then
                    AddHandler _selectedModule.PropertyChanged, AddressOf PropertyChanged_Handler
                    RepeatBufferCount = _selectedModule.RepeatBufferCount.ToString()
                    DesiredOutputFrequency = String.Format(_fmtStr, _selectedModule.DesiredOutputFrequency)
                    NumberOfValues = _selectedModule.NumberOfValues.ToString()
                Else
                    RepeatBufferCount = "1"
                    DesiredOutputFrequency = "1000"
                    NumberOfValues = "8"
                End If

                RaisePropertyChanged(NameOf(SelectedModule))
                RaisePropertyChanged(NameOf(ActualOutputFrequency))
                RaisePropertyChanged(NameOf(OutputTimeFormatted))
            End If
        End Set
    End Property

    Private _desiredOutputFrequency As String
    Public Property DesiredOutputFrequency As String
        Get
            Return _desiredOutputFrequency
        End Get
        Set
            If _desiredOutputFrequency <> Value Then
                _desiredOutputFrequency = Value
                If _selectedModule Is Nothing Then Return

                Dim msg As String = String.Empty
                Dim frequency As Double
                msg = ValidateDoubleValue(_desiredOutputFrequency, frequency)
                If String.IsNullOrEmpty(msg) Then
                    msg = _selectedModule.ValidateDesiredOutputFrequency(frequency)
                End If
                UpdateErrorInfo(NameOf(DesiredOutputFrequency), msg)

                If String.IsNullOrEmpty(msg) Then
                    _selectedModule.DesiredOutputFrequency = frequency
                End If

                RaisePropertyChanged(NameOf(DesiredOutputFrequency))
                StartStopWriteBufferCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Public ReadOnly Property ActualOutputFrequency As Double
        Get
            If _selectedModule IsNot Nothing Then
                Return _selectedModule.ActualOutputFrequency
            End If
            Return 0.0
        End Get
    End Property

    Private _numberOfValues As String
    Public Property NumberOfValues As String
        Get
            Return _numberOfValues
        End Get
        Set
            If _numberOfValues <> Value Then
                _numberOfValues = Value
                Dim numValues As UInteger
                Dim msg As String = ValidateUintTextInput(_numberOfValues, numValues, AddressOf _selectedModule.ValidateNumberOfValues)
                UpdateErrorInfo(NameOf(NumberOfValues), msg)

                If String.IsNullOrEmpty(msg) Then
                    _selectedModule.NumberOfValues = numValues
                End If

                RaisePropertyChanged(NameOf(NumberOfValues))
                StartStopWriteBufferCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Public Property LastValueZeroChecked As Boolean

    Private _repeatBufferCount As String
    Public Property RepeatBufferCount As String
        Get
            Return _repeatBufferCount
        End Get
        Set
            If _repeatBufferCount <> Value Then
                _repeatBufferCount = Value
                Dim count As UInteger
                Dim msg As String = ValidateUintTextInput(_repeatBufferCount, count, AddressOf _selectedModule.ValidateRepeatBufferCount)
                UpdateErrorInfo(NameOf(RepeatBufferCount), msg)

                If String.IsNullOrEmpty(msg) Then
                    _selectedModule.RepeatBufferCount = count
                End If

                RaisePropertyChanged(NameOf(RepeatBufferCount))
                StartStopWriteBufferCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private Function ValidateDoubleValue(textValue As String, ByRef doubleValue As Double) As String
        doubleValue = 0.0
        Dim msg As String = String.Empty

        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        ElseIf Not Double.TryParse(textValue, doubleValue) Then
            msg = "The entered text """ + textValue + """ is not a valid number"
        End If
        Return msg
    End Function

    Private Function ValidateUintTextInput(textValue As String, ByRef value As UInteger, moduleValidationMethod As Func(Of UInteger, String)) As String
        value = 0
        Dim msg As String = String.Empty

        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        Else
            If (Not UInteger.TryParse(textValue, value)) Then
                msg = "The entered text """ + textValue + """ is not a valid number"
            Else
                msg = moduleValidationMethod(value)
            End If
        End If
        Return msg
    End Function

    Private Sub UpdateErrorInfo(propertyName As String, msg As String)
        If String.IsNullOrEmpty(msg) Then
            _errorInfo.Remove(propertyName)
        Else
            _errorInfo.AddOrUpdate(propertyName, msg)
        End If
    End Sub

    Public ReadOnly Property OutputTimeFormatted As String
        Get
            If _selectedModule IsNot Nothing Then
                Dim seconds As Double = _selectedModule.GetSecondsToWait()
                Dim txt As String = "Buffer: " & AdqCommon.FormatOutputTime(seconds)
                If _selectedModule.RepeatBufferCount > 1 Then
                    txt &= ", Total: " & AdqCommon.FormatOutputTime(seconds * _selectedModule.RepeatBufferCount)
                End If
                Return txt
            Else
                Return String.Empty
            End If
        End Get
    End Property

    Dim _infoText As String = String.Empty
    Public ReadOnly Property InfoText As String
        Get
            Return _infoText
        End Get
    End Property

    Private Sub AppendInfoText(text As String)
        If Not String.IsNullOrEmpty(_infoText) Then
            _infoText &= Environment.NewLine
        End If
        _infoText &= "[" & Now.ToString("HH:mm:ss.fff") & "] " & text
        RaisePropertyChanged(NameOf(InfoText))
    End Sub

    Private Function CanResetModuleExecute(ByVal param As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub ResetModuleExecute(ByVal param As Object)
        _selectedModule.ResetModule()
    End Sub

    Private Sub CloseApplicationExecute(ByVal param As Object)
        If Not _isModuleIdle Then
            _isAppClosing = True
            _selectedModule.StopOutput()
        End If

        Windows.Application.Current.MainWindow.Close()
    End Sub

    Private Sub ClearInfoTextExecute(ByVal param As Object)
        _infoText = String.Empty
        RaisePropertyChanged(NameOf(InfoText))
    End Sub

    Private Function CanStartStopWriteBufferExecute(ByVal param As Object) As Boolean
        Return _selectedModule IsNot Nothing _
            AndAlso _errorInfo.Count = 0
    End Function

    Private Sub StartStopWriteBufferExecute(ByVal param As Object)

        If _isModuleIdle Then
            Dim isConfigured = _selectedModule.ConfigureModule()
            If isConfigured Then
                _selectedModule.Values = _selectedValueGenerator.GenerateValues(_selectedModule.PortWidthBits, _selectedModule.NumberOfValues, LastValueZeroChecked)
                StartBackgroundWorker()
            End If
        Else
            _selectedModule.StopOutput()
        End If
    End Sub

    Private _isModuleIdle As Boolean = True
    Public Property IsModuleIdle As Boolean
        Get
            Return _isModuleIdle
        End Get
        Set
            If _isModuleIdle <> Value Then
                _isModuleIdle = Value

                ButtonContent = If(_isModuleIdle, "Start", "Stop")
                RaisePropertyChanged(NameOf(IsModuleIdle))
            End If
        End Set
    End Property

    Public ReadOnly Property CanSelectModule As Boolean
        Get
            Return _moduleLocator.DigitalModules.Count > 0 AndAlso _isModuleIdle
        End Get
    End Property

    Private _buttonContent As String = "Start"
    Public Property ButtonContent As String
        Get
            Return _buttonContent
        End Get
        Set
            _buttonContent = Value
            RaisePropertyChanged(NameOf(ButtonContent))
        End Set
    End Property

    Public ReadOnly Property IsInputValid As Boolean
        Get
            Return _errorInfo.Count = 0
        End Get
    End Property

    Default Public ReadOnly Property Item(propertyName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim value As String = String.Empty
            If _errorInfo.TryGetValue(propertyName, value) Then
                Return value
            End If
            Return Nothing
        End Get
    End Property

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return "MainViewModel.Error"
        End Get
    End Property

    Private Sub StartBackgroundWorker()
        IsModuleIdle = False
        AppendInfoStartWriting()
        _worker = New BackgroundWorker With {
            .WorkerReportsProgress = False,
            .WorkerSupportsCancellation = False
        }
        AddHandler _worker.DoWork, AddressOf Worker_DoWork
        AddHandler _worker.RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
        CommandManager.InvalidateRequerySuggested()
        _worker.RunWorkerAsync(_selectedModule)
    End Sub

    Private Sub AppendInfoStartWriting()
        Dim s As String = String.Format("Started writing to Board {0} Module {1}, {2}, values: {3}, repeat: {4}, frequency: {5}",
                                        _selectedModule.BoardIndex, _selectedModule.ModuleIndex,
                                        _selectedValueGenerator.GeneratorName,
                                        _selectedModule.NumberOfValues, _selectedModule.RepeatBufferCount,
                                        AdqCommon.FormatFrequency(_selectedModule.ActualOutputFrequency))
        AppendInfoText(s)
    End Sub

    Private Sub AppendInfoValuesWritten(adqError As UInteger)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            AppendInfoText("Values written to internal data buffer")
        Else
            AppendInfoText(String.Format("Error: Function adqWriteBuffer returned error number: 0x{0:X}", adqError))
        End If
    End Sub

    Private Sub AppendInfoOutputFinished()
        Dim s As String = String.Format("Finished writing values, number of values written: {0}",
                                        _selectedModule.NumberOfValuesWritten)
        AppendInfoText(s)
    End Sub

    Private Sub Worker_DoWork(ByVal sender As Object, ByVal e As DoWorkEventArgs)
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)
        Dim adqModule As DigitalModule = CType(e.Argument, DigitalModule)
        adqModule.WriteBuffer()
        e.Result = adqModule
    End Sub

    Private Sub Worker_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs)
        If _isAppClosing Then Return

        AppendInfoValuesWritten(_selectedModule.AdqError)
        _dispatcherTimer.Start()
    End Sub

    Private Sub DispatcherTimer_Tick(sender As Object, e As EventArgs)
        CheckModuleStatus()
    End Sub

    Private Sub CheckModuleStatus()
        Dim isModuleRunning As Boolean = _selectedModule.GetModuleStatus()
        If Not isModuleRunning Then
            _dispatcherTimer.Stop()
            IsModuleIdle = True
            AppendInfoOutputFinished()
            CommandManager.InvalidateRequerySuggested()
        End If
    End Sub

    Public Sub RaisePropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub PropertyChanged_Handler(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(ActualOutputFrequency) Then
            RaisePropertyChanged(NameOf(ActualOutputFrequency))
            RaisePropertyChanged(NameOf(OutputTimeFormatted))
        ElseIf e.PropertyName = NameOf(NumberOfValues) Then
            RaisePropertyChanged(NameOf(OutputTimeFormatted))
        ElseIf e.PropertyName = NameOf(RepeatBufferCount) Then
            RaisePropertyChanged(NameOf(OutputTimeFormatted))
        ElseIf e.PropertyName = NameOf(ModuleLocator.InfoText) Then
            AppendInfoText(_moduleLocator.InfoText)
        ElseIf e.PropertyName = NameOf(DigitalModule.ModuleInfoText) Then
            AppendInfoText(_selectedModule.ModuleInfoText)
        End If
    End Sub
End Class
